import java.awt.BorderLayout;
import java.lang.Object;
import java.lang.String;
import javax.swing.JPanel;

/**
 * LogicCircuits.java 1.0 03/02/01
 * An abstract class that contains all the details of a particular question circuit
 * <br>
 * Including details of
 * <br>
 * <ul>
 * <li>the diagram itself</li>
 * <li>the TruthTable set up details</li>
 * <li>hint options</li>
 * <li>correct answer - etc</li>
 * </ul>
 * <br>
 * This abstract class also provides a method that allows a random
 * logic gate to be returned.
 * <br><br>
 * Created      : 1/02/2003
 * <br><br>
 * Last Updated : 1/03/2003
 * <br><br>
 * 
 * @author   James McMahon
 */
public abstract class Circuit {
	
	/**
	 * The panel that is used to draw the diagram onto and
	 * also optionally any hints that may be requested
	 */
	private JPanel circuitPanel = new JPanel(new BorderLayout());
	
	/**
	 * Allows access to all of the logic gates that are allowed for a diagram
	 */
	private LogicGates gates = new LogicGates();

	/**
	 * Sets the gates that are available to those provided by logicGates
	 * if you want to modify them in some way
	 * 
	 * @param logicGates The Logic gates that can be used to draw random gate types from
	 */
	public void setGateComponents(LogicGates logicGates) {
		this.gates = logicGates;
	}

	/**
	 * Should return the data that is displayed.
	 * <br>
	 * Namely all possible input values and question marks for anything the user should input themselves.
	 */
	public abstract Object[][] getTruthTableDetails();

	/**
	 * should return the headings for the truth table (such as A, B, C, Output etc)
	 */
	public abstract String[] getTruthTableHeadings();

	/**
	 * Returns the circuit panel without hints.
	 * <br>
	 * Provided as a convenient wrapper
	 * 
	 * @return   unhintedCircuitDiagram The diagram for this question drawn on to a JPanel without hints
	 */
	public JPanel getCircuitDiagram() {
		return getCircuitDiagramPanel(false);
	}

	/**
	 * Returns the circuit panel with a random hint drawn on to it
	 * <br>
	 * provided as a convenient wrapper
	 * 
	 * @return   hintedCircuitDiagram The diagram for this question is returned with hints drawn on to it
	 */
	public JPanel getRandomHintedCircuitDiagram() {
		return getCircuitDiagramPanel(true);
	}

	/**
	 * Should return the filled in circuitPanel with a randomly generated hint draw on top
	 * if hintsOn is true, if this is false then it should only draw the circuit & no hints
	 * <br>
	 * The hint is envisaged to take one possible set of inputs and follow the changes on the
	 * connector lines until the output line if , otherwise it should not draw the hints
	 */
	public abstract JPanel getCircuitDiagramPanel(boolean hintsOn);

	/**
	 * Sshould return the full representation of the truth table for display or checking
	 * <br>
	 * Essentially the same as getTruthTableDetails but without the questionmarks -
	 * instead it has the actual answers
	 */
	public abstract Object[][] getCorrectAnswer();
	
	/**
	 * Needs to provide the number of columns that are non-user modifiable
	 * within the truthtables JTable.
	 */
	public abstract int getNumberOfFixedColumns();
}