import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.lang.Integer;
import java.lang.Object;
import java.lang.String;
import java.util.Random;
import javax.swing.JPanel;

/**
 * LogicCircuits.java 1.0 03/02/01
 * <br>
 * Author       : James McMahon
 * <br>
 * Implements the Circuit for a two gate approach of the following format.
 * <br>
 * <pre>
 * A--N1--|
 *        G1--N4--D--|
 * B--N2--|	         G2---Output
 *                   |
 * C--N3-------------|
 * </pre>
 * <br>
 * where G's are logic gates, N's are inverters and A, B, C are inputs.
 * <br>
 * Created      : 1/02/2003
 * <br>
 * Last Updated : 1/02/2003
 */
public class Circuit2GatesTop extends Circuit {
	
	//the labels for the inputs and outputs
	private String aLabel = "A";
	private String bLabel = "B";
	private String cLabel = "C";
	private String dLabel = "D";

    //inputs to 2 gate
	private String outLabel = "OUTPUT";
	private String[] truthTableHeaders = {"A","B","C","D","Output"};

	//the general case that is returned to a user so that they can fill in the blanks
	//recall that the truth table listener model restricts editing of the first three columns
	//and that only a 0, 1 or ? can be used
	private Object[][] defaultTableDetails = {{"0","0","0","?","?"},
											  {"0","0","1","?","?"},
											  {"0","1","0","?","?"},
											  {"0","1","1","?","?"},
											  {"1","0","0","?","?"},
											  {"1","0","1","?","?"},
											  {"1","1","0","?","?"},
											  {"1","1","1","?","?"}};
	
	//the number of columns for the truthtable that the user shouldn't be able to modify
	//equivalent to the first x columns
	private int numberOfFixedColumns = 3;
	
	//allows access to gates
	private LogicGates logicGates;

	//objects for all of the possible gates that can be held in the circuit
	//namely the two gates plus the 4 possible not gates
	private LogicGate gate1;
	private LogicGate gate2;
	private LogicGate notGate1;
	private LogicGate notGate2;
	private LogicGate notGate3;
	private LogicGate notGate4;

	//booleans to decide if the not gates need to be taken into account
	private boolean notGate1Exists = false;
	private boolean notGate2Exists = false;
	private boolean notGate3Exists = false;
	private boolean notGate4Exists = false;

	/**
	 * Set's up a random circuit that can be queryed using the methods provided
	 * 
	 * @param   logicComponents   The logicGates object that random gates can be drawn from
	 */
	public Circuit2GatesTop(LogicGates logicComponents) {
		logicGates = logicComponents;

		//get two random gates for the circuit structure
		gate1 = logicGates.getRandomLogicGate();
		gate2 = logicGates.getRandomLogicGate();

		//decide randomly on the allocation of not gates

		//random number generator
		Random randNum = new Random();

		if (randNum.nextBoolean()) {
			notGate1 = logicGates.getNotGate();
			notGate1Exists = true;
		} else {
			notGate1Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate2 = logicGates.getNotGate();
			notGate2Exists = true;
		} else {
			notGate2Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate3 = logicGates.getNotGate();
			notGate3Exists = true;
		} else {
			notGate3Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate4 = logicGates.getNotGate();
			notGate4Exists = true;
		} else {
			notGate4Exists = false;
		}
	}

	/**
	 * returns the data that is displayed in the truth table with question marks where user input is expected
	 * 
	 * @return   TruthTableDetails The actual contents of the default table
	 */
	public Object[][] getTruthTableDetails() {
		return defaultTableDetails;
	}

	/**
	 * returns the headings for the truth table (such as A, B, C, Output etc)
	 * 
	 * @return   Headings The headings of the truthtable that can be used to instantiate a truthtable object
	 */
	public String[] getTruthTableHeadings() {
		return truthTableHeaders;
	}

	/**
	 * Returns the number of fixed columns that the user shouldn't be able to modify
	 * 
	 * @return the number of columns to be fixed from the far left
	 */
	public int getNumberOfFixedColumns() {
		return this.numberOfFixedColumns;
	}

	/**
	 * returns the ouput at D given a b and c input conditions
	 * <br>
	 * (in reality with this circuit only b and c actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0  is the output of the D Gate
	 */
	private String getDOutput(int a, int b, int c) {
		if (notGate1Exists) {
			a = notGate1.getOutput(a);//returns the inverison of a's output
		}

		if (notGate2Exists) {
			b = notGate2.getOutput(b);
		}

		int d = gate1.getOutput(a,b);//gets the result from gate 1 with the two (possibly inverted) inputs

		if (notGate4Exists) {//may need to invert d if the inverter exists
			d = notGate4.getOutput(d);
		}

		return Integer.toString(d);
	}

	/**
	 * returns the final output of the circuit given the three initial starting conditions
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0 is the output of the full circuit
	 */
	private String getOutput(int a, int b, int c) {
		String dOut = getDOutput(a,b,c);
		int d = Integer.parseInt(dOut);

		if (notGate3Exists) {
			c = notGate3.getOutput(c);
		}

		int output = gate2.getOutput(d, c);

		return Integer.toString(output);
	}

	/**
	 * should return the full representation of the truth table for display or checking
	 * <br>
	 * Essentially the same as getTruthTableDetails but without the questionmarks -
	 * instead it has the actual answers
	 */
	public Object[][] getCorrectAnswer() {
		Object[][] correctAnswerDetails = {{"0","0","0", getDOutput(0,0,0), getOutput(0,0,0)},
										   {"0","0","1", getDOutput(0,0,1), getOutput(0,0,1)},
							    		   {"0","1","0", getDOutput(0,1,0), getOutput(0,1,0)},
							    		   {"0","1","1", getDOutput(0,1,1), getOutput(0,1,1)},
							    		   {"1","0","0", getDOutput(1,0,0), getOutput(1,0,0)},
							  			   {"1","0","1", getDOutput(1,0,1), getOutput(1,0,1)},
							    		   {"1","1","0", getDOutput(1,1,0), getOutput(1,1,0)},
							    		   {"1","1","1", getDOutput(1,1,1), getOutput(1,1,1)}};
		return correctAnswerDetails;
	}

	/**
	 * Sets up the JPanel that contains the circuit
	 * 
	 * @param    hints   If true then hints are drawn on to the circuit otherwise they aren't
	 * 
	 * @return   CircuitDiagram A diagram drawn on to a JPanel with or without hints based on the hints input
	 */
	public JPanel getCircuitDiagramPanel(boolean hints) {
		
		//these need to be final because we may need to access them inside the paint method so that it a row can be painted
		final Object[][] answers = getCorrectAnswer();
		Random rand = new Random();
		//return a random int range 0 to length-1
		final int hintRow = rand.nextInt(answers.length);
		final boolean hintsOn = hints;

		return new JPanel(){
   			public void paintComponent(Graphics g) {
   	    		super.paintComponent(g); //paint background

    			//set the image width and height
    			int imageWidth  = 75;
    			int imageHeight = 50;

				//all of the gate locations are set up as relative as possible
				//hence they need to be set up in the order in which they appear in the diagram

				//set up the first three not gates
     			int notGate1x = 120;
    			int notGate1y = 20;

    			int notGate2x = notGate1x;
    			int notGate2y = notGate1y+imageHeight+20; //20 is spacing

    			int notGate3x = notGate1x;
    			int notGate3y = notGate1y+notGate2y+imageHeight;

    			//set the root locations of the 1st random gate
    			int gate1x = notGate1x+imageWidth+20;//20 is spacing
    			int gate1y = notGate1y+35;           //25 is half of image height with 15 spacing

    			//set the location of the 4th not gate
    			int notGate4x = gate1x+imageWidth;//20 is spacing
    			int notGate4y = gate1y;              //

    			//set the root location of the second random gate
    			int gate2x = notGate4x+imageWidth;
    			int gate2y = notGate4y+imageHeight+10;//10 is spacing

    			//set the background of the actual panel
    			g.setColor(Color.lightGray);
				g.fillRect(0,0,getWidth(),getHeight());
    			g.setColor(Color.black);
    			g.drawRect(0,0,getWidth()-2,getHeight()-2);

    			//draw the text on to the screen
    			g.drawString(aLabel, notGate1x-15, notGate1y+28);
    			g.drawString(bLabel, notGate2x-15, notGate2y+28);
    			g.drawString(cLabel, notGate3x-15, notGate3y+28);
    			g.drawString(dLabel, notGate4x+imageWidth+8, notGate4y+imageHeight+4);
    			g.drawString(outLabel, gate2x+imageWidth, gate2y+30);//30 is image start position (10) + halfway down the circuit (15)
    																 //with 5 spacing

        		//first paint on the two random gates
        		//get the image of the 1st gate after first painting the background of the image
    			Image image = createImage(imageWidth, imageHeight);
    			Graphics gImg = image.getGraphics();
				gImg.setColor(Color.lightGray);
				gImg.fillRect(0,0,image.getWidth(this),image.getHeight(this));

    			//draw random gate1
    			image = gate1.getImageRepresentation(image, 10, 10, hintsOn);//get the actual image
        		g.drawImage(image, gate1x, gate1y, this);//Draw image at its natural size first (75x50 image);

        		//get the image of the 2nd gate after first painting the background of the image
        		Image image2 = createImage(imageWidth, imageHeight);
    			Graphics gImg2 = image2.getGraphics();
    			gImg2.setColor(Color.lightGray);
				gImg2.fillRect(0,0,image2.getWidth(this),image2.getHeight(this));

    			//draw random gate2
    			image2 = gate2.getImageRepresentation(image2, 10, 10, hintsOn);
        		g.drawImage(image2, gate2x, gate2y, this);


        		//draw all of the connecters in such a way that a line is already drawn underneath any possible inverter gates
        		//draw a connector between notgate1 and random gate 1
        		int x1 = notGate1x+imageWidth;
        		int y1 = notGate1y+10+15;//10 is image start position, 15 is connecter start point
        		int x2 = x1;
        		int y2 = gate1y+10+8;//start position (10) plus first connecter point
        		g.drawLine(x1,y1,x2,y2);//top connecter down line

        		x1 = notGate1x+imageWidth;
        		y1 = gate1y+10+8;//10 is image start position, 8 is first connecter start point
        		x2 = gate1x;
        		y2 = y1;
        		g.drawLine(x1,y1,x2,y2);//top across connecter up line

        		//draw a connecter that will fill any gaps left by a missing not gate
        		x1 = notGate1x;
        		y1 = notGate1y+10+15;//10 is image start position, 15 is connecter start point
        		x2 = notGate1x+imageWidth;
        		y2 = y1;//start position (10) plus second  connecter point
        		g.drawLine(x1,y1,x2,y2);//middle across connecter line


        		//draw a connector between notgate2 and random gate 1
        		x1 = notGate2x+imageWidth;
        		y1 = notGate2y+10+15;//10 is image start position, 15 is connecter start point
        		x2 = x1;
        		y2 = gate1y+10+22;//start position (10) plus second  connecter point
        		g.drawLine(x1,y1,x2,y2);//middle connecter up line

				x1 = notGate2x+imageWidth;
        		y1 = gate1y+10+22;//10 is image start position, 22 is second connecter start point
        		x2 = gate1x;
        		y2 = y1;//start position (10) plus second connecter point
        		g.drawLine(x1,y1,x2,y2);//middle across connecter up line

        		//draw a connecter that will fill any gaps left by a missing not gate
        		x1 = notGate2x;
        		y1 = notGate2y+10+15;//10 is image start position, 15 is connecter start point
        		x2 = notGate2x+imageWidth;
        		y2 = y1;//start position (10) plus second  connecter point
        		g.drawLine(x1,y1,x2,y2);//middle across connecter line

        		//connect bottom not gate position to second random gate
        		x1 = notGate3x;
        		y1 = notGate3y+10+15;//10 is image start position, 15 is connecter start point
        		x2 = gate2x;
        		y2 = y1;//start position (10) plus second  connecter point
        		g.drawLine(x1,y1,x2,y2);//bottom connecter across line

        		x1 = gate2x;
        		y1 = notGate3y+10+15;//10 is image start position, 15 is connecter start point
        		x2 = x1;
        		y2 = gate2y+10+22;//start position (10) plus second connecter point
        		g.drawLine(x1,y1,x2,y2);//bottom connecter up line

        		//connect 4th not gate to the second random gate
        		x1 = notGate4x+imageWidth;
        		y1 = notGate4y+10+15;//10 is image start position, 15 is connecter start point
        		x2 = x1;
        		y2 = gate2y+10+8;//start position (10) plus first connecter point
        		g.drawLine(x1,y1,x2,y2);//bottom connecter across line

        		//draw a connecter that will fill any gaps left by a missing not gate
        		x1 = notGate4x;
        		y1 = notGate4y+10+15;//10 is image start position, 15 is connecter start point
        		x2 = notGate4x+imageWidth;
        		y2 = y1;//start position (10) plus second  connecter point
        		g.drawLine(x1,y1,x2,y2);//middle across connecter line

        		 //next draw the not gates if they have been set for this particular diagram

				if (notGate1Exists) {

        			//get the image of the 1st not gate after first painting the background of the image
        			Image image3 = createImage(imageWidth, imageHeight);
    				Graphics gImg3 = image3.getGraphics();
    				gImg3.setColor(Color.lightGray);
					gImg3.fillRect(0,0,image3.getWidth(this),image3.getHeight(this));

    				//draw not gate	1
    				image3 = notGate1.getImageRepresentation(image3, 10, 10, hintsOn);
        			g.drawImage(image3, notGate1x, notGate1y, this);
        		}

        		if (notGate2Exists) {
        			//get the image of the 2nd not gate after first painting the background of the image
        			Image image4 = createImage(imageWidth, imageHeight);
    				Graphics gImg4 = image4.getGraphics();
    				gImg4.setColor(Color.lightGray);
					gImg4.fillRect(0,0,image4.getWidth(this),image4.getHeight(this));

    				//draw not gate	2
    				image4 = notGate2.getImageRepresentation(image4, 10, 10, hintsOn);
        			g.drawImage(image4, notGate2x, notGate2y, this);
        		}

        		if (notGate3Exists) {
        			//get the image of the 3rd not gate after first painting the background of the image
        			Image image5 = createImage(imageWidth, imageHeight);
    				Graphics gImg5 = image5.getGraphics();
    				gImg5.setColor(Color.lightGray);
					gImg5.fillRect(0,0,image5.getWidth(this),image5.getHeight(this));

    				//draw not gate	3
    				image5 = notGate3.getImageRepresentation(image5, 10, 10, hintsOn);
        			g.drawImage(image5, notGate3x, notGate3y, this);
        		}

        		if (notGate4Exists) {
        			//get the image of the 4th not gate after first painting the background of the image
        			Image image6 = createImage(imageWidth, imageHeight);
    				Graphics gImg6 = image6.getGraphics();
    				gImg6.setColor(Color.lightGray);
					gImg6.fillRect(0, 0, image6.getWidth(this), image6.getHeight(this));

    				//draw not gate	4
    				image6 = notGate4.getImageRepresentation(image6, 10, 10, hintsOn);
        			g.drawImage(image6, notGate4x, notGate4y, this);
        		}

				if (hintsOn) {//then need to draw the hints
							  //paints a row of the correct answers numbers on to the panel

					//known outputs
					g.drawString(answers[hintRow][0].toString(), notGate1x-35, notGate1y+28);
					g.drawString(answers[hintRow][1].toString(), notGate2x-35, notGate2y+28);
					g.drawString(answers[hintRow][2].toString(), notGate3x-35, notGate3y+28);
					g.drawString(answers[hintRow][3].toString(), notGate4x+imageWidth-15, notGate4y+imageHeight+4);
					g.drawString(answers[hintRow][4].toString(), gate2x+imageWidth+65, gate2y+30);

					//after inverters outputs (if necessary)
					if(notGate1Exists) {
						g.drawString(Integer.toString(notGate1.getOutput(Integer.parseInt((answers[hintRow][0].toString())))), notGate1x+imageWidth+5, notGate1y+25);
					}

					if(notGate2Exists) {
						g.drawString(Integer.toString(notGate2.getOutput(Integer.parseInt((answers[hintRow][1].toString())))), notGate2x+imageWidth+5, notGate2y+25);
					}

					if(notGate3Exists) {
						g.drawString(Integer.toString(notGate3.getOutput(Integer.parseInt((answers[hintRow][2].toString())))), notGate3x+imageWidth+5, notGate3y+20);
					}

					if(notGate4Exists) {//the d value in the answer is already displayed so want the value before the not gate
						g.drawString(Integer.toString(notGate4.getOutput(Integer.parseInt((answers[hintRow][3].toString())))), notGate4x-5, notGate4y+20);
					}
				}
			}
		};
	}
}