import java.awt.*;
import java.lang.*;
import java.util.Random;
import javax.swing.JPanel;

/**
 * LogicCircuits.java 1.0 03/02/01
 * <br>
 * Author       : James McMahon
 * <br>
 * Implements the Circuit for a three gate linear approach of the following format.
 * <br>
 * <pre>
 * A--N1--|
 *        |
 *   N2--G1--N5---|
 * B-|            G2--N6--|
 *   N3------|	  |       G3---Output
 *           ----_|_------|
 * C--N4----------|
 * </pre>
 * <br>
 * where G's are logic gates, N's are inverters and A, B, C are inputs.
 * <br>
 * Created      : 1/02/2003
 * <br>
 * Last Updated : 1/02/2003
 */
public class Circuit3LinearGates extends Circuit {
	
	//the labels for the inputs and outputs
	private String aLabel = "A";
	private String bLabel = "B";
	private String cLabel = "C";
	private String dLabel = "D";

	//inputs to 2nd gate
	private String eLabel = "E";

	//inputs to 3rd gate
	private String outLabel = "OUTPUT";
	private String[] truthTableHeaders = {"A","B","C","D","E","Output"};

	//the general case that is returned to a user so that they can fill in the blanks
	//recall that the truth table listener model restricts editing of the first three columns
	//and that only a 0, 1 or ? can be used
	private Object[][] defaultTableDetails = {{"0","0","0","?","?","?"},
											  {"0","0","1","?","?","?"},
											  {"0","1","0","?","?","?"},
											  {"0","1","1","?","?","?"},
											  {"1","0","0","?","?","?"},
											  {"1","0","1","?","?","?"},
											  {"1","1","0","?","?","?"},
											  {"1","1","1","?","?","?"}};

	//the number of columns for the truthtable that the user shouldn't be able to modify
	//equivalent to the first x columns
	private int numberOfFixedColumns = 3;

	//allows access to gates
	private LogicGates logicGates;

	//objects for all of the possible gates that can be held in the circuit
	//namely the three gates plus the 6 possible not gates
	private LogicGate gate1;
	private LogicGate gate2;
	private LogicGate gate3;
	private LogicGate notGate1;
	private LogicGate notGate2;
	private LogicGate notGate3;
	private LogicGate notGate4;
	private LogicGate notGate5;
	private LogicGate notGate6;

	//booleans to decide if the not gates need to be taken into account
	private boolean notGate1Exists = false;
	private boolean notGate2Exists = false;
	private boolean notGate3Exists = false;
	private boolean notGate4Exists = false;
	private boolean notGate5Exists = false;
	private boolean notGate6Exists = false;

	/**
	 * Set's up a random circuit that can be queryed using the methods provided
	 * 
	 * @param   logicComponents   The logicGates object that random gates can be drawn from
	 */
	public Circuit3LinearGates(LogicGates logicComponents) {
		logicGates = logicComponents;

		//get three random gates for the circuit structure
		gate1 = logicGates.getRandomLogicGate();
		gate2 = logicGates.getRandomLogicGate();
		gate3 = logicGates.getRandomLogicGate();

		//decide randomly on the allocation of not gates

		//random number generator
		Random randNum = new Random();

		if (randNum.nextBoolean()) {
			notGate1 = logicGates.getNotGate();
			notGate1Exists = true;
		} else {
			notGate1Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate2 = logicGates.getNotGate();
			notGate2Exists = true;
		} else {
			notGate2Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate3 = logicGates.getNotGate();
			notGate3Exists = true;
		} else {
			notGate3Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate4 = logicGates.getNotGate();
			notGate4Exists = true;
		} else {
			notGate4Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate5 = logicGates.getNotGate();
			notGate5Exists = true;
		} else {
			notGate5Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate6 = logicGates.getNotGate();
			notGate6Exists = true;
		} else {
			notGate6Exists = false;
		}
	}

	/**
	 * should return the full representation of the truth table for display or checking
	 * <br>
	 * Essentially the same as getTruthTableDetails but without the questionmarks -
	 * instead it has the actual answers
	 */
	public Object[][] getCorrectAnswer() {
		Object[][] correctAnswerDetails = {{"0","0","0", getDOutput(0,0,0), getEOutput(0,0,0), getOutput(0,0,0)},
										   {"0","0","1", getDOutput(0,0,1), getEOutput(0,0,1), getOutput(0,0,1)},
							    		   {"0","1","0", getDOutput(0,1,0), getEOutput(0,1,0), getOutput(0,1,0)},
							    		   {"0","1","1", getDOutput(0,1,1), getEOutput(0,1,1), getOutput(0,1,1)},
							    		   {"1","0","0", getDOutput(1,0,0), getEOutput(1,0,0), getOutput(1,0,0)},
							  			   {"1","0","1", getDOutput(1,0,1), getEOutput(1,0,1), getOutput(1,0,1)},
							    		   {"1","1","0", getDOutput(1,1,0), getEOutput(1,1,0), getOutput(1,1,0)},
							    		   {"1","1","1", getDOutput(1,1,1), getEOutput(1,1,1), getOutput(1,1,1)}};
		return correctAnswerDetails;
	}

	/**
	 * returns the ouput at D given a b and c input conditions
	 * <br>
	 * (in reality with this circuit only b and c actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0  is the output of the D Gate
	 */
	private String getDOutput(int a, int b, int c) {
		if (notGate1Exists) {
			a = notGate1.getOutput(a);
		}
		if (notGate2Exists) {
			b = notGate2.getOutput(b);
		}

		int d = gate1.getOutput(a,b);

		if (notGate5Exists) {//then need to invert d as well now
			d = notGate5.getOutput(d);
		}
		return Integer.toString(d);
	}

	/**
	 * returns the ouput at E given a b and c input conditions
	 * <br>
	 * (in reality with this circuit only b and c actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0  is the output of the D Gate
	 */
	private String getEOutput(int a, int b, int c) {
		String dOut = getDOutput(a,b,c);
		int d = Integer.parseInt(dOut);

		if (notGate4Exists) {//invert c
			c = notGate4.getOutput(c);
		}

		int e = gate2.getOutput(d, c);

		if (notGate6Exists) {//need to invert the e output as well
			e = notGate6.getOutput(e);
		}
		return Integer.toString(e);
	}

	/**
	 * returns the final output of the circuit given the three initial starting conditions
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0 is the output of the full circuit
	 */
	private String getOutput(int a, int b, int c) {
		String eOut = getEOutput(a,b,c);
		int e = Integer.parseInt(eOut);

		if (notGate3Exists) {
			b = notGate3.getOutput(b);
		}

		int output = gate3.getOutput(e,b);

		return Integer.toString(output);
	}

	/**
	 * Returns the headings for the truth table (such as A, B, C, Output etc).
	 * 
	 * @return   Headings The headings of the truthtable that can be used to instantiate a truthtable object
	 */
	public String[] getTruthTableHeadings() {
		return truthTableHeaders;
	}

	/**
	 * Returns the data that is displayed in the truth table with question marks where user input is expected.
	 * 
	 * @return   TruthTableDetails The actual contents of the default table
	 */
	public Object[][] getTruthTableDetails() {
		return defaultTableDetails;
	}
	
	/**
	 * Returns the number of fixed columns that the user shouldn't be able to modify
	 * 
	 * @return the number of columns to be fixed from the far left
	 */
	public int getNumberOfFixedColumns() {
		return this.numberOfFixedColumns;
	}
	
	/**
	 * Sets up the JPanel that contains the circuit.
	 * 
	 * @param    hints   If true then hints are drawn on to the circuit otherwise they aren't
	 * 
	 * @return   CircuitDiagram A diagram drawn on to a JPanel with or without hints based on the hints input
	 */
	public JPanel getCircuitDiagramPanel(boolean hints) {
		//these need to be final because we may need to access them inside
		//the paint method so that it a row can be painted
		final Object[][] answers = getCorrectAnswer();
		Random rand = new Random();
		//return a random int range 0 to length-1
		final int hintRow = rand.nextInt(answers.length);
		final boolean hintsOn = hints;

		return new JPanel() {
			public void paintComponent(Graphics g) {
				super.paintComponent(g); //paint background

    			//set the image width and height
    			int imageWidth  = 75;
    			int imageHeight = 50;

				//all of the gate locations are set up as relative as possible
				//hence they need to be set up in the order in which they appear in the diagram

				//set up the first four not gates
     			int notGate1x = 40;
    			int notGate1y = 10;

    			int notGate2x = notGate1x+20;
    			int notGate2y = notGate1y+imageHeight;

    			int notGate3x = notGate2x;
    			int notGate3y = notGate2y+imageHeight;

    			int notGate4x = notGate1x;
    			int notGate4y = notGate3y+imageHeight;

    			//set the root locations of the 1st random gate
    			int gate1x = notGate2x+imageWidth;
    			int gate1y = notGate2y-7;

    			//set the location of the 5th not gate
    			int notGate5x = gate1x+imageWidth;
    			int notGate5y = gate1y;

   				//set the root location of the second random gate
    			int gate2x = notGate5x+imageWidth;
    			int gate2y = notGate5y+7;//7 is spacing to line it up

    			//set the location of the 6th not gate
    			int notGate6x = gate2x+imageWidth;
    			int notGate6y = gate2y;

 				//set the root location of the third random gate
    			int gate3x = notGate6x+imageWidth;
    			int gate3y = notGate6y+imageHeight-7;//10 is spacing

    			//set the background of the actual panel
    			g.setColor(Color.lightGray);
				g.fillRect(0,0,getWidth(),getHeight());
    			g.setColor(Color.black);
    			g.drawRect(0,0,getWidth()-2,getHeight()-2);

				//first paint on the three random gates
        		//get the image of the 1st gate after first painting the background of the image
    			Image image = createImage(imageWidth, imageHeight);
    			Graphics gImg = image.getGraphics();
				gImg.setColor(Color.lightGray);
				gImg.fillRect(0,0,image.getWidth(this),image.getHeight(this));

    			//draw random gate1
    			image = gate1.getImageRepresentation(image, 10, 10, hintsOn);//get the actual image
        		g.drawImage(image, gate1x, gate1y, this);//Draw image at its natural size first (75x50 image);

        		//get the image of the 2nd gate after first painting the background of the image
        		Image image2 = createImage(imageWidth, imageHeight);
    			Graphics gImg2 = image2.getGraphics();
    			gImg2.setColor(Color.lightGray);
				gImg2.fillRect(0,0,image2.getWidth(this),image2.getHeight(this));

    			//draw random gate2
    			image2 = gate2.getImageRepresentation(image2, 10, 10, hintsOn);
        		g.drawImage(image2, gate2x, gate2y, this);

        		//get the image of the 3rd gate after first painting the background of the image
        		Image image3 = createImage(imageWidth, imageHeight);
    			Graphics gImg3 = image3.getGraphics();
    			gImg3.setColor(Color.lightGray);
				gImg3.fillRect(0,0,image3.getWidth(this),image3.getHeight(this));

    			//draw random gate3
    			image3 = gate3.getImageRepresentation(image3, 10, 10, hintsOn);
        		g.drawImage(image3, gate3x, gate3y, this);

				//next draw all of the connecters (with asumption the not gates don't exist

				//draw the connecters between a and gate 1
				int x1 = notGate1x;
				int y1 = notGate1y+25;
				int x2 = gate1x;
				int y2 = notGate1y+25;
				g.drawLine(x1,y1,x2,y2);//draws the line from a to top of the notgate1

				x1 = gate1x;
				y1 = notGate1y+25;
				x2 = gate1x;
				y2 = notGate1y+60;
				g.drawLine(x1,y1,x2,y2);//draws the line from a to the connecter of notgate1

				//draws the connecter from gate1 to gate2 assuming no inverter gate
				x1 = gate1x+imageWidth;
				y1 = gate1y+25;
				x2 = gate2x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//draws the line from a to the connecter of notgate1

				//draw the connecters from b to the split line
				x1 = notGate2x-20;
				y1 = notGate2y+imageWidth-25;
				x2 = notGate2x;
				y2 = notGate2y+imageWidth-25;
				g.drawLine(x1,y1,x2,y2);//draws the line from b to the split point

				//draw the connecters from split line point to the two not gates 2 and 3
				x1 = notGate2x-1;//the minus 1 stops it being overwritten by the notgate
				y1 = notGate2y+25;
				x2 = notGate3x-1;
				y2 = notGate3y+25;
				g.drawLine(x1,y1,x2,y2);//draws the line from the split point to not gates 2 and 3

				//draw the connecters from top split line point to the not gate 2
				x1 = notGate2x-1;
				y1 = notGate2y+25;
				x2 = gate1x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//draws the line from the split point to not gate 2

				x1 = notGate3x;
				y1 = notGate3y+25;
				x2 = gate3x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//draws the bottom line split from b to gate 3

				//draws the line from c to gate 2
				x1 = notGate4x;
				y1 = notGate4y+25;
				x2 = gate2x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//draws the line from c to below gate2

				x1 = gate2x;
				y1 = gate2y+32;
				x2 = x1;
				y2 = notGate4y+25;
				g.drawLine(x1,y1,x2,y2);//draws the line from below gate2 to gate2

				//draws the line from notgate5 (D) to the top of gate 3
				x1 = gate2x+imageWidth;
				y1 = gate2y+25;
				x2 = gate3x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//draws the line from notgate5 to top of gate 3

				x1 = gate3x;
				y1 = gate2y+25;
				x2 = x1;
				y2 = gate3y+10+8;
				g.drawLine(x1,y1,x2,y2);//draws the line from top of gate 3	to connecter of gate 3

				//next draw the not gates if they have been set for this particular diagram
				if (notGate1Exists) {

        			//get the image of the 1st not gate after first painting the background of the image
        			Image notImage1 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg1 = notImage1.getGraphics();
    				gNotImg1.setColor(Color.lightGray);
					gNotImg1.fillRect(0,0,notImage1.getWidth(this),notImage1.getHeight(this));

    				//draw not gate	1
    				notImage1 = notGate1.getImageRepresentation(notImage1, 10, 10, hintsOn);
        			g.drawImage(notImage1, notGate1x, notGate1y, this);
        		}

        		if (notGate2Exists) {
        			//get the image of the 2nd not gate after first painting the background of the image
        			Image notImage2 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg2 = notImage2.getGraphics();
    				gNotImg2.setColor(Color.lightGray);
					gNotImg2.fillRect(0,0,notImage2.getWidth(this),notImage2.getHeight(this));

    				//draw not gate	2
    				notImage2 = notGate2.getImageRepresentation(notImage2, 10, 10, hintsOn);
        			g.drawImage(notImage2, notGate2x, notGate2y, this);
        		}

        		if (notGate3Exists) {
        			//get the image of the 3rd not gate after first painting the background of the image
        			Image notImage3 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg3 = notImage3.getGraphics();
    				gNotImg3.setColor(Color.lightGray);
					gNotImg3.fillRect(0,0,notImage3.getWidth(this),notImage3.getHeight(this));

    				//draw not gate	3
    				notImage3 = notGate3.getImageRepresentation(notImage3, 10, 10, hintsOn);
        			g.drawImage(notImage3, notGate3x, notGate3y, this);
        		}

        		if (notGate4Exists) {
        			//get the image of the 4th not gate after first painting the background of the image
        			Image notImage4 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg4 = notImage4.getGraphics();
    				gNotImg4.setColor(Color.lightGray);
					gNotImg4.fillRect(0, 0, notImage4.getWidth(this), notImage4.getHeight(this));

    				//draw not gate	4
    				notImage4 = notGate4.getImageRepresentation(notImage4, 10, 10, hintsOn);
        			g.drawImage(notImage4, notGate4x, notGate4y, this);
        		}

				if (notGate5Exists) {
        			//get the image of the 5th not gate after first painting the background of the image
        			Image notImage5 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg5 = notImage5.getGraphics();
    				gNotImg5.setColor(Color.lightGray);
					gNotImg5.fillRect(0,0,notImage5.getWidth(this),notImage5.getHeight(this));

    				//draw not gate	5
    				notImage5 = notGate5.getImageRepresentation(notImage5, 10, 10, hintsOn);
        			g.drawImage(notImage5, notGate5x, notGate5y, this);
        		}

        		if (notGate6Exists) {
        			//get the image of the 4th not gate after first painting the background of the image
        			Image notImage6 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg6 = notImage6.getGraphics();
    				gNotImg6.setColor(Color.lightGray);
					gNotImg6.fillRect(0, 0, notImage6.getWidth(this), notImage6.getHeight(this));

    				//draw not gate	6
    				notImage6 = notGate6.getImageRepresentation(notImage6, 10, 10, hintsOn);
        			g.drawImage(notImage6, notGate6x, notGate6y, this);
        		}

        		//the split connecter of the b input
        		g.fillOval(notGate2x-5, notGate2y+47,8,8);

        		//next draw the text on to the circuit
        		g.drawString(aLabel, notGate1x-15, notGate1y+28);
    			g.drawString(bLabel, notGate1x-15, gate2y+55);
    			g.drawString(cLabel, notGate4x-15, notGate4y+28);
    			g.drawString(dLabel, gate2x-15, gate2y+10);
    			g.drawString(eLabel, gate3x+10, gate3y+6);
    			g.drawString(outLabel, gate3x+imageWidth+5, gate3y+30);

				if (hintsOn) {
					//paint the row of the correct answers numbers that was selected previously on to the panel
					//known outputs
					g.drawString(answers[hintRow][0].toString(), notGate1x-25, notGate1y+28);//a
					g.drawString(answers[hintRow][1].toString(), notGate1x-25, gate2y+55);//b
					g.drawString(answers[hintRow][2].toString(), notGate4x-25, notGate4y+28);//c

					g.drawString(answers[hintRow][3].toString(), gate2x-15, gate2y-10);//d
					g.drawString(answers[hintRow][4].toString(), gate3x-10, gate3y+6);//e
					g.drawString(answers[hintRow][5].toString(), gate3x+imageWidth+60, gate3y+30);//output

					//after inverters outputs (if necessary)
					if(notGate1Exists) {
						g.drawString(Integer.toString(notGate1.getOutput(Integer.parseInt((answers[hintRow][0].toString())))), notGate1x+imageWidth-15, notGate1y+20);
					}

					if(notGate2Exists) {
						g.drawString(Integer.toString(notGate2.getOutput(Integer.parseInt((answers[hintRow][1].toString())))), notGate2x+imageWidth-15, notGate2y+40);
					}

					if(notGate3Exists) {
						g.drawString(Integer.toString(notGate3.getOutput(Integer.parseInt((answers[hintRow][1].toString())))), notGate3x+imageWidth-15, notGate3y+20);
					}

					if(notGate4Exists) {//the d value in the answer is already displayed so want the value before the not gate
						g.drawString(Integer.toString(notGate4.getOutput(Integer.parseInt((answers[hintRow][2].toString())))), notGate4x+imageWidth-15, notGate4y+20);
					}

					if(notGate5Exists) {
						g.drawString(Integer.toString(notGate5.getOutput(Integer.parseInt((answers[hintRow][3].toString())))), notGate5x-5, notGate5y+20);
					}

					if(notGate6Exists) {//the d value in the answer is already displayed so want the value before the not gate
						g.drawString(Integer.toString(notGate6.getOutput(Integer.parseInt((answers[hintRow][4].toString())))), notGate6x-5, notGate6y+20);
					}
				}//end hintsOn
			}
		};
	}
}