import java.awt.*;
import java.lang.*;
import java.util.Random;
import javax.swing.JPanel;

/**
 * LogicCircuits.java 1.0 03/02/01
 * <br>
 * Author       : James McMahon
 * <br>
 * Implements the Circuit for a four gate approach of the following format.
 * <br>
 * <pre>
 *    N1---G1-----N6--|
 * A--|    |          G3--N8-|
 *    N2--_|_--|      |      |
 *         |   |      |      |
 * B-------N5  |      |      G4----Output
 *             |      |      |
 *    N3-------G2--N7-|      |
 * C--|                      |
 *    N4---------------------|
 * </pre>
 * <br>
 * where G's are logic gates, N's are inverters and A, B, C are inputs.
 * <br>
 * Created      : 1/02/2003
 * <br>
 * Last Updated : 1/02/2003
 */
public class Circuit4Gates extends Circuit {

	//the labels for the inputs and outputs
	private String aLabel = "A";
	private String bLabel = "B";
	private String cLabel = "C";

 	//inputs to 2nd gate
	private String dLabel = "D";

	//inputs to 2nd gate
	private String eLabel = "E";

	//inputs to 3rd gate
	private String fLabel = "F";

	//inputs to 3rd gate
	private String outLabel = "OUTPUT";
	private String[] truthTableHeaders = {"A","B","C","D","E","F","Output"};

	//the general case that is returned to a user so that they can fill in the blanks
	//recall that the truth table listener model restricts editing of the first three columns
	//and that only a 0, 1 or ? can be used
	private Object[][] defaultTableDetails = {{"0","0","0","?","?","?","?"},
											  {"0","0","1","?","?","?","?"},
											  {"0","1","0","?","?","?","?"},
											  {"0","1","1","?","?","?","?"},
											  {"1","0","0","?","?","?","?"},
											  {"1","0","1","?","?","?","?"},
											  {"1","1","0","?","?","?","?"},
											  {"1","1","1","?","?","?","?"}};

	//the number of columns for the truthtable that the user shouldn't be able to modify
	//equivalent to the first x columns
	private int numberOfFixedColumns = 3;

	//allows access to gates
	private LogicGates logicGates;

	//objects for all of the possible gates that can be held in the circuit
	//namely the three gates plus the 8 possible not gates
	private LogicGate gate1;
	private LogicGate gate2;
	private LogicGate gate3;
	private LogicGate gate4;
	private LogicGate notGate1;
	private LogicGate notGate2;
	private LogicGate notGate3;
	private LogicGate notGate4;
	private LogicGate notGate5;
	private LogicGate notGate6;
	private LogicGate notGate7;
	private LogicGate notGate8;

	//booleans to decide if the not gates need to be taken into account
	private boolean notGate1Exists = false;
	private boolean notGate2Exists = false;
	private boolean notGate3Exists = false;
	private boolean notGate4Exists = false;
	private boolean notGate5Exists = false;
	private boolean notGate6Exists = false;
	private boolean notGate7Exists = false;
	private boolean notGate8Exists = false;

	/**
	 * Set's up a random circuit that can be queryed using the methods provided
	 * 
	 * @param   logicComponents   The logicGates object that random gates can be drawn from
	 */
	public Circuit4Gates(LogicGates logicComponents) {
		logicGates = logicComponents;

		//get two random gates for the circuit structure
		gate1 = logicGates.getRandomLogicGate();
		gate2 = logicGates.getRandomLogicGate();
		gate3 = logicGates.getRandomLogicGate();
		gate4 = logicGates.getRandomLogicGate();

		//decide randomly on the allocation of not gates

		//random number generator
		Random randNum = new Random();

		if (randNum.nextBoolean()) {
			notGate1 = logicGates.getNotGate();
			notGate1Exists = true;
		} else {
			notGate1Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate2 = logicGates.getNotGate();
			notGate2Exists = true;
		} else {
			notGate2Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate3 = logicGates.getNotGate();
			notGate3Exists = true;
		} else {
			notGate3Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate4 = logicGates.getNotGate();
			notGate4Exists = true;
		} else {
 			notGate4Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate5 = logicGates.getNotGate();
			notGate5Exists = true;
		} else {
			notGate5Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate6 = logicGates.getNotGate();
			notGate6Exists = true;
		} else {
			notGate6Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate7 = logicGates.getNotGate();
			notGate7Exists = true;
		} else {
			notGate7Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate8 = logicGates.getNotGate();
			notGate8Exists = true;
		} else {
			notGate8Exists = false;
		}
	}

	/**
	 * should return the full representation of the truth table for display or checking
	 * <br>
	 * Essentially the same as getTruthTableDetails but without the questionmarks -
	 * instead it has the actual answers
	 */
	public Object[][] getCorrectAnswer() {
		Object[][] correctAnswerDetails = {{"0","0","0", getDOutput(0,0,0), getEOutput(0,0,0), getFOutput(0,0,0), getOutput(0,0,0)},
										   {"0","0","1", getDOutput(0,0,1), getEOutput(0,0,1), getFOutput(0,0,1), getOutput(0,0,1)},
							    		   {"0","1","0", getDOutput(0,1,0), getEOutput(0,1,0), getFOutput(0,1,0), getOutput(0,1,0)},
							    		   {"0","1","1", getDOutput(0,1,1), getEOutput(0,1,1), getFOutput(0,1,1), getOutput(0,1,1)},
							    		   {"1","0","0", getDOutput(1,0,0), getEOutput(1,0,0), getFOutput(1,0,0), getOutput(1,0,0)},
							  			   {"1","0","1", getDOutput(1,0,1), getEOutput(1,0,1), getFOutput(1,0,1), getOutput(1,0,1)},
							    		   {"1","1","0", getDOutput(1,1,0), getEOutput(1,1,0), getFOutput(1,1,0), getOutput(1,1,0)},
							    		   {"1","1","1", getDOutput(1,1,1), getEOutput(1,1,1), getFOutput(1,1,1), getOutput(1,1,1)}};
		return correctAnswerDetails;
	}

	/**
	 * returns the ouput at D given a b and c input conditions
	 * <br>
	 * (in reality with this circuit only b and c actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0  is the output of the D Gate
	 */
	private String getDOutput(int a, int b, int c) {
		if (notGate1Exists) {//invert a
			a = notGate1.getOutput(a);
		}

		if (notGate2Exists) {//invert b
			b = notGate2.getOutput(b);
		}

		int d = gate1.getOutput(a,b);

		if (notGate6Exists) {//then need to invert d as well now
			d = notGate6.getOutput(d);
		}

		return Integer.toString(d);
	}

	/**
	 * returns the ouput at E given a b and c input conditions
	 * <br>
	 * (in reality with this circuit only b and c actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0  is the output of the D Gate
	 */
	private String getEOutput(int a, int b, int c) {
		if (notGate3Exists) {//invert a
			a = notGate3.getOutput(a);
		}

		if (notGate4Exists) {//invert c
			c = notGate4.getOutput(c);
		}

		int e = gate2.getOutput(a,c);

		if (notGate7Exists) {//need to invert the e output as well
			e = notGate7.getOutput(e);
		}

		return Integer.toString(e);
	}

	/**
	 * returns the ouput at F given a b and c input conditions
	 * <br>
	 * (in reality with this circuit only b and c actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0  is the output of the D Gate
	 */
	private String getFOutput(int a, int b, int c) {
		String dOut = getDOutput(a,b,c);
		String eOut = getEOutput(a,b,c);

		int d = Integer.parseInt(dOut);
		int e = Integer.parseInt(eOut);

		int f = gate3.getOutput(d, e);

		if (notGate8Exists) {//need to invert the f output
			f = notGate8.getOutput(f);
		}

		return Integer.toString(f);
	}

	/**
	 * Returns the final output of the circuit given the three initial starting conditions.
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0 is the output of the full circuit
	 */
	private String getOutput(int a, int b, int c) {
		String fOut = getFOutput(a,b,c);

		int f = Integer.parseInt(fOut);

		if (notGate5Exists) {
			c = notGate5.getOutput(c);
		}

		int output = gate4.getOutput(f, c);

		return Integer.toString(output);
	}

	/**
	 * Returns the headings for the truth table (such as A, B, C, Output etc).
	 * 
	 * @return   Headings The headings of the truthtable that can be used to instantiate a truthtable object
	 */
	public String[] getTruthTableHeadings() {
		return truthTableHeaders;
	}

	/**
	 * Returns the data that is displayed in the truth table with a question mark where user input is expected.
	 * 
	 * @return   TruthTableDetails The actual contents of the default table
	 */
	public Object[][] getTruthTableDetails() {
		return defaultTableDetails;
	}

	/**
	 * Returns the number of fixed columns that the user shouldn't be able to modify
	 * 
	 * @return the number of columns to be fixed from the far left
	 */
	public int getNumberOfFixedColumns() {
		return this.numberOfFixedColumns;
	}

	/**
	 * Sets up the JPanel that contains the circuit.
	 * 
	 * @param    hints   If true then hints are drawn on to the circuit otherwise they aren't
	 * 
	 * @return   CircuitDiagram A diagram drawn on to a JPanel with or without hints based on the hints input
	 */
	public JPanel getCircuitDiagramPanel(boolean hints) {
		//these need to be final because we may need to access them inside
		//the paint method so that it a row can be painted
		final Object[][] answers = getCorrectAnswer();
		Random rand = new Random();
		//return a random int range 0 to length-1
		final int hintRow = rand.nextInt(answers.length);
		final boolean hintsOn = hints;

		return new JPanel() {
			public void paintComponent(Graphics g) {
				 super.paintComponent(g); //paint background

    			//set the image width and height
    			int imageWidth  = 75;
    			int imageHeight = 50;

				//all of the gate locations are set up as relative as possible
				//hence they need to be set up in the order in which they appear in the diagram

				//set up the first four not gates
     			int notGate1x = 60;
    			int notGate1y = -5;

    			int notGate2x = notGate1x;
    			int notGate2y = notGate1y+imageHeight-5;

    			int notGate3x = notGate1x;
    			int notGate3y = notGate2y+imageHeight-5;

    			int notGate4x = notGate1x;
    			int notGate4y = notGate3y+imageHeight-5;

    			int notGate5x = notGate1x;
    			int notGate5y = notGate4y+imageHeight-5;

    			//set the root locations of the 1st random gate
    			int gate1x = notGate1x+imageWidth;
    			int gate1y = notGate1y+22;

   				//set the root location of the second random gate
    			int gate2x = notGate3x+imageWidth;
    			int gate2y = notGate3y+22;

    			//set the location of the 6th not gate
    			int notGate6x = gate1x+imageWidth;
    			int notGate6y = gate1y;

    			//set the location of the 7th not gate
    			int notGate7x = gate2x+imageWidth;
    			int notGate7y = gate2y;

 				//set the root location of the third random gate
    			int gate3x = notGate6x+imageWidth;
    			int gate3y = (notGate6y+notGate7y)/2;//gives the point halfway between

    			//set the location of the 8th not gate
    			int notGate8x = gate3x+imageWidth;
    			int notGate8y = gate3y;

    			//set the location of the final gate 4
    			int gate4x = notGate8x+imageWidth;
    			int gate4y = (notGate8y+notGate5y)/2 + 15;//gives the point halfway between plus 35 to get it closer to the bottom

    			//set the background of the actual panel
    			g.setColor(Color.lightGray);
				g.fillRect(0,0,getWidth(),getHeight());
    			g.setColor(Color.black);
    			g.drawRect(0,0,getWidth()-2,getHeight()-2);

        		//first paint on the four random gates
        		//get the image of the 1st gate after first painting the background of the image
    			Image image = createImage(imageWidth, imageHeight);
    			Graphics gImg = image.getGraphics();
				gImg.setColor(Color.lightGray);
				gImg.fillRect(0,0,image.getWidth(this),image.getHeight(this));

    			//draw random gate1
    			image = gate1.getImageRepresentation(image, 10, 10, hintsOn);//get the actual image
        		g.drawImage(image, gate1x, gate1y, this);//Draw image at its natural size first (75x50 image);

        		//get the image of the 2nd gate after first painting the background of the image
        		Image image2 = createImage(imageWidth, imageHeight);
    			Graphics gImg2 = image2.getGraphics();
    			gImg2.setColor(Color.lightGray);
				gImg2.fillRect(0,0,image2.getWidth(this),image2.getHeight(this));

    			//draw random gate2
    			image2 = gate2.getImageRepresentation(image2, 10, 10, hintsOn);
        		g.drawImage(image2, gate2x, gate2y, this);

        		//get the image of the 3rd gate after first painting the background of the image
        		Image image3 = createImage(imageWidth, imageHeight);
    			Graphics gImg3 = image3.getGraphics();
    			gImg3.setColor(Color.lightGray);
				gImg3.fillRect(0,0,image3.getWidth(this),image3.getHeight(this));

    			//draw random gate3
    			image3 = gate3.getImageRepresentation(image3, 10, 10, hintsOn);
        		g.drawImage(image3, gate3x, gate3y, this);

		   		//get the image of the 4th gate after first painting the background of the image
        		Image image4 = createImage(imageWidth, imageHeight);
    			Graphics gImg4 = image4.getGraphics();
    			gImg4.setColor(Color.lightGray);
				gImg4.fillRect(0,0,image4.getWidth(this),image4.getHeight(this));

    			//draw random gate3
    			image4 = gate4.getImageRepresentation(image4, 10, 10, hintsOn);
        		g.drawImage(image4, gate4x, gate4y, this);

				//Next draw all of the connecter lines
				//draw the line from a to gate 1 through not gate1
				int x1 = notGate1x-25;
				int y1 = notGate1y+25;
				int x2 = gate1x;
				int y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line

				x1 = gate1x;
				y1 = notGate1y+25;
				x2 = x1;
				y2 = notGate1y+40;
				g.drawLine(x1,y1,x2,y2);//down line

				//draw the line from a to gate 2 through not gate 3
				if (notGate3Exists) {//then follow this sequence
					x1 = notGate1x-11;
					y1 = notGate1y+24;
					x2 = x1;
					y2 = notGate3y+25;
					g.drawLine(x1,y1,x2,y2);//down line from a split point

					x1 = notGate1x-11;
					y1 = notGate3y+25;
					x2 = gate2x;
					y2 = y1;
					g.drawLine(x1,y1,x2,y2);//across line from bottom of a split through gate3's space on to top of gate2

					x1 = gate2x;
					y1 = notGate3y+25;
					x2 = x1;
					y2 = gate2y+18;
					g.drawLine(x1,y1,x2,y2);//down line from top a to gate 2 connecter
				}
				else {//follow this sequence which gives a more logical diagram structure
					x1 = notGate1x-11;
					y1 = notGate1y+24;
					x2 = x1;
					y2 = gate2y+18;
					g.drawLine(x1,y1,x2,y2);//down line from a split point

					x1 = notGate1x-11;
					y1 = gate2y+18;
					x2 = gate2x;
					y2 = y1;
					g.drawLine(x1,y1,x2,y2);//across line from bottom of a split straight on to top of gate2
				}

				//draw the line from b through not gate2 to the bottom of gate1
				x1 = notGate2x-25;
				y1 = notGate2y+25;
				x2 = gate1x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line

				x1 = gate1x;
				y1 = notGate2y+25;
				x2 = x2;
				y2 = gate1y+10+22;
				g.drawLine(x1,y1,x2,y2);//up line

				//draw line from c to the split point
				x1 = notGate4x-20;
				y1 = notGate5y+5;
				x2 = notGate4x-5;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//down line from a split point

				if(notGate4Exists) {//go through this sequence
					//draw line vertically through c's split point
					x1 = notGate4x-1;
					y1 = notGate4y+25;
					x2 = notGate5x-1;
					y2 = notGate5y+25;
					g.drawLine(x1,y1,x2,y2);//up line

					//draw line from the top of c's split through not gate 4 to below gate2
					x1 = notGate4x-1;
					y1 = notGate4y+25;
					x2 = gate2x;
					y2 = y1;
					g.drawLine(x1,y1,x2,y2);//across line to bottom of gate 2

					//draw line from the bottom connecter of gate 2 to the top of c's across split line
					x1 = gate2x;
					y1 = gate2y+10+22;
					x2 = x1;
					y2 = notGate4y+25;
					g.drawLine(x1,y1,x2,y2);//up line
				}
				else {//follow this sequence for a more logical diagram
					//draw line vertically through c's split point
					x1 = gate2x-imageWidth-1;
					y1 = gate2y+10+22;
					x2 = notGate5x-1;
					y2 = notGate5y+25;
					g.drawLine(x1,y1,x2,y2);//up line

					//draw line from the top of c's split through not gate 4 to below gate2
					x1 = notGate4x-1;
					y1 = gate2y+32;
					x2 = gate2x;
					y2 = y1;
					g.drawLine(x1,y1,x2,y2);//across line to bottom of gate 2
				}

				//draw line from the bottom of c's split through not gate 5 to the below gate4
				x1 = notGate5x-1;
				y1 = notGate5y+25;
				x2 = gate4x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line to bottom of gate 4

				x1 = gate4x;
				y1 = gate4y+32;
				x2 = x1;
				y2 = notGate5y+25;
				g.drawLine(x1,y1,x2,y2);//down line from gate5 to c's across line

				//draw line from gate1 through not gate6 to top of gate3
				x1 = gate1x+imageWidth;
				y1 = gate1y+25;
				x2 = gate3x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line to bottom of gate 4

				x1 = notGate6x+imageWidth;
				y1 = notGate6y+25;
				x2 = x1;
				y2 = gate3y+10+8;
				g.drawLine(x1,y1,x2,y2);//down line from end of notgate6 to gate3's top connecter

				//draw line from gate2 through not gate7 to bottom of gate3
				x1 = gate2x+imageWidth;
				y1 = gate2y+25;
				x2 = gate3x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line to bottom of gate 4

				x1 = notGate7x+imageWidth;
				y1 = notGate7y+25;
				x2 = x1;
				y2 = gate3y+32;
				g.drawLine(x1,y1,x2,y2);//down line from end of notgate7 to gate3's bottom connecter

				//draw the line from gate3 through notgate8 to top of gate4
				x1 = gate3x+imageWidth;
				y1 = notGate8y+25;
				x2 = gate4x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line to top of gate 4

				//draw the line from top of notgate8 to top connecter of gate4
				x1 = gate4x;
				y1 = gate4y+18;
				x2 = x1;
				y2 = notGate8y+25;
				g.drawLine(x1,y1,x2,y2);//up line from gate4's top connecter

				//next draw the not gates if they have been set for this particular diagram
				if (notGate1Exists) {

        			//get the image of the 1st not gate after first painting the background of the image
        			Image notImage1 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg1 = notImage1.getGraphics();
    				gNotImg1.setColor(Color.lightGray);
					gNotImg1.fillRect(0,0,notImage1.getWidth(this),notImage1.getHeight(this));

    				//draw not gate	1
    				notImage1 = notGate1.getImageRepresentation(notImage1, 10, 10, hintsOn);
        			g.drawImage(notImage1, notGate1x, notGate1y, this);
        		}

        		if (notGate2Exists) {
        			//get the image of the 2nd not gate after first painting the background of the image
        			Image notImage2 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg2 = notImage2.getGraphics();
    				gNotImg2.setColor(Color.lightGray);
					gNotImg2.fillRect(0,0,notImage2.getWidth(this),notImage2.getHeight(this));

    				//draw not gate	2
    				notImage2 = notGate2.getImageRepresentation(notImage2, 10, 10, hintsOn);
        			g.drawImage(notImage2, notGate2x, notGate2y, this);
        		}

        		if (notGate3Exists) {
        			//get the image of the 3rd not gate after first painting the background of the image
        			Image notImage3 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg3 = notImage3.getGraphics();
    				gNotImg3.setColor(Color.lightGray);
					gNotImg3.fillRect(0,0,notImage3.getWidth(this),notImage3.getHeight(this));

    				//draw not gate	3
    				notImage3 = notGate3.getImageRepresentation(notImage3, 10, 10, hintsOn);
        			g.drawImage(notImage3, notGate3x, notGate3y, this);
        		}

        		if (notGate4Exists) {
        			//get the image of the 4th not gate after first painting the background of the image
        			Image notImage4 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg4 = notImage4.getGraphics();
    				gNotImg4.setColor(Color.lightGray);
					gNotImg4.fillRect(0, 0, notImage4.getWidth(this), notImage4.getHeight(this));

    				//draw not gate	4
    				notImage4 = notGate4.getImageRepresentation(notImage4, 10, 10, hintsOn);
        			g.drawImage(notImage4, notGate4x, notGate4y, this);
        		}

				if (notGate5Exists) {
        			//get the image of the 5th not gate after first painting the background of the image
        			Image notImage5 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg5 = notImage5.getGraphics();
    				gNotImg5.setColor(Color.lightGray);
					gNotImg5.fillRect(0,0,notImage5.getWidth(this),notImage5.getHeight(this));

    				//draw not gate	5
    				notImage5 = notGate5.getImageRepresentation(notImage5, 10, 10, hintsOn);
        			g.drawImage(notImage5, notGate5x, notGate5y, this);
        		}

        		if (notGate6Exists) {
        			//get the image of the 4th not gate after first painting the background of the image
        			Image notImage6 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg6 = notImage6.getGraphics();
    				gNotImg6.setColor(Color.lightGray);
					gNotImg6.fillRect(0, 0, notImage6.getWidth(this), notImage6.getHeight(this));

    				//draw not gate	6
    				notImage6 = notGate6.getImageRepresentation(notImage6, 10, 10, hintsOn);
        			g.drawImage(notImage6, notGate6x, notGate6y, this);
        		}

        		if (notGate7Exists) {
        			//get the image of the 7th not gate after first painting the background of the image
        			Image notImage7 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg7 = notImage7.getGraphics();
    				gNotImg7.setColor(Color.lightGray);
					gNotImg7.fillRect(0, 0, notImage7.getWidth(this), notImage7.getHeight(this));

    				//draw not gate	7
    				notImage7 = notGate7.getImageRepresentation(notImage7, 10, 10, hintsOn);
        			g.drawImage(notImage7, notGate7x, notGate7y, this);
        		}

        		 if (notGate8Exists) {
        			//get the image of the 8th not gate after first painting the background of the image
        			Image notImage8 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg8 = notImage8.getGraphics();
    				gNotImg8.setColor(Color.lightGray);
					gNotImg8.fillRect(0, 0, notImage8.getWidth(this), notImage8.getHeight(this));

    				//draw not gate	8
    				notImage8 = notGate8.getImageRepresentation(notImage8, 10, 10, hintsOn);
        			g.drawImage(notImage8, notGate8x, notGate8y, this);
        		}

        		//the split connecter of the a and c inputs input
        		g.fillOval(notGate1x-15, notGate1y+22,8,8);//a
        		g.fillOval(notGate4x-5, notGate4y+47,8,8);//c

        		//next draw the text on to the circuit
        		g.drawString(aLabel, notGate1x-35, notGate1y+28);
    			g.drawString(bLabel, notGate2x-35, notGate2y+28);
    			g.drawString(cLabel, notGate4x-35, notGate5y+10);
    			g.drawString(dLabel, gate3x+5, gate3y);
    			g.drawString(eLabel, gate3x+5, gate3y+imageHeight+10);
    			g.drawString(fLabel, notGate8x+imageWidth+5, notGate8y+60);
    			g.drawString(outLabel, gate4x+imageWidth+3, gate4y+30);

    			g.setColor(Color.black);//redraw the outline box
    			g.drawRect(0,0,getWidth()-2,getHeight()-2);

    			if (hintsOn) {
        			//paint the preselected row of the correct answers numbers on to the panel

					//known outputs
					g.drawString(answers[hintRow][0].toString(), notGate1x-45, notGate1y+28);//a
					g.drawString(answers[hintRow][1].toString(), notGate2x-45, notGate2y+28);//b
					g.drawString(answers[hintRow][2].toString(), notGate4x-45, notGate5y+10);//c

					g.drawString(answers[hintRow][3].toString(), gate3x-12, gate3y);//d
					g.drawString(answers[hintRow][4].toString(), gate3x-12, gate3y+imageHeight+10);//e
					g.drawString(answers[hintRow][5].toString(), notGate8x+imageWidth-12, notGate8y+60);//f

					g.drawString(answers[hintRow][6].toString(), gate4x+imageWidth+60, gate4y+30);//output

					//after inverters outputs (if necessary)
					//note that the first five are after inverter's and the last three are before inverters
					if(notGate1Exists) {
						g.drawString(Integer.toString(notGate1.getOutput(Integer.parseInt((answers[hintRow][0].toString())))), notGate1x+imageWidth-15, notGate1y+20);
					}

					if(notGate2Exists) {
						g.drawString(Integer.toString(notGate2.getOutput(Integer.parseInt((answers[hintRow][1].toString())))), notGate2x+imageWidth-15, notGate2y+40);
					}

					if(notGate3Exists) {
						g.drawString(Integer.toString(notGate3.getOutput(Integer.parseInt((answers[hintRow][0].toString())))), notGate3x+imageWidth-15, notGate3y+20);
					}

					if(notGate4Exists) {
						g.drawString(Integer.toString(notGate4.getOutput(Integer.parseInt((answers[hintRow][2].toString())))), notGate4x+imageWidth-15, notGate4y+20);
					}

					if(notGate5Exists) {
						g.drawString(Integer.toString(notGate5.getOutput(Integer.parseInt((answers[hintRow][2].toString())))), notGate5x+imageWidth-15, notGate5y+20);
					}

					if(notGate6Exists) {//the d value in the answer is already displayed so want the value before the not gate
						g.drawString(Integer.toString(notGate6.getOutput(Integer.parseInt((answers[hintRow][3].toString())))), notGate6x-5, notGate6y+20);
					}

					if(notGate7Exists) {//the e value in the answer is already displayed so want the value before the not gate
						g.drawString(Integer.toString(notGate7.getOutput(Integer.parseInt((answers[hintRow][4].toString())))), notGate7x-5, notGate7y+20);
					}

					if(notGate8Exists) {//the f value in the answer is already displayed so want the value before the not gate
						g.drawString(Integer.toString(notGate8.getOutput(Integer.parseInt((answers[hintRow][5].toString())))), notGate8x-5, notGate8y+20);
					}
    			}
    		}
    	};
	}
}