import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.lang.Integer;
import java.lang.Object;
import java.lang.String;
import java.util.Random;
import javax.swing.JPanel;

/**
 * LogicCircuits.java 1.0 03/02/01
 * <br>
 * Author       : James McMahon
 * <br>
 * Implements the Circuit for a five gate approach of the following format.
 * <br>
 * <pre>
 * 
 * A---N1---|----------------------------|
 *          |                            |
 *          |---|                        |
 *              G1---N5---E---|          G4---H---|
 * B---N2-------|             |          |        |
 *                            G3--N7--G--|        |
 * C---N3-------|             |                   G5---N8---Output
 *              G2---N6---F---|                   |
 *              |                                 |
 * D---N4-------|-----------------------------I---|
 *
 * </pre>
 * <br>
 * where G's are logic gates, N's are inverters and A, B, C, D are inputs.
 * E, F, G, H and I are intermediate stages
 * <br>
 * Created      : 1/02/2003
 * <br>
 * Last Updated : 1/02/2003
 */
 
public class Circuit5Gates extends Circuit{

	//the labels for the inputs and outputs
	private String aLabel = "A";
	private String bLabel = "B";
	private String cLabel = "C";
    private String dLabel = "D";

    //intermediate inputs
	private String eLabel = "E";
	private String fLabel = "F";
	private String gLabel = "G";
    private String hLabel = "H";
    private String iLabel = "I";
    
	private String outLabel = "OUTPUT";
	
	private String[] truthTableHeaders = {"A","B","C","D","E","F","G","H","I","Output"};

	//the general case that is returned to a user so that they can fill in the blanks
	//recall that the truth table listener model restricts editing of the first three columns
	//and that only a 0, 1 or ? can be used
	private Object[][] defaultTableDetails = {{"0","0","0","0","?","?","?","?","?","?"},
											  {"0","0","0","1","?","?","?","?","?","?"},
											  {"0","0","1","0","?","?","?","?","?","?"},
											  {"0","0","1","1","?","?","?","?","?","?"},
											  {"0","1","0","0","?","?","?","?","?","?"},
											  {"0","1","0","1","?","?","?","?","?","?"},
											  {"0","1","1","0","?","?","?","?","?","?"},
											  {"0","1","1","1","?","?","?","?","?","?"},
											  {"1","0","0","0","?","?","?","?","?","?"},
											  {"1","0","0","1","?","?","?","?","?","?"},
											  {"1","0","1","0","?","?","?","?","?","?"},
											  {"1","0","1","1","?","?","?","?","?","?"},
											  {"1","1","0","0","?","?","?","?","?","?"},
											  {"1","1","0","1","?","?","?","?","?","?"},
											  {"1","1","1","0","?","?","?","?","?","?"},
											  {"1","1","1","1","?","?","?","?","?","?"}};

	//the number of columns for the truthtable that the user shouldn't be able to modify
	//equivalent to the first x columns
	private int numberOfFixedColumns = 4;

	//allows access to gates
	private LogicGates logicGates;

	//objects for all of the possible gates that can be held in the circuit
	//namely the three gates plus the 8 possible not gates
	private LogicGate gate1;
	private LogicGate gate2;
	private LogicGate gate3;
	private LogicGate gate4;
	private LogicGate gate5;
	
	private LogicGate notGate1;
	private LogicGate notGate2;
	private LogicGate notGate3;
	private LogicGate notGate4;
	private LogicGate notGate5;
	private LogicGate notGate6;
	private LogicGate notGate7;
	private LogicGate notGate8;

	//booleans to decide if the not gates need to be taken into account
	private boolean notGate1Exists = false;
	private boolean notGate2Exists = false;
	private boolean notGate3Exists = false;
	private boolean notGate4Exists = false;
	private boolean notGate5Exists = false;
	private boolean notGate6Exists = false;
	private boolean notGate7Exists = false;
	private boolean notGate8Exists = false;

	/**
	 * Set's up a random circuit that can be queryed using the methods provided
	 * 
	 * @param   logicComponents   The logicGates object that random gates can be drawn from
	 */
	public Circuit5Gates(LogicGates logicComponents) {
		logicGates = logicComponents;

		//get five random gates for the circuit structure
		gate1 = logicGates.getRandomLogicGate();
		gate2 = logicGates.getRandomLogicGate();
		gate3 = logicGates.getRandomLogicGate();
		gate4 = logicGates.getRandomLogicGate();
		gate5 = logicGates.getRandomLogicGate();

		//decide randomly on the allocation of not gates

		//random number generator
		Random randNum = new Random();

		if (randNum.nextBoolean()) {
			notGate1 = logicGates.getNotGate();
			notGate1Exists = true;
		} else {
			notGate1Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate2 = logicGates.getNotGate();
			notGate2Exists = true;
		} else {
			notGate2Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate3 = logicGates.getNotGate();
			notGate3Exists = true;
		} else {
			notGate3Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate4 = logicGates.getNotGate();
			notGate4Exists = true;
		} else {
 			notGate4Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate5 = logicGates.getNotGate();
			notGate5Exists = true;
		} else {
			notGate5Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate6 = logicGates.getNotGate();
			notGate6Exists = true;
		} else {
			notGate6Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate7 = logicGates.getNotGate();
			notGate7Exists = true;
		} else {
			notGate7Exists = false;
		}

		if (randNum.nextBoolean()) {
			notGate8 = logicGates.getNotGate();
			notGate8Exists = true;
		} else {
			notGate8Exists = false;
		}
	}

	/**
	 * should return the full representation of the truth table for display or checking
	 * <br>
	 * Essentially the same as getTruthTableDetails but without the questionmarks -
	 * instead it has the actual answers
	 */
	public Object[][] getCorrectAnswer() {
		Object[][] correctAnswerDetails = {{"0","0","0","0", getEOutput(0,0,0,0), getFOutput(0,0,0,0), getGOutput(0,0,0,0), getHOutput(0,0,0,0), getIOutput(0,0,0,0), getOutput(0,0,0,0)},
										   {"0","0","0","1", getEOutput(0,0,0,1), getFOutput(0,0,0,1), getGOutput(0,0,0,1), getHOutput(0,0,0,1), getIOutput(0,0,0,1), getOutput(0,0,0,1)},
							    		   {"0","0","1","0", getEOutput(0,0,1,0), getFOutput(0,0,1,0), getGOutput(0,0,1,0), getHOutput(0,0,1,0), getIOutput(0,0,1,0), getOutput(0,0,1,0)},
							    		   {"0","0","1","1", getEOutput(0,0,1,1), getFOutput(0,0,1,1), getGOutput(0,0,1,1), getHOutput(0,0,1,1), getIOutput(0,0,1,1), getOutput(0,0,1,1)},
							    		   {"0","1","0","0", getEOutput(0,1,0,0), getFOutput(0,1,0,0), getGOutput(0,1,0,0), getHOutput(0,1,0,0), getIOutput(0,1,0,0), getOutput(0,1,0,0)},
							  			   {"0","1","0","1", getEOutput(0,1,0,1), getFOutput(0,1,0,1), getGOutput(0,1,0,1), getHOutput(0,1,0,1), getIOutput(0,1,0,1), getOutput(0,1,0,1)},
							    		   {"0","1","1","0", getEOutput(0,1,1,0), getFOutput(0,1,1,0), getGOutput(0,1,1,0), getHOutput(0,1,1,0), getIOutput(0,1,1,0), getOutput(0,1,1,0)},
							    		   {"0","1","1","1", getEOutput(0,1,1,1), getFOutput(0,1,1,1), getGOutput(0,1,1,1), getHOutput(0,1,1,1), getIOutput(0,1,1,1), getOutput(0,1,1,1)},
							    		   {"1","0","0","0", getEOutput(1,0,0,0), getFOutput(1,0,0,0), getGOutput(1,0,0,0), getHOutput(1,0,0,0), getIOutput(1,0,0,0), getOutput(1,0,0,0)},
										   {"1","0","0","1", getEOutput(1,0,0,1), getFOutput(1,0,0,1), getGOutput(1,0,0,1), getHOutput(1,0,0,1), getIOutput(1,0,0,1), getOutput(1,0,0,1)},
							    		   {"1","0","1","0", getEOutput(1,0,1,0), getFOutput(1,0,1,0), getGOutput(1,0,1,0), getHOutput(1,0,1,0), getIOutput(1,0,1,0), getOutput(1,0,1,0)},
							    		   {"1","0","1","1", getEOutput(1,0,1,1), getFOutput(1,0,1,1), getGOutput(1,0,1,1), getHOutput(1,0,1,1), getIOutput(1,0,1,1), getOutput(1,0,1,1)},
							    		   {"1","1","0","0", getEOutput(1,1,0,0), getFOutput(1,1,0,0), getGOutput(1,1,0,0), getHOutput(1,1,0,0), getIOutput(1,1,0,0), getOutput(1,1,0,0)},
							  			   {"1","1","0","1", getEOutput(1,1,0,1), getFOutput(1,1,0,1), getGOutput(1,1,0,1), getHOutput(1,1,0,1), getIOutput(1,1,0,1), getOutput(1,1,0,1)},
							    		   {"1","1","1","0", getEOutput(1,1,1,0), getFOutput(1,1,1,0), getGOutput(1,1,1,0), getHOutput(1,1,1,0), getIOutput(1,1,1,0), getOutput(1,1,1,0)},
							    		   {"1","1","1","1", getEOutput(1,1,1,1), getFOutput(1,1,1,1), getGOutput(1,1,1,1), getHOutput(1,1,1,1), getIOutput(1,1,1,1), getOutput(1,1,1,1)}};
		return correctAnswerDetails;
	}

	/**
	 * returns the ouput at E given a b c and d input conditions
	 * <br>
	 * (in reality with this circuit only a and b actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * @param    d   The input at point d
	 * 
	 * @return   0 If 0  is the output at the E location
	 */
	private String getEOutput(int a, int b, int c, int d) {
		
		if (notGate1Exists) {//invert a
			a = notGate1.getOutput(a);
		}

		if (notGate2Exists) {//invert b
			b = notGate2.getOutput(b);
		}

		int temp = gate1.getOutput(a,b);

		if (notGate5Exists) {//then need to invert temp as well now
			temp = notGate5.getOutput(temp);
		}
		
		return Integer.toString(temp);
	}

	/**
	 * returns the ouput at F given a b c and d input conditions
	 * <br>
	 * (in reality with this circuit only c and d actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * @param    d   The input at point d
	 * 
	 * @return   0 If 0  is the output at the F location
	 */
	private String getFOutput(int a, int b, int c, int d) {
		
		if (notGate3Exists) {//invert c
			c = notGate3.getOutput(c);
		}

		if (notGate4Exists) {//invert d
			d = notGate4.getOutput(d);
		}

		int temp = gate2.getOutput(c,d);

		if (notGate6Exists) {//need to invert the gate2 output as well
			temp = notGate6.getOutput(temp);
		}
		
		return Integer.toString(temp);
	}

	/**
	 * returns the ouput at G given a b c and d input conditions
	 * <br>
	 * (in reality with this circuit only a, b and c actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * @param    d   The input at point d
	 * 
	 * @return   0 If 0  is the output at the G location
	 */
	private String getGOutput(int a, int b, int c, int d) {
		
		String eOut = getEOutput(a,b,c,d);
		String fOut = getFOutput(a,b,c,d);

		int e = Integer.parseInt(eOut);
		int f = Integer.parseInt(fOut);

		int temp = gate3.getOutput(e,f);

		if (notGate7Exists) {//need to invert the f output
			temp = notGate7.getOutput(temp);
		}
		
		return Integer.toString(temp);
	}

	/**
	 * returns the ouput at H given a b c and d input conditions
	 * <br>
	 * (in reality with this circuit only a, b and c actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * @param    d   The input at point d
	 * 
	 * @return   0 If 0  is the output at the H location
	 */
	private String getHOutput(int a, int b, int c, int d) {
		
		String gOut = getGOutput(a,b,c,d);

		int g = Integer.parseInt(gOut);
		
		if (notGate1Exists) {//need to invert the a input
			a = notGate1.getOutput(a);
		}

		int temp = gate4.getOutput(a, g);
		
		return Integer.toString(temp);
	}	
	/**
	 * returns the ouput at i given a b c and d input conditions
	 * <br>
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * @param    d   The input at point d
	 * 
	 * @return   0 If 0  is the output at the H location
	 */
	private String getIOutput(int a, int b, int c, int d) {
		
		if (notGate4Exists) {//need to invert the d input
			d = notGate4.getOutput(d);
		}
		
		return Integer.toString(d);
	}
	
	/**
	 * Returns the final output of the circuit given the four initial starting conditions.
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * @param    d   The input at point d
	 * 
	 * @return   0 If 0 is the output of the full circuit
	 */
	private String getOutput(int a, int b, int c, int d) {
		
		String hOut = getHOutput(a,b,c,d);
		String iOut = getIOutput(a,b,c,d);

		int h = Integer.parseInt(hOut);
		int i = Integer.parseInt(iOut);

		int temp = gate5.getOutput(h, i);
		
		if (notGate8Exists) {
			temp = notGate8.getOutput(temp);
		}

		return Integer.toString(temp);
	}

	/**
	 * Returns the headings for the truth table (such as A, B, C, Output etc).
	 * 
	 * @return   Headings The headings of the truthtable that can be used to instantiate a truthtable object
	 */
	public String[] getTruthTableHeadings() {
		return truthTableHeaders;
	}

	/**
	 * Returns the data that is displayed in the truth table with a question mark where user input is expected.
	 * 
	 * @return   TruthTableDetails The actual contents of the default table
	 */
	public Object[][] getTruthTableDetails() {
		return defaultTableDetails;
	}

	/**
	 * Returns the number of fixed columns that the user shouldn't be able to modify
	 * 
	 * @return the number of columns to be fixed from the far left
	 */
	public int getNumberOfFixedColumns() {
		return this.numberOfFixedColumns;
	}

	/**
	 * Sets up the JPanel that contains the circuit.
	 * 
	 * @param    hints   If true then hints are drawn on to the circuit otherwise they aren't
	 * 
	 * @return   CircuitDiagram A diagram drawn on to a JPanel with or without hints based on the hints input
	 */
	public JPanel getCircuitDiagramPanel(boolean hints) {
		
		//these need to be final because we may need to access them inside
		//the paint method so that it a row can be painted
		final Object[][] answers = getCorrectAnswer();
		Random rand = new Random();
		//return a random int range 0 to length-1
		final int hintRow = rand.nextInt(answers.length);
		final boolean hintsOn = hints;

		return new JPanel() {
			public void paintComponent(Graphics g) {
				 super.paintComponent(g); //paint background

    			//set the image width and height
    			int imageWidth  = 75;
    			int imageHeight = 50;

				//all of the gate locations are set up as relative as possible
				//hence they need to be set up in the order in which they appear in the diagram

				//set up the first four not gates
     			int notGate1x = 60;
    			int notGate1y =  5;

    			int notGate2x = notGate1x;
    			int notGate2y = notGate1y+imageHeight+5;

    			int notGate3x = notGate1x;
    			int notGate3y = notGate2y+imageHeight+5;

    			int notGate4x = notGate1x;
    			int notGate4y = notGate3y+imageHeight+5;

    			//set the root locations of the 1st random gate
    			int gate1x = notGate2x+imageWidth+15;
    			int gate1y = notGate2y-7;

   				//set the root location of the second random gate
    			int gate2x = notGate3x+imageWidth+15;
    			int gate2y = notGate3y+7;
    			
    			//set the root location of the fifth and sixth not gates
    			int notGate5x = gate1x+imageWidth;
    			int notGate5y = gate1y;

    			int notGate6x = gate2x+imageWidth;
    			int notGate6y = gate2y;

 				//set the root location of the third random gate
    			int gate3x = notGate5x+imageWidth;
    			int gate3y = (notGate5y+notGate6y)/2;//gives the point halfway between
    			
    			//set the location of the 7th not gate
    			int notGate7x = gate3x+imageWidth;
    			int notGate7y = gate3y;
    			
    			//set the location of the fourth random gate
    			int gate4x = notGate7x+imageWidth;
    			int gate4y = (notGate1y+notGate7y)/2;

    			//set the location of the fifth random gate
    			int gate5x = gate4x+imageWidth;
    			int gate5y = (gate4y+notGate4y)/2;

    			//set the location of the 8th not gate
    			int notGate8x = gate5x+imageWidth;
    			int notGate8y = gate5y;

    			//set the background of the actual panel
    			g.setColor(Color.lightGray);
				g.fillRect(0,0,getWidth(),getHeight());
    			g.setColor(Color.black);
    			g.drawRect(0,0,getWidth()-2,getHeight()-2);
    			
        		//first paint on the four random gates
        		//get the image of the 1st gate after first painting the background of the image
    			Image image = createImage(imageWidth, imageHeight);
    			Graphics gImg = image.getGraphics();
				gImg.setColor(Color.lightGray);
				gImg.fillRect(0,0,image.getWidth(this),image.getHeight(this));

    			//draw random gate1
    			image = gate1.getImageRepresentation(image, 10, 10, hintsOn);//get the actual image
        		g.drawImage(image, gate1x, gate1y, this);//Draw image at its natural size first (75x50 image);

        		//get the image of the 2nd gate after first painting the background of the image
        		Image image2 = createImage(imageWidth, imageHeight);
    			Graphics gImg2 = image2.getGraphics();
    			gImg2.setColor(Color.lightGray);
				gImg2.fillRect(0,0,image2.getWidth(this),image2.getHeight(this));

    			//draw random gate2
    			image2 = gate2.getImageRepresentation(image2, 10, 10, hintsOn);
        		g.drawImage(image2, gate2x, gate2y, this);

        		//get the image of the 3rd gate after first painting the background of the image
        		Image image3 = createImage(imageWidth, imageHeight);
    			Graphics gImg3 = image3.getGraphics();
    			gImg3.setColor(Color.lightGray);
				gImg3.fillRect(0,0,image3.getWidth(this),image3.getHeight(this));

    			//draw random gate3
    			image3 = gate3.getImageRepresentation(image3, 10, 10, hintsOn);
        		g.drawImage(image3, gate3x, gate3y, this);

		   		//get the image of the 4th gate after first painting the background of the image
        		Image image4 = createImage(imageWidth, imageHeight);
    			Graphics gImg4 = image4.getGraphics();
    			gImg4.setColor(Color.lightGray);
				gImg4.fillRect(0,0,image4.getWidth(this),image4.getHeight(this));

    			//draw random gate4
    			image4 = gate4.getImageRepresentation(image4, 10, 10, hintsOn);
        		g.drawImage(image4, gate4x, gate4y, this);
        		
 		   		//get the image of the 5th gate after first painting the background of the image
        		Image image5 = createImage(imageWidth, imageHeight);
    			Graphics gImg5 = image5.getGraphics();
    			gImg5.setColor(Color.lightGray);
				gImg5.fillRect(0,0,image5.getWidth(this),image5.getHeight(this));

    			//draw random gate5
    			image5 = gate5.getImageRepresentation(image5, 10, 10, hintsOn);
        		g.drawImage(image5, gate5x, gate5y, this);       		

				//Next draw all of the connecter lines
				//draw the line from a to gate 1 through not gate1
				int x1 = notGate1x-15;
				int y1 = notGate1y+25;
				int x2 = gate4x;
				int y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line to above gate4

				x1 = gate1x;
				y1 = notGate1y+25;
				x2 = x1;
				y2 = gate1y+18;
				g.drawLine(x1,y1,x2,y2);//down line from a's line to gate1 input 1
				
				x1 = gate4x;
				y1 = notGate1y+25;
				x2 = x1;
				y2 = gate4y+18;
				g.drawLine(x1,y1,x2,y2);//down line from a's line to gate4 input 1
				
				//draw line from input b
				x1 = notGate2x-15;
				y1 = notGate2y+25;
				x2 = gate1x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line from b through notgate2 into gate1's 2nd input
				
				//draw line from gate1 through notgate5 to top of gate3
				x1 = gate1x+imageWidth;
				y1 = gate1y+25;
				x2 = gate3x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line from gate1 to top gate3
				
				x1 = notGate5x+imageWidth;
				y1 = notGate5y+25;
				x2 = x1;
				y2 = gate3y+18;
				g.drawLine(x1,y1,x2,y2);//down line from gate1 to gate3 input
					
				//draw the line from c to gate 2 through notgate3
				x1 = notGate3x-15;
				y1 = notGate3y+25;
				x2 = gate2x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line through notgate3 to gate2	
				
				//draw line from gate2 through notgate6 to bottom of gate3
				x1 = gate2x+imageWidth;
				y1 = gate2y+25;
				x2 = gate3x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line from gate2 to bottom gate3
				
				x1 = gate3x;
				y1 = gate2y+25;
				x2 = x1;
				y2 = gate3y+32;
				g.drawLine(x1,y1,x2,y2);//up line from bottom gate3 to 2nd input gate3
				
				//draw line from gate3 through notgate7 to bottom of gate4
				x1 = gate3x+imageWidth;
				y1 = gate3y+25;
				x2 = gate4x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line from gate3 to bottom gate4 through notgate7				

				x1 = gate4x;
				y1 = gate3y+25;
				x2 = x1;
				y2 = gate4y+32;
				g.drawLine(x1,y1,x2,y2);//up line from gate3 to 2nd input gate4 				

				//down ine from gate4 to gate5
				x1 = gate4x+imageWidth;
				y1 = gate4y+25;
				x2 = x1;
				y2 = gate5y+18;
				g.drawLine(x1,y1,x2,y2);//down line from gate4 to 1st input gate5 				
		
				//draw the line from d to gate 5 through not gate4
				x1 = notGate4x-15;
				y1 = notGate4y+25;
				x2 = gate5x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line to below gate5
				
				x1 = gate2x;
				y1 = notGate4y+25;
				x2 = x1;
				y2 = gate2y+33;
				g.drawLine(x1,y1,x2,y2);//up line from below gate2 to 2nd input gate2 
				
				x1 = gate5x;
				y1 = notGate4y+25;
				x2 = x1;
				y2 = gate5y+33;
				g.drawLine(x1,y1,x2,y2);//up line from below gate5 to 2nd input gate5 
				
				//draw line from gate5 through notgate8 to output
				x1 = gate5x+imageWidth;
				y1 = gate5y+25;
				x2 = notGate8x+imageWidth;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//down line from gate4 to 1st input gate5 

				//next draw the not gates if they have been set for this particular diagram
				if (notGate1Exists) {

        			//get the image of the 1st not gate after first painting the background of the image
        			Image notImage1 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg1 = notImage1.getGraphics();
    				gNotImg1.setColor(Color.lightGray);
					gNotImg1.fillRect(0,0,notImage1.getWidth(this),notImage1.getHeight(this));

    				//draw not gate	1
    				notImage1 = notGate1.getImageRepresentation(notImage1, 10, 10, hintsOn);
        			g.drawImage(notImage1, notGate1x, notGate1y, this);
        		}

        		if (notGate2Exists) {
        			//get the image of the 2nd not gate after first painting the background of the image
        			Image notImage2 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg2 = notImage2.getGraphics();
    				gNotImg2.setColor(Color.lightGray);
					gNotImg2.fillRect(0,0,notImage2.getWidth(this),notImage2.getHeight(this));

    				//draw not gate	2
    				notImage2 = notGate2.getImageRepresentation(notImage2, 10, 10, hintsOn);
        			g.drawImage(notImage2, notGate2x, notGate2y, this);
        		}

        		if (notGate3Exists) {
        			//get the image of the 3rd not gate after first painting the background of the image
        			Image notImage3 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg3 = notImage3.getGraphics();
    				gNotImg3.setColor(Color.lightGray);
					gNotImg3.fillRect(0,0,notImage3.getWidth(this),notImage3.getHeight(this));

    				//draw not gate	3
    				notImage3 = notGate3.getImageRepresentation(notImage3, 10, 10, hintsOn);
        			g.drawImage(notImage3, notGate3x, notGate3y, this);
        		}

        		if (notGate4Exists) {
        			//get the image of the 4th not gate after first painting the background of the image
        			Image notImage4 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg4 = notImage4.getGraphics();
    				gNotImg4.setColor(Color.lightGray);
					gNotImg4.fillRect(0, 0, notImage4.getWidth(this), notImage4.getHeight(this));

    				//draw not gate	4
    				notImage4 = notGate4.getImageRepresentation(notImage4, 10, 10, hintsOn);
        			g.drawImage(notImage4, notGate4x, notGate4y, this);
        		}

				if (notGate5Exists) {
        			//get the image of the 5th not gate after first painting the background of the image
        			Image notImage5 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg5 = notImage5.getGraphics();
    				gNotImg5.setColor(Color.lightGray);
					gNotImg5.fillRect(0,0,notImage5.getWidth(this),notImage5.getHeight(this));

    				//draw not gate	5
    				notImage5 = notGate5.getImageRepresentation(notImage5, 10, 10, hintsOn);
        			g.drawImage(notImage5, notGate5x, notGate5y, this);
        		}

        		if (notGate6Exists) {
        			//get the image of the 4th not gate after first painting the background of the image
        			Image notImage6 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg6 = notImage6.getGraphics();
    				gNotImg6.setColor(Color.lightGray);
					gNotImg6.fillRect(0, 0, notImage6.getWidth(this), notImage6.getHeight(this));

    				//draw not gate	6
    				notImage6 = notGate6.getImageRepresentation(notImage6, 10, 10, hintsOn);
        			g.drawImage(notImage6, notGate6x, notGate6y, this);
        		}

        		if (notGate7Exists) {
        			//get the image of the 7th not gate after first painting the background of the image
        			Image notImage7 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg7 = notImage7.getGraphics();
    				gNotImg7.setColor(Color.lightGray);
					gNotImg7.fillRect(0, 0, notImage7.getWidth(this), notImage7.getHeight(this));

    				//draw not gate	7
    				notImage7 = notGate7.getImageRepresentation(notImage7, 10, 10, hintsOn);
        			g.drawImage(notImage7, notGate7x, notGate7y, this);
        		}

        		 if (notGate8Exists) {
        			//get the image of the 8th not gate after first painting the background of the image
        			Image notImage8 = createImage(imageWidth, imageHeight);
    				Graphics gNotImg8 = notImage8.getGraphics();
    				gNotImg8.setColor(Color.lightGray);
					gNotImg8.fillRect(0, 0, notImage8.getWidth(this), notImage8.getHeight(this));

    				//draw not gate	8
    				notImage8 = notGate8.getImageRepresentation(notImage8, 10, 10, hintsOn);
        			g.drawImage(notImage8, notGate8x, notGate8y, this);
        		}

        		//the split connecters of the a and d inputs 
        		g.fillOval(gate1x-4, notGate1y+22,8,8);//a
        		g.fillOval(gate2x-4, notGate4y+22,8,8);//d

        		//next draw the text on to the circuit
        		g.drawString(aLabel, notGate1x-35, notGate1y+28);
    			g.drawString(bLabel, notGate2x-35, notGate2y+28);
    			g.drawString(cLabel, notGate3x-35, notGate3y+28);
    			g.drawString(dLabel, notGate4x-35, notGate4y+28);
    			g.drawString(eLabel, gate3x+5, gate3y+5);
    			g.drawString(fLabel, gate3x+5, gate3y+55);
    			g.drawString(gLabel, gate4x+5, gate4y+55);
    			g.drawString(hLabel, gate5x+5, gate5y+5);
    			g.drawString(iLabel, gate5x+5, gate5y+55);
    			g.drawString(outLabel, notGate8x+imageWidth-20, notGate8y+20);

    			g.setColor(Color.black);//redraw the outline box
    			g.drawRect(0,0,getWidth()-2,getHeight()-2);

    			if (hintsOn) {
        			//paint the preselected row of the correct answers numbers on to the panel

					//known outputs
					g.drawString(answers[hintRow][0].toString(), notGate1x-45, notGate1y+28);//a
					g.drawString(answers[hintRow][1].toString(), notGate2x-45, notGate2y+28);//b
					g.drawString(answers[hintRow][2].toString(), notGate3x-45, notGate3y+28);//c
					g.drawString(answers[hintRow][3].toString(), notGate4x-45, notGate4y+28);//d
					
					g.drawString(answers[hintRow][4].toString(), gate3x-10, gate3y+5); //e
					g.drawString(answers[hintRow][5].toString(), gate3x-10, gate3y+55);//f
					g.drawString(answers[hintRow][6].toString(), gate4x-10, gate4y+55);//g
					g.drawString(answers[hintRow][7].toString(), gate5x-10, gate5y+5); //h
					g.drawString(answers[hintRow][8].toString(), gate5x-10, gate5y+55);//i

					g.drawString(answers[hintRow][9].toString(), notGate8x+imageWidth-2, notGate8y+38);//output

					//after inverters outputs (if necessary)
					//note that the first five are after inverter's and the last three are before inverters
					if(notGate1Exists) {
						g.drawString(Integer.toString(notGate1.getOutput(Integer.parseInt((answers[hintRow][0].toString())))), notGate1x+imageWidth-15, notGate1y+20);
					}

					if(notGate2Exists) {
						g.drawString(Integer.toString(notGate2.getOutput(Integer.parseInt((answers[hintRow][1].toString())))), notGate2x+imageWidth-15, notGate2y+40);
					}

					if(notGate3Exists) {
						g.drawString(Integer.toString(notGate3.getOutput(Integer.parseInt((answers[hintRow][2].toString())))), notGate3x+imageWidth-15, notGate3y+20);
					}

					if(notGate4Exists) {
						g.drawString(Integer.toString(notGate4.getOutput(Integer.parseInt((answers[hintRow][3].toString())))), notGate4x+imageWidth-15, notGate4y+20);
					}

					if(notGate5Exists) {//the e value in the answer is already displayed so want the value before the not gate
						g.drawString(Integer.toString(notGate5.getOutput(Integer.parseInt((answers[hintRow][4].toString())))), notGate5x-5, notGate5y+20);
					}

					if(notGate6Exists) {//the f value in the answer is already displayed so want the value before the not gate
						g.drawString(Integer.toString(notGate6.getOutput(Integer.parseInt((answers[hintRow][5].toString())))), notGate6x-5, notGate6y+20);
					}

					if(notGate7Exists) {//the g value in the answer is already displayed so want the value before the not gate
						g.drawString(Integer.toString(notGate7.getOutput(Integer.parseInt((answers[hintRow][6].toString())))), notGate7x-5, notGate7y+20);
					}

					if(notGate8Exists) {//the output value in the answer is already displayed so want the value before the not gate
						g.drawString(Integer.toString(notGate8.getOutput(Integer.parseInt((answers[hintRow][9].toString())))), notGate8x-5, notGate8y+20);
					}
    			}
    		}
    	};
    }
}