import java.awt.*;
import java.lang.*;
import java.util.Random;
import javax.swing.JPanel;
import java.awt.dnd.*;
import java.awt.datatransfer.*;

/**
 * LogicCircuits.java 1.0 03/02/01
 * <br>
 * Author       : James McMahon
 * <br>
 * Implements the Circuit for a two gate approach of the following format.
 * <br>
 * <pre>
 * A--N1--|
 *        G1--N4--D--|
 * B--N2--|	         G2---Output
 *                   |
 * C--N3-------------|
 * </pre>
 * <br>
 * where G's are logic gates, N's are inverters and A, B, C are inputs.
 * All gates are drop target locations
 * <br>
 * Created      : 1/02/2003
 * <br>
 * Last Updated : 1/02/2003
 */
public class DropCircuit2GatesTop extends DropTargetCircuit {
	
	//the labels for the inputs and outputs
	private String aLabel = "A";
	private String bLabel = "B";
	private String cLabel = "C";
	private String dLabel = "D";

    //inputs to 2 gate
	private String outLabel = "OUTPUT";
	private String[] truthTableHeaders = {"A","B","C","D","Output"};

	//the number of columns for the truthtable that the user shouldn't be able to modify
	//equivalent to the first x columns
	private int numberOfFixedColumns = 5;

	//allows access to gates
	private LogicGates logicGates;

	//objects for all of the possible gates that can be held in the circuit
	//namely the two gates plus the 4 possible not gates
	private LogicGate actualGate1;
	private LogicGate actualGate2;
	private LogicGate actualNotGate1;
	private LogicGate actualNotGate2;
	private LogicGate actualNotGate3;
	private LogicGate actualNotGate4;

	//booleans to decide if the not gates need to be taken into account
	private boolean notGate1Exists = false;
	private boolean notGate2Exists = false;
	private boolean notGate3Exists = false;
	private boolean notGate4Exists = false;
	
	//objects to maintain state of the objects that are dropped on to this circuit
	private LogicGate userGate1 = null;
	private LogicGate userGate2 = null;
	private LogicGate userNotGate1 = null;
	private LogicGate userNotGate2 = null;
	private LogicGate userNotGate3 = null;
	private LogicGate userNotGate4 = null;
	
	//Allows access to the parent so a repaint can be accessed
	private QuestionGenerator parent;

	/**
	 * Set's up a random circuit that can be queryed using the methods provided
	 * 
	 * @param   logicComponents   The logicGates object that random gates can be drawn from
	 */
	public DropCircuit2GatesTop(LogicGates logicComponents, QuestionGenerator parentOwner) {
		logicGates = logicComponents;
		parent = parentOwner;

		//get two random gates for the circuit structure
		actualGate1 = logicGates.getRandomLogicGate();
		actualGate2 = logicGates.getRandomLogicGate();

		//decide randomly on the allocation of not gates

		//random number generator
		Random randNum = new Random();

		if (randNum.nextBoolean()) {
			actualNotGate1 = logicGates.getNotGate();
			notGate1Exists = true;
		} else {
			notGate1Exists = false;
		}

		if (randNum.nextBoolean()) {
			actualNotGate2 = logicGates.getNotGate();
			notGate2Exists = true;
		} else {
			notGate2Exists = false;
		}

		if (randNum.nextBoolean()) {
			actualNotGate3 = logicGates.getNotGate();
			notGate3Exists = true;
		} else {
			notGate3Exists = false;
		}

		if (randNum.nextBoolean()) {
			actualNotGate4 = logicGates.getNotGate();
			notGate4Exists = true;
		} else {
			notGate4Exists = false;
		}
	}

	/**
	 * Returns the data that is displayed in the truth table with question marks where user input is expected.
	 * 
	 * @return TruthTableDetails which are the actual contents of the default table
	 */
	public Object[][] getTruthTableDetails() {
		Object[][] correctAnswerDetails = {{"0","0","0", getDOutput(0,0,0), getOutput(0,0,0)},
										   {"0","0","1", getDOutput(0,0,1), getOutput(0,0,1)},
							    		   {"0","1","0", getDOutput(0,1,0), getOutput(0,1,0)},
							    		   {"0","1","1", getDOutput(0,1,1), getOutput(0,1,1)},
							    		   {"1","0","0", getDOutput(1,0,0), getOutput(1,0,0)},
							  			   {"1","0","1", getDOutput(1,0,1), getOutput(1,0,1)},
							    		   {"1","1","0", getDOutput(1,1,0), getOutput(1,1,0)},
							    		   {"1","1","1", getDOutput(1,1,1), getOutput(1,1,1)}};
		return correctAnswerDetails;
	}

	/**
	 * returns the headings for the truth table (such as A, B, C, Output etc)
	 * 
	 * @return   Headings The headings of the truthtable that can be used to instantiate a truthtable object
	 */
	public String[] getTruthTableHeadings() {
		return truthTableHeaders;
	}

	/**
	 * Returns the number of fixed columns that the user shouldn't be able to modify
	 * 
	 * @return the number of columns to be fixed from the far left
	 */
	public int getNumberOfFixedColumns() {
		return this.numberOfFixedColumns;
	}
	
	/**
	 * returns the ouput at D given a b and c input conditions
	 * <br>
	 * (in reality with this circuit only b and c actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0  is the output of the D Gate
	 */
	private String getDOutput(int a, int b, int c) {
		if (notGate1Exists) {
			a = actualNotGate1.getOutput(a);//returns the inverison of a's output
		}

		if (notGate2Exists) {
			b = actualNotGate2.getOutput(b);
		}

		int d = actualGate1.getOutput(a,b);//gets the result from gate 1 with the two (possibly inverted) inputs

		if (notGate4Exists) {//may need to invert d if the inverter exists
			d = actualNotGate4.getOutput(d);
		}

		return Integer.toString(d);
	}

	/**
	 * returns the final output of the circuit given the three initial starting conditions
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0 is the output of the full circuit
	 */
	private String getOutput(int a, int b, int c) {
		String dOut = getDOutput(a,b,c);
		int d = Integer.parseInt(dOut);

		if (notGate3Exists) {
			c = actualNotGate3.getOutput(c);
		}

		int output = actualGate2.getOutput(d, c);

		return Integer.toString(output);
	}

	/**
	 * returns the ouput at D given a b and c input conditions
	 * <br>
	 * (in reality with this circuit only b and c actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0  is the output of the D Gate
	 */
	private String getUserBasedDOutput(int a, int b, int c) {
		try {
			a = userNotGate1.getOutput(a);//returns the inverison of a's output whther its a line or not
			b = userNotGate2.getOutput(b);

			int d = userGate1.getOutput(a,b);//gets the result from user placed gate 1 with the two (possibly inverted) inputs

			d = userNotGate4.getOutput(d);//invert or propagate depending on the user placed gate
		
			return Integer.toString(d);
		}
		catch(NullPointerException npe) {
			return "-1";
		}
	}

	/**
	 * returns the final output of the circuit given the three initial starting conditions
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0 is the output of the full circuit
	 */
	private String getUserBasedOutput(int a, int b, int c) {
		try {
			String dOut = getUserBasedDOutput(a,b,c);
			int d = Integer.parseInt(dOut);

			//invert or propagate based on userplaced gate
			c = userNotGate3.getOutput(c);
		
			int output = userGate2.getOutput(d, c);

			return Integer.toString(output);
		}
		catch(NullPointerException npe) {
			return "-1";
		}
	}

	/**
	 * gives access to whatever truthtable the gates that the user has placed creates
	 * <p>
	 * N.B. It is important that the gates are checked for correspondance to the actual
	 * truth table details - not just residual information regarding the gates which were
	 * intially used to set up the question! (as more than one setup could cause the same answers)
	 *
	 * @return The truthtable details corresponding to the user built circuit
	 */
	public Object[][] getUserTruthTable() {
		Object[][] userCreatedTruthTable = {{"0","0","0", getUserBasedDOutput(0,0,0), getUserBasedOutput(0,0,0)},
										    {"0","0","1", getUserBasedDOutput(0,0,1), getUserBasedOutput(0,0,1)},
							    		    {"0","1","0", getUserBasedDOutput(0,1,0), getUserBasedOutput(0,1,0)},
							    		    {"0","1","1", getUserBasedDOutput(0,1,1), getUserBasedOutput(0,1,1)},
							    		    {"1","0","0", getUserBasedDOutput(1,0,0), getUserBasedOutput(1,0,0)},
							  			    {"1","0","1", getUserBasedDOutput(1,0,1), getUserBasedOutput(1,0,1)},
							    		    {"1","1","0", getUserBasedDOutput(1,1,0), getUserBasedOutput(1,1,0)},
						    			    {"1","1","1", getUserBasedDOutput(1,1,1), getUserBasedOutput(1,1,1)}};
		return userCreatedTruthTable;
	}

	/**
	 * redraws the screen when necessary
	 */
	public void redrawScreen() {
		parent.repaintScreen();	
	}
	
	/**
	 * Deals with a dropped gate at location xLoc,yLoc.
	 * If the location of the drop corresponds to a valid gate site then
	 * the gate is set to that site and should be displayed.
	 *
	 * @param gate The gate that was dropped
	 * @param location The point that it was dropped at
	 */
	public void dealWithDrop(LogicGate gate, Point location) {
	
		DropBox b = this.getDropBoxLocations().getDropBoxContainingPoint(location);
		if (b!=null) {
			String gateName = b.toString();
			if (gateName.equals("userGate1")) {
				userGate1 = gate;		
			}
			else if (gateName.equals("userGate2")) {
				userGate2 = gate;
			}
			else if (gateName.equals("userNotGate1")) {
				userNotGate1 = gate;
			}
			else if (gateName.equals("userNotGate2")) {
				userNotGate2 = gate;
			}
			else if (gateName.equals("userNotGate3")) {
				userNotGate3 = gate;
			}
			else if (gateName.equals("userNotGate4")) {
				userNotGate4 = gate;
			}
		}
	}

	/**
	 * Ensures that the user has at least placed random gates in all locations that need them.
	 * If all not gate locations have either not gates or "line" gates and the other gates
	 * have some kind of gate registered then it returns true- otherwise false
	 *
	 * @return If all gates have been filled and not gate positions have only a line or not gate placed
	 */
	public boolean allUserGatesPlaced() {
		if ((userGate1 != null) && (userGate2 != null) && (userNotGate1 != null) &&
		    (userNotGate2 != null) && (userNotGate3 != null) && (userNotGate4 != null)) {
		       
			if (((userNotGate1.toString().equals("LINE \"gate\"")) ||
				 (userNotGate1.toString().equals("NOT gate"))) &&
				((userNotGate2.toString().equals("LINE \"gate\"")) ||
				 (userNotGate2.toString().equals("NOT gate"))) &&
				((userNotGate3.toString().equals("LINE \"gate\"")) ||
				 (userNotGate3.toString().equals("NOT gate"))) &&
				((userNotGate4.toString().equals("LINE \"gate\"")) ||
				 (userNotGate4.toString().equals("NOT gate")))) {
					
				if (((!(userGate1.toString().equals("LINE \"gate\""))) &&
				     (!(userGate1.toString().equals("NOT gate")))) && 
				    ((!(userGate2.toString().equals("LINE \"gate\""))) &&
				     (!(userGate2.toString().equals("NOT gate"))))) {
				     //all gates are placed and all notgate positions have NOt gates or LINE gates only
					return true;
				}
				else {		
					return false;//one of the user gates with two inputs has a one input gate placed
				}
			}
			else {
				return false;//user has been silly and put a non NOT|LINE gate in one of the places for it	
			}
		}
		else {
			return false;//user has not filled all of the positions
		}
	}

	/**
	 * Allows access to all of the gate locations users are allowed to drop into
	 * <p>
	 * For simplification the relative gate locations defined in the JPanel 
	 * that the gates are drawn onto were recreated here and not actually referenced.
	 * Hence these need to be manually updated if the actual locations change!
	 * 
	 * @return A list of all dropbox locations that are available for user placed gates
	 */
	public DropBoxList getDropBoxLocations() {
		DropBoxList list = new DropBoxList();
		
		//set up the locations for all of the boxes that are the drop target locations
		// - the figures come from the main JPAnel where they are actually drawn on to it
    	int imageWidth  = 75;
    	int imageHeight = 50;

		int not1X1 = 120;
		int not1Y1 =  20;
	
		int not2X1 = not1X1;
		int not2Y1 = not1Y1+imageHeight+20;
		
		int not3X1 = not1X1;
		int not3Y1 = not1Y1+not2Y1+imageHeight;
	
		int gate1X1 = not1X1+imageWidth+20;
		int gate1Y1 = not1Y1+35;
		
		int not4X1 = gate1X1+imageWidth;
		int not4Y1 = gate1Y1;
		
		int gate2X1 = not4X1+imageWidth;
		int gate2Y1 = not4Y1+imageHeight+10;
			
		list.addDropBoxtoList(new DropBox(new Point(gate1X1, gate1Y1), new Point(gate1X1+imageWidth, gate1Y1+imageHeight), "userGate1"));
		list.addDropBoxtoList(new DropBox(new Point(gate2X1, gate2Y1), new Point(gate2X1+imageWidth, gate2Y1+imageHeight), "userGate2"));
		list.addDropBoxtoList(new DropBox(new Point(not1X1, not1Y1), new Point(not1X1+imageWidth, not1Y1+imageHeight), "userNotGate1"));
		list.addDropBoxtoList(new DropBox(new Point(not2X1, not2Y1), new Point(not2X1+imageWidth, not2Y1+imageHeight), "userNotGate2"));
		list.addDropBoxtoList(new DropBox(new Point(not3X1, not3Y1), new Point(not3X1+imageWidth, not3Y1+imageHeight), "userNotGate3"));
		list.addDropBoxtoList(new DropBox(new Point(not4X1, not4Y1), new Point(not4X1+imageWidth, not4Y1+imageHeight), "userNotGate4"));		
		
		return list;
	}

	/**
	 * Sets up the JPanel that contains the circuit
	 * 
	 * @param    hints   If true then hints are drawn on to the circuit otherwise they aren't
	 * @param    complete If false then the drop targets are highlighted, otherwise there are filled in 
	 *
	 * @return   CircuitDiagram - A diagram drawn on to a JPanel with or without hints based on the hints input
	 */
	public JPanel getCircuitDiagramPanel(boolean hints, boolean complete)	{
		
		//these need to be final because we may need to access them inside the paint method so that it a row can be painted
		final Object[][] answers = getTruthTableDetails();
		Random rand = new Random();
		//return a random int range 0 to length-1
		final int hintRow = rand.nextInt(answers.length);
		
		final boolean hintsOn = hints;
		final boolean completed = complete;
		setRevealedStatus(complete);//store this so we can do sensible output when revealing hints etc
		
		JPanel temp = new JPanel(){
   			public void paintComponent(Graphics g) {
   	    		super.paintComponent(g); //paint background

    			//set the image width and height
    			int imageWidth  = 75;
    			int imageHeight = 50;

    			//need to create a standard box image that can be used as a blank target spacer
    			Image boxImage = createImage(imageWidth, imageHeight);
    			Graphics boxImg = boxImage.getGraphics();
				//make it the right colour of background
				boxImg.setColor(Color.lightGray);
				boxImg.fillRect(0,0,boxImage.getWidth(this),boxImage.getHeight(this));
	  			//fill the external border to make it stand out
	  			boxImg.setColor(Color.blue);
			  	boxImg.draw3DRect(1,1,imageWidth-6,imageHeight-4,true);
				//put on the big question marke
				boxImg.setColor(Color.black);
				boxImg.drawString("?",((imageWidth-6)/2),((imageHeight+6)/2));

				//all of the gate locations are set up as relative as possible
				//hence they need to be set up in the order in which they appear in the diagram

				//set up the first three not gates
     			int notGate1x = 120;
    			int notGate1y = 20;

    			int notGate2x = notGate1x;
    			int notGate2y = notGate1y+imageHeight+20; //20 is spacing

    			int notGate3x = notGate1x;
    			int notGate3y = notGate1y+notGate2y+imageHeight;

    			//set the root locations of the 1st random gate
    			int gate1x = notGate1x+imageWidth+20;//20 is spacing
    			int gate1y = notGate1y+35;           //25 is half of image height with 15 spacing

    			//set the location of the 4th not gate
    			int notGate4x = gate1x+imageWidth;//20 is spacing
    			int notGate4y = gate1y;              //

    			//set the root location of the second random gate
    			int gate2x = notGate4x+imageWidth;
    			int gate2y = notGate4y+imageHeight+10;//10 is spacing

    			//set the background of the actual panel
    			g.setColor(Color.lightGray);
				g.fillRect(0,0,getWidth(),getHeight());
    			g.setColor(Color.black);
    			g.drawRect(0,0,getWidth()-2,getHeight()-2);

    			//draw the text on to the screen
    			g.drawString(aLabel, notGate1x-15, notGate1y+28);
    			g.drawString(bLabel, notGate2x-15, notGate2y+28);
    			g.drawString(cLabel, notGate3x-15, notGate3y+28);
    			g.drawString(dLabel, notGate4x+imageWidth+8, notGate4y+imageHeight+4);
    			g.drawString(outLabel, gate2x+imageWidth, gate2y+30);//30 is image start position (10) + halfway down the circuit (15)
    																 //with 5 spacing

        		//first paint on the two random gates
        		//get the image of the 1st gate after first painting the background of the image
    			Image image = createImage(imageWidth, imageHeight);
    			Graphics gImg = image.getGraphics();
				gImg.setColor(Color.lightGray);
				gImg.fillRect(0,0,image.getWidth(this),image.getHeight(this));

    			//draw random gate1 or blank box or user gate depending on the status
    			if (completed) {
    				image = actualGate1.getImageRepresentation(image, 10, 10, hintsOn);//get the actual image
        			g.drawImage(image, gate1x, gate1y, this);//Draw image at its natural size first (75x50 image);
				}
				else {
					if (userGate1==null) {
						g.drawImage(boxImage, gate1x, gate1y, this);
					}
					else {//the user has placed a gate in here so we must show that
						image = userGate1.getImageRepresentation(image, 10, 10, hintsOn);//get the actual image
        				
        				//fill the external border to make it stand out
	  					gImg.setColor(Color.gray);
					  	gImg.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
					  	
        				g.drawImage(image, gate1x, gate1y, this);//Draw image at its natural size first (75x50 image);
					}
				}

        		//get the image of the 2nd gate after first painting the background of the image
        		Image image2 = createImage(imageWidth, imageHeight);
    			Graphics gImg2 = image2.getGraphics();
    			gImg2.setColor(Color.lightGray);
				gImg2.fillRect(0,0,image2.getWidth(this),image2.getHeight(this));

    			//draw random gate2 or blank box or user gate depending on status
    			if (completed) {
    				image2 = actualGate2.getImageRepresentation(image2, 10, 10, hintsOn);
        			g.drawImage(image2, gate2x, gate2y, this);
				}
				else {
					if (userGate2==null) {
						g.drawImage(boxImage, gate2x, gate2y, this);
					}
					else {//the user has placed a gate in here so we must show that
    					image2 = userGate2.getImageRepresentation(image2, 10, 10, hintsOn);
    					
        				//fill the external border to make it stand out
	  					gImg2.setColor(Color.gray);
					  	gImg2.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);

        				g.drawImage(image2, gate2x, gate2y, this);
        			}						
				}

        		//draw all of the connecters in such a way that a line is already drawn underneath any possible inverter gates
        		//draw a connector between notgate1 and random gate 1
        		int x1 = notGate1x+imageWidth;
        		int y1 = notGate1y+10+15;//10 is image start position, 15 is connecter start point
        		int x2 = x1;
        		int y2 = gate1y+10+8;//start position (10) plus first connecter point
        		g.drawLine(x1,y1,x2,y2);//top connecter down line

        		x1 = notGate1x+imageWidth;
        		y1 = gate1y+10+8;//10 is image start position, 8 is first connecter start point
        		x2 = gate1x;
        		y2 = y1;
        		g.drawLine(x1,y1,x2,y2);//top across connecter up line

        		//draw a connecter that will fill any gaps left by a missing not gate
        		x1 = notGate1x;
        		y1 = notGate1y+10+15;//10 is image start position, 15 is connecter start point
        		x2 = notGate1x+imageWidth;
        		y2 = y1;//start position (10) plus second  connecter point
        		g.drawLine(x1,y1,x2,y2);//middle across connecter line

        		//draw a connector between notgate2 and random gate 1
        		x1 = notGate2x+imageWidth;
        		y1 = notGate2y+10+15;//10 is image start position, 15 is connecter start point
        		x2 = x1;
        		y2 = gate1y+10+22;//start position (10) plus second  connecter point
        		g.drawLine(x1,y1,x2,y2);//middle connecter up line

				x1 = notGate2x+imageWidth;
        		y1 = gate1y+10+22;//10 is image start position, 22 is second connecter start point
        		x2 = gate1x;
        		y2 = y1;//start position (10) plus second connecter point
        		g.drawLine(x1,y1,x2,y2);//middle across connecter up line

        		//draw a connecter that will fill any gaps left by a missing not gate
        		x1 = notGate2x;
        		y1 = notGate2y+10+15;//10 is image start position, 15 is connecter start point
        		x2 = notGate2x+imageWidth;
        		y2 = y1;//start position (10) plus second  connecter point
        		g.drawLine(x1,y1,x2,y2);//middle across connecter line

        		//connect bottom not gate position to second random gate
        		x1 = notGate3x;
        		y1 = notGate3y+10+15;//10 is image start position, 15 is connecter start point
        		x2 = gate2x;
        		y2 = y1;//start position (10) plus second  connecter point
        		g.drawLine(x1,y1,x2,y2);//bottom connecter across line

        		x1 = gate2x;
        		y1 = notGate3y+10+15;//10 is image start position, 15 is connecter start point
        		x2 = x1;
        		y2 = gate2y+10+22;//start position (10) plus second connecter point
        		g.drawLine(x1,y1,x2,y2);//bottom connecter up line

        		//connect 4th not gate to the second random gate
        		x1 = notGate4x+imageWidth;
        		y1 = notGate4y+10+15;//10 is image start position, 15 is connecter start point
        		x2 = x1;
        		y2 = gate2y+10+8;//start position (10) plus first connecter point
        		g.drawLine(x1,y1,x2,y2);//bottom connecter across line

        		//draw a connecter that will fill any gaps left by a missing not gate
        		x1 = notGate4x;
        		y1 = notGate4y+10+15;//10 is image start position, 15 is connecter start point
        		x2 = notGate4x+imageWidth;
        		y2 = y1;//start position (10) plus second  connecter point
        		g.drawLine(x1,y1,x2,y2);//middle across connecter line

        		 //next draw the not gates if they have been set for this particular diagram

        		//get the image of the 1st not gate after first painting the background of the image
        		Image image3 = createImage(imageWidth, imageHeight);
    			Graphics gImg3 = image3.getGraphics();
    			gImg3.setColor(Color.lightGray);
				gImg3.fillRect(0,0,image3.getWidth(this),image3.getHeight(this));
        	
        		if (completed) {//need to draw the notGate1 properly or just a normal line through
        			if (notGate1Exists) {      				
    					//draw not gate	1
    					image3 = actualNotGate1.getImageRepresentation(image3, 10, 10, hintsOn);
        				g.drawImage(image3, notGate1x, notGate1y, this);        				
        			}
        		}
        		else {//need to draw the blank box or the user placed gate
        			if (userNotGate1 == null) { //draw the blank box
        				g.drawImage(boxImage, notGate1x, notGate1y, this);
        			}
        			else {//draw the gate the user thinks should be here
    					image3 = userNotGate1.getImageRepresentation(image3, 10, 10, hintsOn);
        				
         				//fill the external border to make it stand out
	  					gImg3.setColor(Color.gray);
					  	gImg3.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
        				
        				g.drawImage(image3, notGate1x, notGate1y, this);
        			}
        		}

      			//get the image of the 2nd not gate after first painting the background of the image
       			Image image4 = createImage(imageWidth, imageHeight);
  				Graphics gImg4 = image4.getGraphics();
   				gImg4.setColor(Color.lightGray);
				gImg4.fillRect(0,0,image4.getWidth(this),image4.getHeight(this));			
			
				if (completed) {//need to draw the notGate2 properly or just a normal line through
               		if (notGate2Exists) {
    					//draw not gate	2
    					image4 = actualNotGate2.getImageRepresentation(image4, 10, 10, hintsOn);
        				g.drawImage(image4, notGate2x, notGate2y, this);        				
        			}
        		}
        		else {//need to draw the blank box or the user placed gate
        			if (userNotGate2 == null) { //draw the blank box
        				g.drawImage(boxImage, notGate2x, notGate2y, this);
        			}
        			else {//draw the gate the user thinks should be here
    					image4 = userNotGate2.getImageRepresentation(image4, 10, 10, hintsOn);
        				
         				//fill the external border to make it stand out
	  					gImg4.setColor(Color.gray);
					  	gImg4.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);   	
     
        				g.drawImage(image4, notGate2x, notGate2y, this);
        			}
        		}

       			//get the image of the 3rd not gate after first painting the background of the image
       			Image image5 = createImage(imageWidth, imageHeight);
   				Graphics gImg5 = image5.getGraphics();
   				gImg5.setColor(Color.lightGray);
				gImg5.fillRect(0,0,image5.getWidth(this),image5.getHeight(this));

				if (completed) {//need to draw the notGate3 properly or just a normal line through
        			if (notGate3Exists) {
    					//draw not gate	3
    					image5 = actualNotGate3.getImageRepresentation(image5, 10, 10, hintsOn);
        				g.drawImage(image5, notGate3x, notGate3y, this);        				
        			}
        		}
        		else {//need to draw the blank box or the user placed gate
        			if (userNotGate3 == null) { //draw the blank box
        				g.drawImage(boxImage, notGate3x, notGate3y, this);
        			}
        			else {//draw the gate the user thinks should be here
    					image5 = userNotGate3.getImageRepresentation(image5, 10, 10, hintsOn);
        			
         				//fill the external border to make it stand out
	  					gImg5.setColor(Color.gray);
					  	gImg5.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
        			
        				g.drawImage(image5, notGate3x, notGate3y, this);
        			}
        		}

        		//get the image of the 4th not gate after first painting the background of the image
        		Image image6 = createImage(imageWidth, imageHeight);
    			Graphics gImg6 = image6.getGraphics();
    			gImg6.setColor(Color.lightGray);
				gImg6.fillRect(0, 0, image6.getWidth(this), image6.getHeight(this));        		
        		
  				if (completed) {//need to draw the notGate4 properly or just a normal line through  		
        			if (notGate4Exists) {
   					//draw not gate	3
    					image6 = actualNotGate4.getImageRepresentation(image6, 10, 10, hintsOn);
        				g.drawImage(image6, notGate4x, notGate4y, this);        				
        			}
        		}
        		else {//need to draw the blank box or the user placed gate
        			if (userNotGate4 == null) { //draw the blank box
        				g.drawImage(boxImage, notGate4x, notGate4y, this);
        			}
        			else {//draw the gate the user thinks should be here
    					image6 = userNotGate4.getImageRepresentation(image6, 10, 10, hintsOn);
 
         				//fill the external border to make it stand out
	  					gImg6.setColor(Color.gray);
					  	gImg6.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);       				
 
        				g.drawImage(image6, notGate4x, notGate4y, this);
        			}
        		}

				if (hintsOn) {//then need to draw the hints
							  //paints a row of the correct answers numbers on to the panel

					//known outputs
					g.drawString(answers[hintRow][0].toString(), notGate1x-35, notGate1y+28);
					g.drawString(answers[hintRow][1].toString(), notGate2x-35, notGate2y+28);
					g.drawString(answers[hintRow][2].toString(), notGate3x-35, notGate3y+28);
					g.drawString(answers[hintRow][3].toString(), notGate4x+imageWidth-15, notGate4y+imageHeight+4);
					g.drawString(answers[hintRow][4].toString(), gate2x+imageWidth+65, gate2y+30);

					//after inverters outputs (if necessary)
					if(notGate1Exists) {
						g.drawString(Integer.toString(actualNotGate1.getOutput(Integer.parseInt((answers[hintRow][0].toString())))), notGate1x+imageWidth+5, notGate1y+25);
					}

					if(notGate2Exists) {
						g.drawString(Integer.toString(actualNotGate2.getOutput(Integer.parseInt((answers[hintRow][1].toString())))), notGate2x+imageWidth+5, notGate2y+25);
					}

					if(notGate3Exists) {
						g.drawString(Integer.toString(actualNotGate3.getOutput(Integer.parseInt((answers[hintRow][2].toString())))), notGate3x+imageWidth+5, notGate3y+20);
					}

					if(notGate4Exists) {//the d value in the answer is already displayed so want the value before the not gate
						g.drawString(Integer.toString(actualNotGate4.getOutput(Integer.parseInt((answers[hintRow][3].toString())))), notGate4x-5, notGate4y+20);
					}
				}
			}
		};
		dropTarget = new DropTarget(temp, new DTListener(this));
		return temp;
	}
}