import java.awt.*;
import java.lang.*;
import java.util.Random;
import javax.swing.JPanel;
import java.awt.dnd.*;
import java.awt.datatransfer.*;

/**
 * LogicCircuits.java 1.0 03/02/01
 * <br>
 * Author       : James McMahon
 * <br>
 * Implements the Circuit for a three gate linear approach of the following format.
 * <br>
 * <pre>
 * A--N1--|
 *        |
 *   N2--G1--N5---|
 * B-|            G2--N6--|
 *   N3------|	  |       G3---Output
 *           ----_|_------|
 * C--N4----------|
 * </pre>
 * <br>
 * where G's are logic gates, N's are inverters and A, B, C are inputs.
 * All gates aredrop locations.
 * <br>
 * Created      : 1/02/2003
 * <br>
 * Last Updated : 1/02/2003
 */
 
public class DropCircuit3LinearGates extends DropTargetCircuit {
	
	//the labels for the inputs and outputs
	private String aLabel = "A";
	private String bLabel = "B";
	private String cLabel = "C";
	private String dLabel = "D";

	//inputs to 2nd gate
	private String eLabel = "E";

	//inputs to 3rd gate
	private String outLabel = "OUTPUT";
	private String[] truthTableHeaders = {"A","B","C","D","E","Output"};

	//the number of columns for the truthtable that the user shouldn't be able to modify
	//equivalent to the first x columns
	private int numberOfFixedColumns = 6;

	//allows access to gates
	private LogicGates logicGates;

	//objects for all of the possible gates that can be held in the circuit
	//namely the three gates plus the 6 possible not gates
	private LogicGate actualGate1;
	private LogicGate actualGate2;
	private LogicGate actualGate3;
	private LogicGate actualNotGate1;
	private LogicGate actualNotGate2;
	private LogicGate actualNotGate3;
	private LogicGate actualNotGate4;
	private LogicGate actualNotGate5;
	private LogicGate actualNotGate6;

	//booleans to decide if the not gates need to be taken into account
	private boolean notGate1Exists = false;
	private boolean notGate2Exists = false;
	private boolean notGate3Exists = false;
	private boolean notGate4Exists = false;
	private boolean notGate5Exists = false;
	private boolean notGate6Exists = false;

	//objects for all of the possible gates that can user placed onto the circuit
	//namely the three gates plus the 6 possible not gates 
	private LogicGate userGate1 = null;
	private LogicGate userGate2 = null;
	private LogicGate userGate3 = null;
	private LogicGate userNotGate1 = null;
	private LogicGate userNotGate2 = null;
	private LogicGate userNotGate3 = null;
	private LogicGate userNotGate4 = null;
	private LogicGate userNotGate5 = null;
	private LogicGate userNotGate6 = null;
	
	//Allows access to the parent so a repaint can be accessed
	private QuestionGenerator parent;	
	
	/**
	 * Set's up a random circuit that can be queryed using the methods provided
	 * 
	 * @param   logicComponents   The logicGates object that random gates can be drawn from
	 */
	public DropCircuit3LinearGates(LogicGates logicComponents, QuestionGenerator parentOwner) {
		logicGates = logicComponents;
		parent = parentOwner;

		//get three random gates for the circuit structure
		actualGate1 = logicGates.getRandomLogicGate();
		actualGate2 = logicGates.getRandomLogicGate();
		actualGate3 = logicGates.getRandomLogicGate();

		//decide randomly on the allocation of not gates

		//random number generator
		Random randNum = new Random();

		if (randNum.nextBoolean()) {
			actualNotGate1 = logicGates.getNotGate();
			notGate1Exists = true;
		} else {
			notGate1Exists = false;
		}

		if (randNum.nextBoolean()) {
			actualNotGate2 = logicGates.getNotGate();
			notGate2Exists = true;
		} else {
			notGate2Exists = false;
		}

		if (randNum.nextBoolean()) {
			actualNotGate3 = logicGates.getNotGate();
			notGate3Exists = true;
		} else {
			notGate3Exists = false;
		}

		if (randNum.nextBoolean()) {
			actualNotGate4 = logicGates.getNotGate();
			notGate4Exists = true;
		} else {
			notGate4Exists = false;
		}

		if (randNum.nextBoolean()) {
			actualNotGate5 = logicGates.getNotGate();
			notGate5Exists = true;
		} else {
			notGate5Exists = false;
		}

		if (randNum.nextBoolean()) {
			actualNotGate6 = logicGates.getNotGate();
			notGate6Exists = true;
		} else {
			notGate6Exists = false;
		}
	}

	/**
	 * Returns the data that is displayed in the truth table with question marks where user input is expected.
	 * 
	 * @return TruthTableDetails which are the actual contents of the default table
	 */
	public Object[][] getTruthTableDetails() {
		Object[][] correctAnswerDetails = {{"0","0","0", getDOutput(0,0,0), getEOutput(0,0,0), getOutput(0,0,0)},
										   {"0","0","1", getDOutput(0,0,1), getEOutput(0,0,1), getOutput(0,0,1)},
							    		   {"0","1","0", getDOutput(0,1,0), getEOutput(0,1,0), getOutput(0,1,0)},
							    		   {"0","1","1", getDOutput(0,1,1), getEOutput(0,1,1), getOutput(0,1,1)},
							    		   {"1","0","0", getDOutput(1,0,0), getEOutput(1,0,0), getOutput(1,0,0)},
							  			   {"1","0","1", getDOutput(1,0,1), getEOutput(1,0,1), getOutput(1,0,1)},
							    		   {"1","1","0", getDOutput(1,1,0), getEOutput(1,1,0), getOutput(1,1,0)},
							    		   {"1","1","1", getDOutput(1,1,1), getEOutput(1,1,1), getOutput(1,1,1)}};
		return correctAnswerDetails;
	}

	/**
	 * returns the ouput at D given a b and c input conditions
	 * <br>
	 * (in reality with this circuit only b and c actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0  is the output of the D Gate
	 */
	private String getDOutput(int a, int b, int c) {
		if (notGate1Exists) {
			a = actualNotGate1.getOutput(a);
		}
		if (notGate2Exists) {
			b = actualNotGate2.getOutput(b);
		}

		int d = actualGate1.getOutput(a,b);

		if (notGate5Exists) {//then need to invert d as well now
			d = actualNotGate5.getOutput(d);
		}
		return Integer.toString(d);
	}

	/**
	 * returns the ouput at E given a b and c input conditions
	 * <br>
	 * (in reality with this circuit only b and c actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0  is the output of the D Gate
	 */
	private String getEOutput(int a, int b, int c) {
		String dOut = getDOutput(a,b,c);
		int d = Integer.parseInt(dOut);

		if (notGate4Exists) {//invert c
			c = actualNotGate4.getOutput(c);
		}

		int e = actualGate2.getOutput(d, c);

		if (notGate6Exists) {//need to invert the e output as well
			e = actualNotGate6.getOutput(e);
		}
		return Integer.toString(e);
	}

	/**
	 * returns the final output of the circuit given the three initial starting conditions
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0 is the output of the full circuit
	 */
	private String getOutput(int a, int b, int c) {
		String eOut = getEOutput(a,b,c);
		int e = Integer.parseInt(eOut);

		if (notGate3Exists) {
			b = actualNotGate3.getOutput(b);
		}

		int output = actualGate3.getOutput(e,b);

		return Integer.toString(output);
	}

	/**
	 * Returns the headings for the truth table (such as A, B, C, Output etc).
	 * 
	 * @return   Headings The headings of the truthtable that can be used to instantiate a truthtable object
	 */
	public String[] getTruthTableHeadings() {
		return truthTableHeaders;
	}

	/**
	 * Returns the number of fixed columns that the user shouldn't be able to modify
	 * 
	 * @return the number of columns to be fixed from the far left
	 */
	public int getNumberOfFixedColumns() {
		return this.numberOfFixedColumns;
	}

	/**
	 * returns the ouput at D given a b and c input conditions and the user placed gates
	 * <br>
	 * (in reality with this circuit only b and c actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0  is the output of the D Gate
	 */
	private String getUserBasedDOutput(int a, int b, int c) {
		try {
			a = userNotGate1.getOutput(a);
			b = userNotGate2.getOutput(b);
		
			int d = userGate1.getOutput(a,b);
			d = userNotGate5.getOutput(d);
	
			return Integer.toString(d);
		}
		catch(NullPointerException npe) {
			return "-1";
		}
	}

	/**
	 * returns the ouput at E given a b and c input conditions and the user placed gates
	 * <br>
	 * (in reality with this circuit only b and c actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0  is the output of the D Gate
	 */
	private String getUserBasedEOutput(int a, int b, int c) {
		try {
			String dOut = getUserBasedDOutput(a,b,c);
		
			int d = Integer.parseInt(dOut);
			c = userNotGate4.getOutput(c);
			int e = userGate2.getOutput(d, c);
			//we may need to invert the e output as well
			e = userNotGate6.getOutput(e);
		
			return Integer.toString(e);
		}
		catch(NullPointerException npe) {
			return "-1";
		}
	}

	/**
	 * returns the final output of the circuit given the three initial starting conditions
	 * and the user placed gates 
	 *
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * 
	 * @return   0 If 0 is the output of the full circuit
	 */
	private String getUserBasedOutput(int a, int b, int c) {
		try {
			String eOut = getUserBasedEOutput(a,b,c);
			int e = Integer.parseInt(eOut);

			b = userNotGate3.getOutput(b);

			int output = userGate3.getOutput(e,b);

			return Integer.toString(output);
		}
		catch(NullPointerException npe) {
			return "-1";
		}
	}

	/**
	 * gives access to whatever truthtable the gates that the user has placed creates
	 * <p>
	 * N.B. It is important that the gates are checked for correspondance to the actual
	 * truth table details - not just residual information regarding the gates which were
	 * intially used to set up the question! (as more than one setup could cause the same answers)
	 *
	 * @return The truthtable details corresponding to the user built circuit
	 */
	public Object[][] getUserTruthTable() {
		Object[][] userCreatedTruthTable = {{"0","0","0", getUserBasedDOutput(0,0,0), getUserBasedEOutput(0,0,0), getUserBasedOutput(0,0,0)},
										    {"0","0","1", getUserBasedDOutput(0,0,1), getUserBasedEOutput(0,0,1), getUserBasedOutput(0,0,1)},
							    		    {"0","1","0", getUserBasedDOutput(0,1,0), getUserBasedEOutput(0,1,0), getUserBasedOutput(0,1,0)},
							    		    {"0","1","1", getUserBasedDOutput(0,1,1), getUserBasedEOutput(0,1,1), getUserBasedOutput(0,1,1)},
							    		    {"1","0","0", getUserBasedDOutput(1,0,0), getUserBasedEOutput(1,0,0), getUserBasedOutput(1,0,0)},
							  			    {"1","0","1", getUserBasedDOutput(1,0,1), getUserBasedEOutput(1,0,1), getUserBasedOutput(1,0,1)},
							    		    {"1","1","0", getUserBasedDOutput(1,1,0), getUserBasedEOutput(1,1,0), getUserBasedOutput(1,1,0)},
							    		    {"1","1","1", getUserBasedDOutput(1,1,1), getUserBasedEOutput(1,1,1), getUserBasedOutput(1,1,1)}};
		return userCreatedTruthTable;
	}

	/**
	 * redraws the screen when necessary
	 */
	public void redrawScreen() {
		parent.repaintScreen();	
	}
	
	/**
	 * Deals with a dropped gate at location xLoc,yLoc.
	 * If the location of the drop corresponds to a valid gate site then
	 * the gate is set to that site and should be displayed.
	 *
	 * @param gate The gate that was dropped
	 * @param location The point that it was dropped at
	 */
	public void dealWithDrop(LogicGate gate, Point location) {
	
		DropBox b = this.getDropBoxLocations().getDropBoxContainingPoint(location);
		if (b!=null) {
			String gateName = b.toString();
			if (gateName.equals("userGate1")) {
				userGate1 = gate;		
			}
			else if (gateName.equals("userGate2")) {
				userGate2 = gate;
			}
			else if (gateName.equals("userGate3")) {
				userGate3 = gate;
			}			
			else if (gateName.equals("userNotGate1")) {
				userNotGate1 = gate;
			}
			else if (gateName.equals("userNotGate2")) {
				userNotGate2 = gate;
			}
			else if (gateName.equals("userNotGate3")) {
				userNotGate3 = gate;
			}
			else if (gateName.equals("userNotGate4")) {
				userNotGate4 = gate;
			}
			else if (gateName.equals("userNotGate5")) {
				userNotGate5 = gate;
			}
			else if (gateName.equals("userNotGate6")) {
				userNotGate6 = gate;
			}
		}
	}

	/**
	 * Ensures that the user has at least placed random gates in all locations that need them.
	 * If all not gate locations have either not gates or "line" gates and the other gates
	 * have some kind of gate registered then it returns true- otherwise false
	 *
	 * @return If all gates have been filled and not gate positions have only a line or not gate placed
	 */
	public boolean allUserGatesPlaced() {
		if ((userGate1 != null) && (userGate2 != null) && (userGate3 != null) &&
		    (userNotGate1 != null) && (userNotGate2 != null) && (userNotGate3 != null) && 
		    (userNotGate4 != null) && (userNotGate5 != null) && (userNotGate6 != null)) {
		       
			if (((userNotGate1.toString().equals("LINE \"gate\"")) ||
				 (userNotGate1.toString().equals("NOT gate"))) &&
				((userNotGate2.toString().equals("LINE \"gate\"")) ||
				 (userNotGate2.toString().equals("NOT gate"))) &&
				((userNotGate3.toString().equals("LINE \"gate\"")) ||
				 (userNotGate3.toString().equals("NOT gate"))) &&
				((userNotGate4.toString().equals("LINE \"gate\"")) ||
				 (userNotGate4.toString().equals("NOT gate"))) &&
				((userNotGate5.toString().equals("LINE \"gate\"")) ||
				 (userNotGate5.toString().equals("NOT gate"))) &&
				((userNotGate6.toString().equals("LINE \"gate\"")) ||
				 (userNotGate6.toString().equals("NOT gate")))) {
			
				if (((!(userGate1.toString().equals("LINE \"gate\""))) &&
				     (!(userGate1.toString().equals("NOT gate")))) && 
				    ((!(userGate2.toString().equals("LINE \"gate\""))) &&
				     (!(userGate2.toString().equals("NOT gate")))) && 
				    ((!(userGate3.toString().equals("LINE \"gate\""))) &&
				     (!(userGate3.toString().equals("NOT gate"))))) {
				    
				     //all gates are placed and all notgate positions have NOt gates or LINE gates only
					return true;
				}
				else {		
					return false;//one of the user gates with two inputs has a one input gate placed
				}
			}
			else {
				return false;//user has been silly and put a non NOT|LINE gate in one of the places for it	
			}
		}
		else {
			return false;//user has not filled all of the positions
		}
	}
	
	/**
	 * Allows access to all of the gate locations users are allowed to drop into
	 * <p>
	 * For simplification the relative gate locations defined in the JPanel 
	 * that the gates are drawn onto were recreated here and not actually referenced.
	 * Hence these need to be manually updated if the actual locations change!
	 * 
	 * @return A list of all dropbox locations that are available for user placed gates
	 */
	public DropBoxList getDropBoxLocations() {
		DropBoxList list = new DropBoxList();
		
		//set up the locations for all of the boxes that are the drop target locations
		// - the figures come from the main JPanel where they are actually drawn on to it
    	int imageWidth  = 75;
    	int imageHeight = 50;

		int not1X1 = 40;
		int not1Y1 = 10;
	
		int not2X1 = not1X1+20;
		int not2Y1 = not1Y1+imageHeight;
		
		int not3X1 = not2X1;
		int not3Y1 = not2Y1+imageHeight;

		int not4X1 = not1X1;
		int not4Y1 = not3Y1+imageHeight;	
	
		int gate1X1 = not2X1+imageWidth;
		int gate1Y1 = not2Y1-7;
		
		int not5X1 = gate1X1+imageWidth;
		int not5Y1 = gate1Y1;
		
		int gate2X1 = not5X1+imageWidth;
		int gate2Y1 = not5Y1+7;		
		
		int not6X1 = gate2X1+imageWidth;
		int not6Y1 = gate2Y1;		
		
		int gate3X1 = not6X1+imageWidth;
		int gate3Y1 = not6Y1+imageHeight-7;
					
		list.addDropBoxtoList(new DropBox(new Point(gate1X1, gate1Y1), new Point(gate1X1+imageWidth, gate1Y1+imageHeight), "userGate1"));
		list.addDropBoxtoList(new DropBox(new Point(gate2X1, gate2Y1), new Point(gate2X1+imageWidth, gate2Y1+imageHeight), "userGate2"));
		list.addDropBoxtoList(new DropBox(new Point(gate3X1, gate3Y1), new Point(gate3X1+imageWidth, gate3Y1+imageHeight), "userGate3"));
		list.addDropBoxtoList(new DropBox(new Point(not1X1, not1Y1), new Point(not1X1+imageWidth, not1Y1+imageHeight), "userNotGate1"));
		list.addDropBoxtoList(new DropBox(new Point(not2X1, not2Y1), new Point(not2X1+imageWidth, not2Y1+imageHeight), "userNotGate2"));
		list.addDropBoxtoList(new DropBox(new Point(not3X1, not3Y1), new Point(not3X1+imageWidth, not3Y1+imageHeight), "userNotGate3"));
		list.addDropBoxtoList(new DropBox(new Point(not4X1, not4Y1), new Point(not4X1+imageWidth, not4Y1+imageHeight), "userNotGate4"));		
		list.addDropBoxtoList(new DropBox(new Point(not5X1, not5Y1), new Point(not5X1+imageWidth, not5Y1+imageHeight), "userNotGate5"));		
		list.addDropBoxtoList(new DropBox(new Point(not6X1, not6Y1), new Point(not6X1+imageWidth, not6Y1+imageHeight), "userNotGate6"));		
	
		return list;
	}

	/**
	 * Sets up the JPanel that contains the circuit
	 * 
	 * @param    hints   If true then hints are drawn on to the circuit otherwise they aren't
	 * @param    complete If false then the drop targets are highlighted, otherwise there are filled in 
	 *
	 * @return   CircuitDiagram - A diagram drawn on to a JPanel with or without hints based on the hints input
	 */
	public JPanel getCircuitDiagramPanel(boolean hints, boolean complete) {
		
		//these need to be final because we may need to access them inside
		//the paint method so that it a row can be painted
		final Object[][] answers = getTruthTableDetails();
		Random rand = new Random();
		//return a random int range 0 to length-1
		final int hintRow = rand.nextInt(answers.length);
		
		final boolean hintsOn = hints;
		final boolean completed = complete;
		setRevealedStatus(complete);//store this so we can do sensible output when revealing hints etc
		

		JPanel temp = new JPanel() {
			public void paintComponent(Graphics g) {
				super.paintComponent(g); //paint background

    			//set the image width and height
    			int imageWidth  = 75;
    			int imageHeight = 50;

	  			//need to create a standard box image that can be used as a blank target spacer
    			Image boxImage = createImage(imageWidth, imageHeight);
    			Graphics boxImg = boxImage.getGraphics();
				//make it the right colour of background
				boxImg.setColor(Color.lightGray);
				boxImg.fillRect(0,0,boxImage.getWidth(this),boxImage.getHeight(this));
	  			//fill the external border to make it stand out
	  			boxImg.setColor(Color.blue);
			  	boxImg.draw3DRect(1,1,imageWidth-6,imageHeight-4,true);
				//put on the big question marke
				boxImg.setColor(Color.black);
				boxImg.drawString("?",((imageWidth-6)/2),((imageHeight+6)/2));

				//all of the gate locations are set up as relative as possible
				//hence they need to be set up in the order in which they appear in the diagram

				//set up the first four not gates
     			int notGate1x = 40;
    			int notGate1y = 10;

    			int notGate2x = notGate1x+20;
    			int notGate2y = notGate1y+imageHeight;

    			int notGate3x = notGate2x;
    			int notGate3y = notGate2y+imageHeight;

    			int notGate4x = notGate1x;
    			int notGate4y = notGate3y+imageHeight;

    			//set the root locations of the 1st random gate
    			int gate1x = notGate2x+imageWidth;
    			int gate1y = notGate2y-7;

    			//set the location of the 5th not gate
    			int notGate5x = gate1x+imageWidth;
    			int notGate5y = gate1y;

   				//set the root location of the second random gate
    			int gate2x = notGate5x+imageWidth;
    			int gate2y = notGate5y+7;//7 is spacing to line it up

    			//set the location of the 6th not gate
    			int notGate6x = gate2x+imageWidth;
    			int notGate6y = gate2y;

 				//set the root location of the third random gate
    			int gate3x = notGate6x+imageWidth;
    			int gate3y = notGate6y+imageHeight-7;//10 is spacing

    			//set the background of the actual panel
    			g.setColor(Color.lightGray);
				g.fillRect(0,0,getWidth(),getHeight());
    			g.setColor(Color.black);
    			g.drawRect(0,0,getWidth()-2,getHeight()-2);

				//first paint on the three random gates
        		//get the image of the 1st gate after first painting the background of the image
    			Image image = createImage(imageWidth, imageHeight);
    			Graphics gImg = image.getGraphics();
				gImg.setColor(Color.lightGray);
				gImg.fillRect(0,0,image.getWidth(this), image.getHeight(this));

    			//draw random gate1 or blank box or user gate depending on the status
    			if (completed) {
    				image = actualGate1.getImageRepresentation(image, 10, 10, hintsOn);//get the actual image
        			g.drawImage(image, gate1x, gate1y, this);//Draw image at its natural size first (75x50 image);
				}
				else {
					if (userGate1==null) {
						g.drawImage(boxImage, gate1x, gate1y, this);
					}
					else {//the user has placed a gate in here so we must show that
						image = userGate1.getImageRepresentation(image, 10, 10, hintsOn);//get the actual image
        				
        				//fill the external border to make it stand out
	  					gImg.setColor(Color.gray);
					  	gImg.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
					  	
        				g.drawImage(image, gate1x, gate1y, this);//Draw image at its natural size first (75x50 image);
					}
				}
				
         		//get the image of the 2nd gate after first painting the background of the image
        		Image image2 = createImage(imageWidth, imageHeight);
    			Graphics gImg2 = image2.getGraphics();
    			gImg2.setColor(Color.lightGray);
				gImg2.fillRect(0,0,image2.getWidth(this),image2.getHeight(this));

    			//draw random gate2 or blank box or user gate depending on status
    			if (completed) {
    				image2 = actualGate2.getImageRepresentation(image2, 10, 10, hintsOn);
        			g.drawImage(image2, gate2x, gate2y, this);
				}
				else {
					if (userGate2==null) {
						g.drawImage(boxImage, gate2x, gate2y, this);
					}
					else {//the user has placed a gate in here so we must show that
    					image2 = userGate2.getImageRepresentation(image2, 10, 10, hintsOn);
    					
        				//fill the external border to make it stand out
	  					gImg2.setColor(Color.gray);
					  	gImg2.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);

        				g.drawImage(image2, gate2x, gate2y, this);
        			}						
				}
				
        		//get the image of the 3rd gate after first painting the background of the image
        		Image image3 = createImage(imageWidth, imageHeight);
    			Graphics gImg3 = image3.getGraphics();
    			gImg3.setColor(Color.lightGray);
				gImg3.fillRect(0,0,image3.getWidth(this),image3.getHeight(this));

    			//draw random gate3 or blank box or user gate depending on status
    			if (completed) {
    				image3 = actualGate3.getImageRepresentation(image3, 10, 10, hintsOn);
        			g.drawImage(image3, gate3x, gate3y, this);
				}
				else {
					if (userGate3==null) {
						g.drawImage(boxImage, gate3x, gate3y, this);
					}
					else {//the user has placed a gate in here so we must show that
    					image3 = userGate3.getImageRepresentation(image3, 10, 10, hintsOn);
    					
        				//fill the external border to make it stand out
	  					gImg3.setColor(Color.gray);
					  	gImg3.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);

        				g.drawImage(image3, gate3x, gate3y, this);
        			}
				}
				
				//next draw all of the connecters (with asumption the not gates don't exist
				//draw the connecters between a and gate 1
				int x1 = notGate1x;
				int y1 = notGate1y+25;
				int x2 = gate1x;
				int y2 = notGate1y+25;
				g.drawLine(x1,y1,x2,y2);//draws the line from a to top of the notgate1

				x1 = gate1x;
				y1 = notGate1y+25;
				x2 = gate1x;
				y2 = notGate1y+60;
				g.drawLine(x1,y1,x2,y2);//draws the line from a to the connecter of notgate1

				//draws the connecter from gate1 to gate2 assuming no inverter gate
				x1 = gate1x+imageWidth;
				y1 = gate1y+25;
				x2 = gate2x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//draws the line from a to the connecter of notgate1

				//draw the connecters from b to the split line
				x1 = notGate2x-20;
				y1 = notGate2y+imageWidth-25;
				x2 = notGate2x;
				y2 = notGate2y+imageWidth-25;
				g.drawLine(x1,y1,x2,y2);//draws the line from b to the split point

				//draw the connecters from split line point to the two not gates 2 and 3
				x1 = notGate2x-1;//the minus 1 stops it being overwritten by the notgate
				y1 = notGate2y+25;
				x2 = notGate3x-1;
				y2 = notGate3y+25;
				g.drawLine(x1,y1,x2,y2);//draws the line from the split point to not gates 2 and 3

				//draw the connecters from top split line point to the not gate 2
				x1 = notGate2x-1;
				y1 = notGate2y+25;
				x2 = gate1x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//draws the line from the split point to not gate 2

				x1 = notGate3x;
				y1 = notGate3y+25;
				x2 = gate3x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//draws the bottom line split from b to gate 3

				//draws the line from c to gate 2
				x1 = notGate4x;
				y1 = notGate4y+25;
				x2 = gate2x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//draws the line from c to below gate2

				x1 = gate2x;
				y1 = gate2y+32;
				x2 = x1;
				y2 = notGate4y+25;
				g.drawLine(x1,y1,x2,y2);//draws the line from below gate2 to gate2

				//draws the line from notgate5 (D) to the top of gate 3
				x1 = gate2x+imageWidth;
				y1 = gate2y+25;
				x2 = gate3x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//draws the line from notgate5 to top of gate 3

				x1 = gate3x;
				y1 = gate2y+25;
				x2 = x1;
				y2 = gate3y+10+8;
				g.drawLine(x1,y1,x2,y2);//draws the line from top of gate 3	to connecter of gate 3

				//next draw the not gates if they have been set for this particular diagram
        		Image notImage1 = createImage(imageWidth, imageHeight);
    			Graphics gNotImg1 = notImage1.getGraphics();
    			gNotImg1.setColor(Color.lightGray);
				gNotImg1.fillRect(0,0,notImage1.getWidth(this),notImage1.getHeight(this));

        		if (completed) {//need to draw the notGate1 properly or just a normal line through
        			if (notGate1Exists) {      				
    					//draw not gate	1
    					notImage1 = actualNotGate1.getImageRepresentation(notImage1, 10, 10, hintsOn);
        				g.drawImage(notImage1, notGate1x, notGate1y, this);        				
        			}
        		}
        		else {//need to draw the blank box or the user placed gate
        			if (userNotGate1 == null) { //draw the blank box
        				g.drawImage(boxImage, notGate1x, notGate1y, this);
        			}
        			else {//draw the gate the user thinks should be here
    					notImage1 = userNotGate1.getImageRepresentation(notImage1, 10, 10, hintsOn);
        				
         				//fill the external border to make it stand out
	  					gNotImg1.setColor(Color.gray);
					  	gNotImg1.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
        				
        				g.drawImage(notImage1, notGate1x, notGate1y, this);
        			}
        		}

       			//get the image of the 2nd not gate after first painting the background of the image
       			Image notImage2 = createImage(imageWidth, imageHeight);
   				Graphics gNotImg2 = notImage2.getGraphics();
   				gNotImg2.setColor(Color.lightGray);
				gNotImg2.fillRect(0,0,notImage2.getWidth(this),notImage2.getHeight(this));

        		if (completed) {//need to draw the notGate2 properly or just a normal line through
        			if (notGate2Exists) {
    					//draw not gate	2
    					notImage2 = actualNotGate2.getImageRepresentation(notImage2, 10, 10, hintsOn);
        				g.drawImage(notImage2, notGate2x, notGate2y, this);        				
        			}
        		}
        		else {//need to draw the blank box or the user placed gate
        			if (userNotGate2 == null) { //draw the blank box
        				g.drawImage(boxImage, notGate2x, notGate2y, this);
        			}
        			else {//draw the gate the user thinks should be here
    					notImage2 = userNotGate2.getImageRepresentation(notImage2, 10, 10, hintsOn);
        				
         				//fill the external border to make it stand out
	  					gNotImg2.setColor(Color.gray);
					  	gNotImg2.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
        				
        				g.drawImage(notImage2, notGate2x, notGate2y, this);
        			}
        		}

       			Image notImage3 = createImage(imageWidth, imageHeight);
   				Graphics gNotImg3 = notImage3.getGraphics();
    			gNotImg3.setColor(Color.lightGray);
				gNotImg3.fillRect(0,0,notImage3.getWidth(this),notImage3.getHeight(this));

        		if (completed) {//need to draw the notGate3 properly or just a normal line through
        			if (notGate3Exists) {
    					//draw not gate	3
    					notImage3 = actualNotGate3.getImageRepresentation(notImage3, 10, 10, hintsOn);
        				g.drawImage(notImage3, notGate3x, notGate3y, this);        				
        			}
        		}
        		else {//need to draw the blank box or the user placed gate
        			if (userNotGate3 == null) { //draw the blank box
        				g.drawImage(boxImage, notGate3x, notGate3y, this);
        			}
        			else {//draw the gate the user thinks should be here
    					notImage3 = userNotGate3.getImageRepresentation(notImage3, 10, 10, hintsOn);
        				
         				//fill the external border to make it stand out
	  					gNotImg3.setColor(Color.gray);
					  	gNotImg3.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
        				
        				g.drawImage(notImage3, notGate3x, notGate3y, this);
        			}
        		}

       			Image notImage4 = createImage(imageWidth, imageHeight);
   				Graphics gNotImg4 = notImage4.getGraphics();
   				gNotImg4.setColor(Color.lightGray);
				gNotImg4.fillRect(0, 0, notImage4.getWidth(this), notImage4.getHeight(this));

        		if (completed) {//need to draw the notGate4 properly or just a normal line through
        			if (notGate4Exists) {
    					//draw not gate	4
    					notImage4 = actualNotGate4.getImageRepresentation(notImage4, 10, 10, hintsOn);
        				g.drawImage(notImage4, notGate4x, notGate4y, this);        				
        			}
        		}
        		else {//need to draw the blank box or the user placed gate
        			if (userNotGate4 == null) { //draw the blank box
        				g.drawImage(boxImage, notGate4x, notGate4y, this);
        			}
        			else {//draw the gate the user thinks should be here
    					notImage4 = userNotGate4.getImageRepresentation(notImage4, 10, 10, hintsOn);
        				
         				//fill the external border to make it stand out
	  					gNotImg4.setColor(Color.gray);
					  	gNotImg4.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
        				
        				g.drawImage(notImage4, notGate4x, notGate4y, this);
        			}
        		}

        		Image notImage5 = createImage(imageWidth, imageHeight);
    			Graphics gNotImg5 = notImage5.getGraphics();
    			gNotImg5.setColor(Color.lightGray);
				gNotImg5.fillRect(0,0,notImage5.getWidth(this),notImage5.getHeight(this));

        		if (completed) {//need to draw the notGate5 properly or just a normal line through
        			if (notGate5Exists) {
    					//draw not gate	1
    					notImage5 = actualNotGate5.getImageRepresentation(notImage5, 10, 10, hintsOn);
        				g.drawImage(notImage5, notGate5x, notGate5y, this);        				
        			}
        		}
        		else {//need to draw the blank box or the user placed gate
        			if (userNotGate5 == null) { //draw the blank box
        				g.drawImage(boxImage, notGate5x, notGate5y, this);
        			}
        			else {//draw the gate the user thinks should be here
    					notImage5 = userNotGate5.getImageRepresentation(notImage5, 10, 10, hintsOn);
        				
         				//fill the external border to make it stand out
	  					gNotImg5.setColor(Color.gray);
					  	gNotImg5.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
        				
        				g.drawImage(notImage5, notGate5x, notGate5y, this);
        			}
        		}
    
       			Image notImage6 = createImage(imageWidth, imageHeight);
    			Graphics gNotImg6 = notImage6.getGraphics();
    			gNotImg6.setColor(Color.lightGray);
				gNotImg6.fillRect(0, 0, notImage6.getWidth(this), notImage6.getHeight(this));

        		if (completed) {//need to draw the notGate6 properly or just a normal line through
        			if (notGate6Exists) {
    					//draw not gate	6
    					notImage6 = actualNotGate6.getImageRepresentation(notImage6, 10, 10, hintsOn);
        				g.drawImage(notImage6, notGate6x, notGate6y, this);        				
        			}
        		}
        		else {//need to draw the blank box or the user placed gate
        			if (userNotGate6 == null) { //draw the blank box
        				g.drawImage(boxImage, notGate6x, notGate6y, this);
        			}
        			else {//draw the gate the user thinks should be here
    					notImage6 = userNotGate6.getImageRepresentation(notImage6, 10, 10, hintsOn);
        				
         				//fill the external border to make it stand out
	  					gNotImg6.setColor(Color.gray);
					  	gNotImg6.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
        				
        				g.drawImage(notImage6, notGate6x, notGate6y, this);
        			}
        		}

        		//the split connecter of the b input
        		g.fillOval(notGate2x-5, notGate2y+47,8,8);

        		//next draw the text on to the circuit
        		g.drawString(aLabel, notGate1x-15, notGate1y+28);
    			g.drawString(bLabel, notGate1x-15, gate2y+55);
    			g.drawString(cLabel, notGate4x-15, notGate4y+28);
    			g.drawString(dLabel, gate2x-15, gate2y+10);
    			g.drawString(eLabel, gate3x+10, gate3y+6);
    			g.drawString(outLabel, gate3x+imageWidth+5, gate3y+30);

				if (hintsOn) {
					//paint the row of the correct answers numbers that was selected previously on to the panel
					//known outputs
					g.drawString(answers[hintRow][0].toString(), notGate1x-25, notGate1y+28);//a
					g.drawString(answers[hintRow][1].toString(), notGate1x-25, gate2y+55);//b
					g.drawString(answers[hintRow][2].toString(), notGate4x-25, notGate4y+28);//c

					g.drawString(answers[hintRow][3].toString(), gate2x-15, gate2y-10);//d
					g.drawString(answers[hintRow][4].toString(), gate3x-10, gate3y+6);//e
					g.drawString(answers[hintRow][5].toString(), gate3x+imageWidth+60, gate3y+30);//output

					//after inverters outputs (if necessary)
					if(notGate1Exists) {
						g.drawString(Integer.toString(actualNotGate1.getOutput(Integer.parseInt((answers[hintRow][0].toString())))), notGate1x+imageWidth-15, notGate1y+20);
					}

					if(notGate2Exists) {
						g.drawString(Integer.toString(actualNotGate2.getOutput(Integer.parseInt((answers[hintRow][1].toString())))), notGate2x+imageWidth-15, notGate2y+40);
					}

					if(notGate3Exists) {
						g.drawString(Integer.toString(actualNotGate3.getOutput(Integer.parseInt((answers[hintRow][1].toString())))), notGate3x+imageWidth-15, notGate3y+20);
					}

					if(notGate4Exists) {//the d value in the answer is already displayed so want the value before the not gate
						g.drawString(Integer.toString(actualNotGate4.getOutput(Integer.parseInt((answers[hintRow][2].toString())))), notGate4x+imageWidth-15, notGate4y+20);
					}

					if(notGate5Exists) {
						g.drawString(Integer.toString(actualNotGate5.getOutput(Integer.parseInt((answers[hintRow][3].toString())))), notGate5x-5, notGate5y+20);
					}

					if(notGate6Exists) {//the d value in the answer is already displayed so want the value before the not gate
						g.drawString(Integer.toString(actualNotGate6.getOutput(Integer.parseInt((answers[hintRow][4].toString())))), notGate6x-5, notGate6y+20);
					}
				}//end hintsOn
			}
		};
		dropTarget = new DropTarget(temp, new DTListener(this));
		return temp;
	}
}