import java.awt.*;
import java.lang.*;
import java.util.Random;
import javax.swing.JPanel;
import java.awt.dnd.*;
import java.awt.datatransfer.*;

/**
 * LogicCircuits.java 1.0 03/02/01
 * <br>
 * Author       : James McMahon
 * <br>
 * Implements the Circuit for a five gate approach of the following format.
 * <br>
 * <pre>
 * 
 * A---N1---|----------------------------|
 *          |                            |
 *          |---|                        |
 *              G1---N5---E---|          G4---H---|
 * B---N2-------|             |          |        |
 *                            G3--N7--G--|        |
 * C---N3-------|             |                   G5---N8---Output
 *              G2---N6---F---|                   |
 *              |                                 |
 * D---N4-------|-----------------------------I---|
 *
 * </pre>
 * <br>
 * where G's are logic gates, N's are inverters and A, B, C, D are inputs.
 * E, F, G, H and I are intermediate stages, and each gate is a drop location target
 * <br>
 * Created      : 1/02/2003
 * <br>
 * Last Updated : 1/02/2003
 */
 
public class DropCircuit5Gates extends DropTargetCircuit{

	//the labels for the inputs and outputs
	private String aLabel = "A";
	private String bLabel = "B";
	private String cLabel = "C";
    private String dLabel = "D";

    //intermediate inputs
	private String eLabel = "E";
	private String fLabel = "F";
	private String gLabel = "G";
    private String hLabel = "H";
    private String iLabel = "I";
    
	private String outLabel = "OUTPUT";
	
	private String[] truthTableHeaders = {"A","B","C","D","E","F","G","H","I","Output"};

	//the number of columns for the truthtable that the user shouldn't be able to modify
	//equivalent to the first x columns
	private int numberOfFixedColumns = 10;

	//allows access to gates
	private LogicGates logicGates;

	//objects for all of the possible gates that can be held in the circuit
	//namely the three gates plus the 8 possible not gates
	private LogicGate actualGate1;
	private LogicGate actualGate2;
	private LogicGate actualGate3;
	private LogicGate actualGate4;
	private LogicGate actualGate5;
	
	private LogicGate actualNotGate1;
	private LogicGate actualNotGate2;
	private LogicGate actualNotGate3;
	private LogicGate actualNotGate4;
	private LogicGate actualNotGate5;
	private LogicGate actualNotGate6;
	private LogicGate actualNotGate7;
	private LogicGate actualNotGate8;

	//booleans to decide if the not gates need to be taken into account
	private boolean notGate1Exists = false;
	private boolean notGate2Exists = false;
	private boolean notGate3Exists = false;
	private boolean notGate4Exists = false;
	private boolean notGate5Exists = false;
	private boolean notGate6Exists = false;
	private boolean notGate7Exists = false;
	private boolean notGate8Exists = false;

	//objects for all of the possible gates that can be placed onto the circuit by the user
	//namely the three gates plus the 8 possible not gates
	private LogicGate userGate1;
	private LogicGate userGate2;
	private LogicGate userGate3;
	private LogicGate userGate4;
	private LogicGate userGate5;
	
	private LogicGate userNotGate1;
	private LogicGate userNotGate2;
	private LogicGate userNotGate3;
	private LogicGate userNotGate4;
	private LogicGate userNotGate5;
	private LogicGate userNotGate6;
	private LogicGate userNotGate7;
	private LogicGate userNotGate8;

	//Allows access to the parent so a repaint can be accessed
	private QuestionGenerator parent;

	/**
	 * Set's up a random circuit that can be queryed using the methods provided
	 * 
	 * @param   logicComponents   The logicGates object that random gates can be drawn from
	 */
	public DropCircuit5Gates(LogicGates logicComponents, QuestionGenerator parentOwner) {
		logicGates = logicComponents;
		parent = parentOwner;

		//get five random gates for the circuit structure
		actualGate1 = logicGates.getRandomLogicGate();
		actualGate2 = logicGates.getRandomLogicGate();
		actualGate3 = logicGates.getRandomLogicGate();
		actualGate4 = logicGates.getRandomLogicGate();
		actualGate5 = logicGates.getRandomLogicGate();

		//decide randomly on the allocation of not gates

		//random number generator
		Random randNum = new Random();

		if (randNum.nextBoolean()) {
			actualNotGate1 = logicGates.getNotGate();
			notGate1Exists = true;
		} else {
			notGate1Exists = false;
		}

		if (randNum.nextBoolean()) {
			actualNotGate2 = logicGates.getNotGate();
			notGate2Exists = true;
		} else {
			notGate2Exists = false;
		}

		if (randNum.nextBoolean()) {
			actualNotGate3 = logicGates.getNotGate();
			notGate3Exists = true;
		} else {
			notGate3Exists = false;
		}

		if (randNum.nextBoolean()) {
			actualNotGate4 = logicGates.getNotGate();
			notGate4Exists = true;
		} else {
 			notGate4Exists = false;
		}

		if (randNum.nextBoolean()) {
			actualNotGate5 = logicGates.getNotGate();
			notGate5Exists = true;
		} else {
			notGate5Exists = false;
		}

		if (randNum.nextBoolean()) {
			actualNotGate6 = logicGates.getNotGate();
			notGate6Exists = true;
		} else {
			notGate6Exists = false;
		}

		if (randNum.nextBoolean()) {
			actualNotGate7 = logicGates.getNotGate();
			notGate7Exists = true;
		} else {
			notGate7Exists = false;
		}

		if (randNum.nextBoolean()) {
			actualNotGate8 = logicGates.getNotGate();
			notGate8Exists = true;
		} else {
			notGate8Exists = false;
		}
	}

	/**
	 * Returns the data that is displayed in the truth table with question marks where user input is expected.
	 * 
	 * @return TruthTableDetails which are the actual contents of the default table
	 */
	public Object[][] getTruthTableDetails() {
		Object[][] correctAnswerDetails = {{"0","0","0","0", getEOutput(0,0,0,0), getFOutput(0,0,0,0), getGOutput(0,0,0,0), getHOutput(0,0,0,0), getIOutput(0,0,0,0), getOutput(0,0,0,0)},
										   {"0","0","0","1", getEOutput(0,0,0,1), getFOutput(0,0,0,1), getGOutput(0,0,0,1), getHOutput(0,0,0,1), getIOutput(0,0,0,1), getOutput(0,0,0,1)},
							    		   {"0","0","1","0", getEOutput(0,0,1,0), getFOutput(0,0,1,0), getGOutput(0,0,1,0), getHOutput(0,0,1,0), getIOutput(0,0,1,0), getOutput(0,0,1,0)},
							    		   {"0","0","1","1", getEOutput(0,0,1,1), getFOutput(0,0,1,1), getGOutput(0,0,1,1), getHOutput(0,0,1,1), getIOutput(0,0,1,1), getOutput(0,0,1,1)},
							    		   {"0","1","0","0", getEOutput(0,1,0,0), getFOutput(0,1,0,0), getGOutput(0,1,0,0), getHOutput(0,1,0,0), getIOutput(0,1,0,0), getOutput(0,1,0,0)},
							  			   {"0","1","0","1", getEOutput(0,1,0,1), getFOutput(0,1,0,1), getGOutput(0,1,0,1), getHOutput(0,1,0,1), getIOutput(0,1,0,1), getOutput(0,1,0,1)},
							    		   {"0","1","1","0", getEOutput(0,1,1,0), getFOutput(0,1,1,0), getGOutput(0,1,1,0), getHOutput(0,1,1,0), getIOutput(0,1,1,0), getOutput(0,1,1,0)},
							    		   {"0","1","1","1", getEOutput(0,1,1,1), getFOutput(0,1,1,1), getGOutput(0,1,1,1), getHOutput(0,1,1,1), getIOutput(0,1,1,1), getOutput(0,1,1,1)},
							    		   {"1","0","0","0", getEOutput(1,0,0,0), getFOutput(1,0,0,0), getGOutput(1,0,0,0), getHOutput(1,0,0,0), getIOutput(1,0,0,0), getOutput(1,0,0,0)},
										   {"1","0","0","1", getEOutput(1,0,0,1), getFOutput(1,0,0,1), getGOutput(1,0,0,1), getHOutput(1,0,0,1), getIOutput(1,0,0,1), getOutput(1,0,0,1)},
							    		   {"1","0","1","0", getEOutput(1,0,1,0), getFOutput(1,0,1,0), getGOutput(1,0,1,0), getHOutput(1,0,1,0), getIOutput(1,0,1,0), getOutput(1,0,1,0)},
							    		   {"1","0","1","1", getEOutput(1,0,1,1), getFOutput(1,0,1,1), getGOutput(1,0,1,1), getHOutput(1,0,1,1), getIOutput(1,0,1,1), getOutput(1,0,1,1)},
							    		   {"1","1","0","0", getEOutput(1,1,0,0), getFOutput(1,1,0,0), getGOutput(1,1,0,0), getHOutput(1,1,0,0), getIOutput(1,1,0,0), getOutput(1,1,0,0)},
							  			   {"1","1","0","1", getEOutput(1,1,0,1), getFOutput(1,1,0,1), getGOutput(1,1,0,1), getHOutput(1,1,0,1), getIOutput(1,1,0,1), getOutput(1,1,0,1)},
							    		   {"1","1","1","0", getEOutput(1,1,1,0), getFOutput(1,1,1,0), getGOutput(1,1,1,0), getHOutput(1,1,1,0), getIOutput(1,1,1,0), getOutput(1,1,1,0)},
							    		   {"1","1","1","1", getEOutput(1,1,1,1), getFOutput(1,1,1,1), getGOutput(1,1,1,1), getHOutput(1,1,1,1), getIOutput(1,1,1,1), getOutput(1,1,1,1)}};
		return correctAnswerDetails;
	}

	/**
	 * returns the ouput at E given a b c and d input conditions
	 * <br>
	 * (in reality with this circuit only a and b actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * @param    d   The input at point d
	 * 
	 * @return   0 If 0  is the output at the E location
	 */
	private String getEOutput(int a, int b, int c, int d) {
		
		if (notGate1Exists) {//invert a
			a = actualNotGate1.getOutput(a);
		}

		if (notGate2Exists) {//invert b
			b = actualNotGate2.getOutput(b);
		}

		int temp = actualGate1.getOutput(a,b);

		if (notGate5Exists) {//then need to invert temp as well now
			temp = actualNotGate5.getOutput(temp);
		}
		
		return Integer.toString(temp);
	}

	/**
	 * returns the ouput at F given a b c and d input conditions
	 * <br>
	 * (in reality with this circuit only c and d actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * @param    d   The input at point d
	 * 
	 * @return   0 If 0  is the output at the F location
	 */
	private String getFOutput(int a, int b, int c, int d) {
		
		if (notGate3Exists) {//invert c
			c = actualNotGate3.getOutput(c);
		}

		if (notGate4Exists) {//invert d
			d = actualNotGate4.getOutput(d);
		}

		int temp = actualGate2.getOutput(c,d);

		if (notGate6Exists) {//need to invert the gate2 output as well
			temp = actualNotGate6.getOutput(temp);
		}
		
		return Integer.toString(temp);
	}

	/**
	 * returns the ouput at G given a b c and d input conditions
	 * <br>
	 * (in reality with this circuit only a, b and c actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * @param    d   The input at point d
	 * 
	 * @return   0 If 0  is the output at the G location
	 */
	private String getGOutput(int a, int b, int c, int d) {
		
		String eOut = getEOutput(a,b,c,d);
		String fOut = getFOutput(a,b,c,d);

		int e = Integer.parseInt(eOut);
		int f = Integer.parseInt(fOut);

		int temp = actualGate3.getOutput(e,f);

		if (notGate7Exists) {//need to invert the f output
			temp = actualNotGate7.getOutput(temp);
		}
		
		return Integer.toString(temp);
	}

	/**
	 * returns the ouput at H given a b c and d input conditions
	 * <br>
	 * (in reality with this circuit only a, b and c actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * @param    d   The input at point d
	 * 
	 * @return   0 If 0  is the output at the H location
	 */
	private String getHOutput(int a, int b, int c, int d) {
		
		String gOut = getGOutput(a,b,c,d);

		int g = Integer.parseInt(gOut);
		
		if (notGate1Exists) {//need to invert the a input
			a = actualNotGate1.getOutput(a);
		}

		int temp = actualGate4.getOutput(a, g);
		
		return Integer.toString(temp);
	}	
	/**
	 * returns the ouput at i given a b c and d input conditions
	 * <br>
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * @param    d   The input at point d
	 * 
	 * @return   0 If 0  is the output at the H location
	 */
	private String getIOutput(int a, int b, int c, int d) {
		
		if (notGate4Exists) {//need to invert the d input
			d = actualNotGate4.getOutput(d);
		}
		
		return Integer.toString(d);
	}
	
	/**
	 * Returns the final output of the circuit given the four initial starting conditions.
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * @param    d   The input at point d
	 * 
	 * @return   0 If 0 is the output of the full circuit
	 */
	private String getOutput(int a, int b, int c, int d) {
		
		String hOut = getHOutput(a,b,c,d);
		String iOut = getIOutput(a,b,c,d);

		int h = Integer.parseInt(hOut);
		int i = Integer.parseInt(iOut);

		int temp = actualGate5.getOutput(h, i);
		
		if (notGate8Exists) {
			temp = actualNotGate8.getOutput(temp);
		}

		return Integer.toString(temp);
	}

	/**
	 * returns the ouput at E given a b c and d input conditions
	 * <br>
	 * (in reality with this circuit only a and b actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * @param    d   The input at point d
	 * 
	 * @return   0 If 0  is the output at the E location
	 */
	private String getUserBasedEOutput(int a, int b, int c, int d) {
		try {
			//invert or propagate depending on the user placed gates at this location
			a = userNotGate1.getOutput(a);
			b = userNotGate2.getOutput(b);

			int temp = userGate1.getOutput(a,b);

			temp = userNotGate5.getOutput(temp);
				
			return Integer.toString(temp);
		}
		catch(NullPointerException npe) {
			return "-1";
		}
	}

	/**
	 * returns the ouput at F given a b c and d input conditions
	 * <br>
	 * (in reality with this circuit only c and d actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * @param    d   The input at point d
	 * 
	 * @return   0 If 0  is the output at the F location
	 */
	private String getUserBasedFOutput(int a, int b, int c, int d) {
		try {
			//invert or propagate depending on the user placed gates at this location
			c = userNotGate3.getOutput(c);
			d = userNotGate4.getOutput(d);
		
			int temp = userGate2.getOutput(c,d);
			temp = userNotGate6.getOutput(temp);
		
			return Integer.toString(temp);
		}
		catch(NullPointerException npe) {
			return "-1";
		}
	}

	/**
	 * returns the ouput at G given a b c and d input conditions
	 * <br>
	 * (in reality with this circuit only a, b and c actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * @param    d   The input at point d
	 * 
	 * @return   0 If 0  is the output at the G location
	 */
	private String getUserBasedGOutput(int a, int b, int c, int d) {
		try {
			//invert or propagate depending on the user placed gates at this location		
			String eOut = getUserBasedEOutput(a,b,c,d);
			String fOut = getUserBasedFOutput(a,b,c,d);

			int e = Integer.parseInt(eOut);
			int f = Integer.parseInt(fOut);

			int temp = userGate3.getOutput(e,f);
			temp = userNotGate7.getOutput(temp);
		
			return Integer.toString(temp);
		}
		catch(NullPointerException npe) {
			return "-1";
		}
	}

	/**
	 * returns the ouput at H given a b c and d input conditions
	 * <br>
	 * (in reality with this circuit only a, b and c actually affect it)
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * @param    d   The input at point d
	 * 
	 * @return   0 If 0  is the output at the H location
	 */
	private String getUserBasedHOutput(int a, int b, int c, int d) {
		try {
			//invert or propagate depending on the user placed gates at this location	
			String gOut = getUserBasedGOutput(a,b,c,d);
			int g = Integer.parseInt(gOut);
		
			a = userNotGate1.getOutput(a);

			int temp = userGate4.getOutput(a, g);
		
			return Integer.toString(temp);
		}
		catch(NullPointerException npe) {
			return "-1";
		}
	}
		
	/**
	 * returns the ouput at i given a b c and d input conditions
	 * <br>
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * @param    d   The input at point d
	 * 
	 * @return   0 If 0  is the output at the H location
	 */
	private String getUserBasedIOutput(int a, int b, int c, int d) {
		try {
			//invert or propagate depending on the user placed gates at this location		
			d = userNotGate4.getOutput(d);
			return Integer.toString(d);
		}	
		catch(NullPointerException npe) {
			return "-1";
		}
	}
	
	/**
	 * Returns the final output of the circuit given the four initial starting conditions.
	 * 
	 * @param    a   The input at point a
	 * @param    b   The input at point b
	 * @param    c   The input at point c
	 * @param    d   The input at point d
	 * 
	 * @return   0 If 0 is the output of the full circuit
	 */
	private String getUserBasedOutput(int a, int b, int c, int d) {
		try {
			//invert or propagate depending on the user placed gates at this location
			
			String hOut = getUserBasedHOutput(a,b,c,d);
			String iOut = getUserBasedIOutput(a,b,c,d);

			int h = Integer.parseInt(hOut);
			int i = Integer.parseInt(iOut);

			int temp = userGate5.getOutput(h, i);
		
			temp = userNotGate8.getOutput(temp);

			return Integer.toString(temp);
		}
		catch(NullPointerException npe) {
			return "-1";
		}
	}

	/**
	 * Returns the headings for the truth table (such as A, B, C, Output etc).
	 * 
	 * @return   Headings The headings of the truthtable that can be used to instantiate a truthtable object
	 */
	public String[] getTruthTableHeadings() {
		return truthTableHeaders;
	}
	
	/**
	 * Returns the number of fixed columns that the user shouldn't be able to modify
	 * 
	 * @return the number of columns to be fixed from the far left
	 */
	public int getNumberOfFixedColumns() {
		return this.numberOfFixedColumns;
	}

	/**
	 * gives access to whatever truthtable the gates that the user has placed creates
	 * <p>
	 * N.B. It is important that the gates are checked for correspondance to the actual
	 * truth table details - not just residual information regarding the gates which were
	 * intially used to set up the question! (as more than one setup could cause the same answers)
	 *
	 * @return The truthtable details corresponding to the user built circuit
	 */
	public Object[][] getUserTruthTable() {
		Object[][] userCreatedTruthTable = {{"0","0","0","0", getUserBasedEOutput(0,0,0,0), getUserBasedFOutput(0,0,0,0), getUserBasedGOutput(0,0,0,0), getUserBasedHOutput(0,0,0,0), getUserBasedIOutput(0,0,0,0), getUserBasedOutput(0,0,0,0)},
										    {"0","0","0","1", getUserBasedEOutput(0,0,0,1), getUserBasedFOutput(0,0,0,1), getUserBasedGOutput(0,0,0,1), getUserBasedHOutput(0,0,0,1), getUserBasedIOutput(0,0,0,1), getUserBasedOutput(0,0,0,1)},
							    		    {"0","0","1","0", getUserBasedEOutput(0,0,1,0), getUserBasedFOutput(0,0,1,0), getUserBasedGOutput(0,0,1,0), getUserBasedHOutput(0,0,1,0), getUserBasedIOutput(0,0,1,0), getUserBasedOutput(0,0,1,0)},
							    		    {"0","0","1","1", getUserBasedEOutput(0,0,1,1), getUserBasedFOutput(0,0,1,1), getUserBasedGOutput(0,0,1,1), getUserBasedHOutput(0,0,1,1), getUserBasedIOutput(0,0,1,1), getUserBasedOutput(0,0,1,1)},
							    		    {"0","1","0","0", getUserBasedEOutput(0,1,0,0), getUserBasedFOutput(0,1,0,0), getUserBasedGOutput(0,1,0,0), getUserBasedHOutput(0,1,0,0), getUserBasedIOutput(0,1,0,0), getUserBasedOutput(0,1,0,0)},
							  			    {"0","1","0","1", getUserBasedEOutput(0,1,0,1), getUserBasedFOutput(0,1,0,1), getUserBasedGOutput(0,1,0,1), getUserBasedHOutput(0,1,0,1), getUserBasedIOutput(0,1,0,1), getUserBasedOutput(0,1,0,1)},
							    		    {"0","1","1","0", getUserBasedEOutput(0,1,1,0), getUserBasedFOutput(0,1,1,0), getUserBasedGOutput(0,1,1,0), getUserBasedHOutput(0,1,1,0), getUserBasedIOutput(0,1,1,0), getUserBasedOutput(0,1,1,0)},
							    		    {"0","1","1","1", getUserBasedEOutput(0,1,1,1), getUserBasedFOutput(0,1,1,1), getUserBasedGOutput(0,1,1,1), getUserBasedHOutput(0,1,1,1), getUserBasedIOutput(0,1,1,1), getUserBasedOutput(0,1,1,1)},
							    		    {"1","0","0","0", getUserBasedEOutput(1,0,0,0), getUserBasedFOutput(1,0,0,0), getUserBasedGOutput(1,0,0,0), getUserBasedHOutput(1,0,0,0), getUserBasedIOutput(1,0,0,0), getUserBasedOutput(1,0,0,0)},
										    {"1","0","0","1", getUserBasedEOutput(1,0,0,1), getUserBasedFOutput(1,0,0,1), getUserBasedGOutput(1,0,0,1), getUserBasedHOutput(1,0,0,1), getUserBasedIOutput(1,0,0,1), getUserBasedOutput(1,0,0,1)},
							    		    {"1","0","1","0", getUserBasedEOutput(1,0,1,0), getUserBasedFOutput(1,0,1,0), getUserBasedGOutput(1,0,1,0), getUserBasedHOutput(1,0,1,0), getUserBasedIOutput(1,0,1,0), getUserBasedOutput(1,0,1,0)},
							    		    {"1","0","1","1", getUserBasedEOutput(1,0,1,1), getUserBasedFOutput(1,0,1,1), getUserBasedGOutput(1,0,1,1), getUserBasedHOutput(1,0,1,1), getUserBasedIOutput(1,0,1,1), getUserBasedOutput(1,0,1,1)},
							    		    {"1","1","0","0", getUserBasedEOutput(1,1,0,0), getUserBasedFOutput(1,1,0,0), getUserBasedGOutput(1,1,0,0), getUserBasedHOutput(1,1,0,0), getUserBasedIOutput(1,1,0,0), getUserBasedOutput(1,1,0,0)},
							  			    {"1","1","0","1", getUserBasedEOutput(1,1,0,1), getUserBasedFOutput(1,1,0,1), getUserBasedGOutput(1,1,0,1), getUserBasedHOutput(1,1,0,1), getUserBasedIOutput(1,1,0,1), getUserBasedOutput(1,1,0,1)},
							    		    {"1","1","1","0", getUserBasedEOutput(1,1,1,0), getUserBasedFOutput(1,1,1,0), getUserBasedGOutput(1,1,1,0), getUserBasedHOutput(1,1,1,0), getUserBasedIOutput(1,1,1,0), getUserBasedOutput(1,1,1,0)},
							    		    {"1","1","1","1", getUserBasedEOutput(1,1,1,1), getUserBasedFOutput(1,1,1,1), getUserBasedGOutput(1,1,1,1), getUserBasedHOutput(1,1,1,1), getUserBasedIOutput(1,1,1,1), getUserBasedOutput(1,1,1,1)}};
		return userCreatedTruthTable;
	}

	/**
	 * redraws the screen when necessary
	 */
	public void redrawScreen() {
		parent.repaintScreen();	
	}	

	/**
	 * Deals with a dropped gate at location xLoc,yLoc.
	 * If the location of the drop corresponds to a valid gate site then
	 * the gate is set to that site and should be displayed.
	 *
	 * @param gate The gate that was dropped
	 * @param location The point that it was dropped at
	 */
	public void dealWithDrop(LogicGate gate, Point location) {
	
		DropBox b = this.getDropBoxLocations().getDropBoxContainingPoint(location);
		if (b!=null) {
			String gateName = b.toString();
			if (gateName.equals("userGate1")) {
				userGate1 = gate;		
			}
			else if (gateName.equals("userGate2")) {
				userGate2 = gate;
			}
			else if (gateName.equals("userGate3")) {
				userGate3 = gate;
			}
			else if (gateName.equals("userGate4")) {
				userGate4 = gate;
			}	
			else if (gateName.equals("userGate5")) {
				userGate5 = gate;
			}		
			else if (gateName.equals("userNotGate1")) {
				userNotGate1 = gate;
			}
			else if (gateName.equals("userNotGate2")) {
				userNotGate2 = gate;
			}
			else if (gateName.equals("userNotGate3")) {
				userNotGate3 = gate;
			}
			else if (gateName.equals("userNotGate4")) {
				userNotGate4 = gate;
			}
			else if (gateName.equals("userNotGate5")) {
				userNotGate5 = gate;
			}
			else if (gateName.equals("userNotGate6")) {
				userNotGate6 = gate;
			}
			else if (gateName.equals("userNotGate7")) {
				userNotGate7 = gate;
			}
			else if (gateName.equals("userNotGate8")) {
				userNotGate8 = gate;
			}
		}
	}

	/**
	 * Ensures that the user has at least placed random gates in all locations that need them.
	 * If all not gate locations have either not gates or "line" gates and the other gates
	 * have some kind of gate registered then it returns true- otherwise false
	 *
	 * @return If all gates have been filled and not gate positions have only a line or not gate placed
	 */
	public boolean allUserGatesPlaced() {
		if ((userGate1 != null) && (userGate2 != null) && 
		    (userGate3 != null) && (userGate4 != null) && (userGate5 != null ) &&
		    (userNotGate1 != null) && (userNotGate2 != null) && (userNotGate3 != null) && 
		    (userNotGate4 != null) && (userNotGate5 != null) && (userNotGate6 != null) && 
		    (userNotGate7 != null) && (userNotGate8 != null)) {
		       
			if (((userNotGate1.toString().equals("LINE \"gate\"")) ||
				 (userNotGate1.toString().equals("NOT gate"))) &&
				((userNotGate2.toString().equals("LINE \"gate\"")) ||
				 (userNotGate2.toString().equals("NOT gate"))) &&
				((userNotGate3.toString().equals("LINE \"gate\"")) ||
				 (userNotGate3.toString().equals("NOT gate"))) &&
				((userNotGate4.toString().equals("LINE \"gate\"")) ||
				 (userNotGate4.toString().equals("NOT gate"))) &&
				((userNotGate5.toString().equals("LINE \"gate\"")) ||
				 (userNotGate5.toString().equals("NOT gate"))) &&
				((userNotGate6.toString().equals("LINE \"gate\"")) ||
				 (userNotGate6.toString().equals("NOT gate"))) &&
				((userNotGate7.toString().equals("LINE \"gate\"")) ||
				 (userNotGate7.toString().equals("NOT gate"))) &&
				((userNotGate8.toString().equals("LINE \"gate\"")) ||
				 (userNotGate8.toString().equals("NOT gate")))) {
			
				if (((!(userGate1.toString().equals("LINE \"gate\""))) &&
				     (!(userGate1.toString().equals("NOT gate")))) && 
				    ((!(userGate2.toString().equals("LINE \"gate\""))) &&
				     (!(userGate2.toString().equals("NOT gate")))) && 
				    ((!(userGate3.toString().equals("LINE \"gate\""))) &&
				     (!(userGate3.toString().equals("NOT gate")))) && 
				    ((!(userGate4.toString().equals("LINE \"gate\""))) &&
				     (!(userGate4.toString().equals("NOT gate")))) && 
				    ((!(userGate5.toString().equals("LINE \"gate\""))) &&
				     (!(userGate5.toString().equals("NOT gate"))))) {
				    
				     //all gates are placed and all notgate positions have NOt gates or LINE gates only
					return true;
				}
				else {		
					return false;//one of the user gates with two inputs has a one input gate placed
				}
			}
			else {
				return false;//user has been silly and put a non NOT|LINE gate in one of the places for it	
			}
		}
		else {
			return false;//user has not filled all of the positions
		}
	}

	/**
	 * Allows access to all of the gate locations users are allowed to drop into
	 * <p>
	 * For simplification the relative gate locations defined in the JPanel 
	 * that the gates are drawn onto were recreated here and not actually referenced.
	 * Hence these need to be manually updated if the actual locations change!
	 * 
	 * @return A list of all dropbox locations that are available for user placed gates
	 */
	public DropBoxList getDropBoxLocations() {
		DropBoxList list = new DropBoxList();
		
		//set up the locations for all of the boxes that are the drop target locations
		// - the figures come from the main JPanel where they are actually drawn on to it
    	int imageWidth  = 75;
    	int imageHeight = 50;
/************************************************************/









    			





    			

/************************************************************/
		int not1X1 = 60;
		int not1Y1 =  5;
	    			
		int not2X1 = not1X1;
		int not2Y1 = not1Y1+imageHeight+5;
		    			
		int not3X1 = not1X1;
		int not3Y1 = not2Y1+imageHeight+5;
    			
		int not4X1 = not1X1;
		int not4Y1 = not3Y1+imageHeight+5;
    			
  		int gate1X1 = not2X1+imageWidth+15;
		int gate1Y1 = not2Y1-7;  			
   			
		int gate2X1 = not3X1+imageWidth+15;
		int gate2Y1 = not3Y1+7;
	
		int not5X1 = gate1X1+imageWidth;
		int not5Y1 = gate1Y1;

		int not6X1 = gate2X1+imageWidth;
		int not6Y1 = gate2Y1;

		int gate3X1 = not5X1+imageWidth;
		int gate3Y1 = (not5Y1+not6Y1)/2;		
		
		int not7X1 = gate3X1+imageWidth;
		int not7Y1 = gate3Y1;
		
		int gate4X1 = not7X1+imageWidth;
		int gate4Y1 = (not1Y1+not7Y1)/2;
		
		int gate5X1 = gate4X1+imageWidth;
		int gate5Y1 = (gate4Y1+not4Y1)/2;
						
		int not8X1 = gate5X1+imageWidth;
		int not8Y1 = gate5Y1;		
					
		list.addDropBoxtoList(new DropBox(new Point(gate1X1, gate1Y1), new Point(gate1X1+imageWidth, gate1Y1+imageHeight), "userGate1"));
		list.addDropBoxtoList(new DropBox(new Point(gate2X1, gate2Y1), new Point(gate2X1+imageWidth, gate2Y1+imageHeight), "userGate2"));
		list.addDropBoxtoList(new DropBox(new Point(gate3X1, gate3Y1), new Point(gate3X1+imageWidth, gate3Y1+imageHeight), "userGate3"));
		list.addDropBoxtoList(new DropBox(new Point(gate4X1, gate4Y1), new Point(gate4X1+imageWidth, gate4Y1+imageHeight), "userGate4"));
		list.addDropBoxtoList(new DropBox(new Point(gate5X1, gate5Y1), new Point(gate5X1+imageWidth, gate5Y1+imageHeight), "userGate5"));
		list.addDropBoxtoList(new DropBox(new Point(not1X1, not1Y1), new Point(not1X1+imageWidth, not1Y1+imageHeight), "userNotGate1"));
		list.addDropBoxtoList(new DropBox(new Point(not2X1, not2Y1), new Point(not2X1+imageWidth, not2Y1+imageHeight), "userNotGate2"));
		list.addDropBoxtoList(new DropBox(new Point(not3X1, not3Y1), new Point(not3X1+imageWidth, not3Y1+imageHeight), "userNotGate3"));
		list.addDropBoxtoList(new DropBox(new Point(not4X1, not4Y1), new Point(not4X1+imageWidth, not4Y1+imageHeight), "userNotGate4"));		
		list.addDropBoxtoList(new DropBox(new Point(not5X1, not5Y1), new Point(not5X1+imageWidth, not5Y1+imageHeight), "userNotGate5"));		
		list.addDropBoxtoList(new DropBox(new Point(not6X1, not6Y1), new Point(not6X1+imageWidth, not6Y1+imageHeight), "userNotGate6"));		
		list.addDropBoxtoList(new DropBox(new Point(not7X1, not7Y1), new Point(not7X1+imageWidth, not7Y1+imageHeight), "userNotGate7"));		
		list.addDropBoxtoList(new DropBox(new Point(not8X1, not8Y1), new Point(not8X1+imageWidth, not8Y1+imageHeight), "userNotGate8"));		
	
		return list;
	}

	/**
	 * Sets up the JPanel that contains the circuit
	 * 
	 * @param    hints   If true then hints are drawn on to the circuit otherwise they aren't
	 * @param    complete If false then the drop targets are highlighted, otherwise there are filled in 
	 *
	 * @return   CircuitDiagram - A diagram drawn on to a JPanel with or without hints based on the hints input
	 */
	public JPanel getCircuitDiagramPanel(boolean hints, boolean complete) {
		
		//these need to be final because we may need to access them inside
		//the paint method so that it a row can be painted
		final Object[][] answers = getTruthTableDetails();
		Random rand = new Random();
		//return a random int range 0 to length-1
		final int hintRow = rand.nextInt(answers.length);
		
		final boolean hintsOn = hints;
		final boolean completed = complete;
		setRevealedStatus(complete);//store this so we can do sensible output when revealing hints etc
		
		JPanel temp = new JPanel() {
			public void paintComponent(Graphics g) {
				 super.paintComponent(g); //paint background

    			//set the image width and height
    			int imageWidth  = 75;
    			int imageHeight = 50;

	  			//need to create a standard box image that can be used as a blank target spacer
    			Image boxImage = createImage(imageWidth, imageHeight);
    			Graphics boxImg = boxImage.getGraphics();
				//make it the right colour of background
				boxImg.setColor(Color.lightGray);
				boxImg.fillRect(0,0,boxImage.getWidth(this),boxImage.getHeight(this));
	  			//fill the external border to make it stand out
	  			boxImg.setColor(Color.blue);
			  	boxImg.draw3DRect(1,1,imageWidth-6,imageHeight-4,true);
				//put on the big question marke
				boxImg.setColor(Color.black);
				boxImg.drawString("?",((imageWidth-6)/2),((imageHeight+6)/2));

				//all of the gate locations are set up as relative as possible
				//hence they need to be set up in the order in which they appear in the diagram

				//set up the first four not gates
     			int notGate1x = 60;
    			int notGate1y =  5;

    			int notGate2x = notGate1x;
    			int notGate2y = notGate1y+imageHeight+5;

    			int notGate3x = notGate1x;
    			int notGate3y = notGate2y+imageHeight+5;

    			int notGate4x = notGate1x;
    			int notGate4y = notGate3y+imageHeight+5;

    			//set the root locations of the 1st random gate
    			int gate1x = notGate2x+imageWidth+15;
    			int gate1y = notGate2y-7;

   				//set the root location of the second random gate
    			int gate2x = notGate3x+imageWidth+15;
    			int gate2y = notGate3y+7;
    			
    			//set the root location of the fifth and sixth not gates
    			int notGate5x = gate1x+imageWidth;
    			int notGate5y = gate1y;

    			int notGate6x = gate2x+imageWidth;
    			int notGate6y = gate2y;

 				//set the root location of the third random gate
    			int gate3x = notGate5x+imageWidth;
    			int gate3y = (notGate5y+notGate6y)/2;//gives the point halfway between
    			
    			//set the location of the 7th not gate
    			int notGate7x = gate3x+imageWidth;
    			int notGate7y = gate3y;
    			
    			//set the location of the fourth random gate
    			int gate4x = notGate7x+imageWidth;
    			int gate4y = (notGate1y+notGate7y)/2;

    			//set the location of the fifth random gate
    			int gate5x = gate4x+imageWidth;
    			int gate5y = (gate4y+notGate4y)/2;

    			//set the location of the 8th not gate
    			int notGate8x = gate5x+imageWidth;
    			int notGate8y = gate5y;

    			//set the background of the actual panel
    			g.setColor(Color.lightGray);
				g.fillRect(0,0,getWidth(),getHeight());
    			g.setColor(Color.black);
    			g.drawRect(0,0,getWidth()-2,getHeight()-2);
    			
        		//first paint on the five random gates
        		//get the image of the 1st gate after first painting the background of the image
    			Image image = createImage(imageWidth, imageHeight);
    			Graphics gImg = image.getGraphics();
				gImg.setColor(Color.lightGray);
				gImg.fillRect(0,0,image.getWidth(this), image.getHeight(this));

    			//draw random gate1 or blank box or user gate depending on the status
    			if (completed) {
    				image = actualGate1.getImageRepresentation(image, 10, 10, hintsOn);//get the actual image
        			g.drawImage(image, gate1x, gate1y, this);//Draw image at its natural size first (75x50 image);
				}
				else {
					if (userGate1==null) {
						g.drawImage(boxImage, gate1x, gate1y, this);
					}
					else {//the user has placed a gate in here so we must show that
						image = userGate1.getImageRepresentation(image, 10, 10, hintsOn);//get the actual image
        				
        				//fill the external border to make it stand out
	  					gImg.setColor(Color.gray);
					  	gImg.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
					  	
        				g.drawImage(image, gate1x, gate1y, this);//Draw image at its natural size first (75x50 image);
					}
				}
				
         		//get the image of the 2nd gate after first painting the background of the image
        		Image image2 = createImage(imageWidth, imageHeight);
    			Graphics gImg2 = image2.getGraphics();
    			gImg2.setColor(Color.lightGray);
				gImg2.fillRect(0,0,image2.getWidth(this),image2.getHeight(this));

    			//draw random gate2 or blank box or user gate depending on status
    			if (completed) {
    				image2 = actualGate2.getImageRepresentation(image2, 10, 10, hintsOn);
        			g.drawImage(image2, gate2x, gate2y, this);
				}
				else {
					if (userGate2==null) {
						g.drawImage(boxImage, gate2x, gate2y, this);
					}
					else {//the user has placed a gate in here so we must show that
    					image2 = userGate2.getImageRepresentation(image2, 10, 10, hintsOn);
    					
        				//fill the external border to make it stand out
	  					gImg2.setColor(Color.gray);
					  	gImg2.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);

        				g.drawImage(image2, gate2x, gate2y, this);
        			}						
				}
				
        		//get the image of the 3rd gate after first painting the background of the image
        		Image image3 = createImage(imageWidth, imageHeight);
    			Graphics gImg3 = image3.getGraphics();
    			gImg3.setColor(Color.lightGray);
				gImg3.fillRect(0,0,image3.getWidth(this),image3.getHeight(this));

    			//draw random gate3 or blank box or user gate depending on status
    			if (completed) {
    				image3 = actualGate3.getImageRepresentation(image3, 10, 10, hintsOn);
        			g.drawImage(image3, gate3x, gate3y, this);
				}
				else {
					if (userGate3==null) {
						g.drawImage(boxImage, gate3x, gate3y, this);
					}
					else {//the user has placed a gate in here so we must show that
    					image3 = userGate3.getImageRepresentation(image3, 10, 10, hintsOn);
    					
        				//fill the external border to make it stand out
	  					gImg3.setColor(Color.gray);
					  	gImg3.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);

        				g.drawImage(image3, gate3x, gate3y, this);
        			}
				}

		   		//get the image of the 4th gate after first painting the background of the image
        		Image image4 = createImage(imageWidth, imageHeight);
    			Graphics gImg4 = image4.getGraphics();
    			gImg4.setColor(Color.lightGray);
				gImg4.fillRect(0,0,image4.getWidth(this),image4.getHeight(this));

    			//draw random gate4 or blank box or user gate depending on status
    			if (completed) {
    				image4 = actualGate4.getImageRepresentation(image4, 10, 10, hintsOn);
        			g.drawImage(image4, gate4x, gate4y, this);
				}
				else {
					if (userGate4==null) {
						g.drawImage(boxImage, gate4x, gate4y, this);
					}
					else {//the user has placed a gate in here so we must show that
    					image4 = userGate4.getImageRepresentation(image4, 10, 10, hintsOn);
    					
        				//fill the external border to make it stand out
	  					gImg4.setColor(Color.gray);
					  	gImg4.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);

        				g.drawImage(image4, gate4x, gate4y, this);
        			}
				}
        		
 		   		//get the image of the 5th gate after first painting the background of the image
        		Image image5 = createImage(imageWidth, imageHeight);
    			Graphics gImg5 = image5.getGraphics();
    			gImg5.setColor(Color.lightGray);
				gImg5.fillRect(0,0,image5.getWidth(this), image5.getHeight(this));

    			//draw random gate5 or blank box or user gate depending on status
    			if (completed) {
    				image5 = actualGate5.getImageRepresentation(image5, 10, 10, hintsOn);
        			g.drawImage(image5, gate5x, gate5y, this);
				}
				else {
					if (userGate5==null) {
						g.drawImage(boxImage, gate5x, gate5y, this);
					}
					else {//the user has placed a gate in here so we must show that
    					image5 = userGate5.getImageRepresentation(image5, 10, 10, hintsOn);
    					
        				//fill the external border to make it stand out
	  					gImg5.setColor(Color.gray);
					  	gImg5.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);

        				g.drawImage(image5, gate5x, gate5y, this);
        			}
				}
				
				//Next draw all of the connecter lines
				//draw the line from a to gate 1 through not gate1
				int x1 = notGate1x-15;
				int y1 = notGate1y+25;
				int x2 = gate4x;
				int y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line to above gate4

				x1 = gate1x;
				y1 = notGate1y+25;
				x2 = x1;
				y2 = gate1y+18;
				g.drawLine(x1,y1,x2,y2);//down line from a's line to gate1 input 1
				
				x1 = gate4x;
				y1 = notGate1y+25;
				x2 = x1;
				y2 = gate4y+18;
				g.drawLine(x1,y1,x2,y2);//down line from a's line to gate4 input 1
				
				//draw line from input b
				x1 = notGate2x-15;
				y1 = notGate2y+25;
				x2 = gate1x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line from b through notgate2 into gate1's 2nd input
				
				//draw line from gate1 through notgate5 to top of gate3
				x1 = gate1x+imageWidth;
				y1 = gate1y+25;
				x2 = gate3x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line from gate1 to top gate3
				
				x1 = notGate5x+imageWidth;
				y1 = notGate5y+25;
				x2 = x1;
				y2 = gate3y+18;
				g.drawLine(x1,y1,x2,y2);//down line from gate1 to gate3 input
					
				//draw the line from c to gate 2 through notgate3
				x1 = notGate3x-15;
				y1 = notGate3y+25;
				x2 = gate2x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line through notgate3 to gate2	
				
				//draw line from gate2 through notgate6 to bottom of gate3
				x1 = gate2x+imageWidth;
				y1 = gate2y+25;
				x2 = gate3x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line from gate2 to bottom gate3
				
				x1 = gate3x;
				y1 = gate2y+25;
				x2 = x1;
				y2 = gate3y+32;
				g.drawLine(x1,y1,x2,y2);//up line from bottom gate3 to 2nd input gate3
				
				//draw line from gate3 through notgate7 to bottom of gate4
				x1 = gate3x+imageWidth;
				y1 = gate3y+25;
				x2 = gate4x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line from gate3 to bottom gate4 through notgate7				

				x1 = gate4x;
				y1 = gate3y+25;
				x2 = x1;
				y2 = gate4y+32;
				g.drawLine(x1,y1,x2,y2);//up line from gate3 to 2nd input gate4 				

				//down ine from gate4 to gate5
				x1 = gate4x+imageWidth;
				y1 = gate4y+25;
				x2 = x1;
				y2 = gate5y+18;
				g.drawLine(x1,y1,x2,y2);//down line from gate4 to 1st input gate5 				
		
				//draw the line from d to gate 5 through not gate4
				x1 = notGate4x-15;
				y1 = notGate4y+25;
				x2 = gate5x;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//across line to below gate5
				
				x1 = gate2x;
				y1 = notGate4y+25;
				x2 = x1;
				y2 = gate2y+33;
				g.drawLine(x1,y1,x2,y2);//up line from below gate2 to 2nd input gate2 
				
				x1 = gate5x;
				y1 = notGate4y+25;
				x2 = x1;
				y2 = gate5y+33;
				g.drawLine(x1,y1,x2,y2);//up line from below gate5 to 2nd input gate5 
				
				//draw line from gate5 through notgate8 to output
				x1 = gate5x+imageWidth;
				y1 = gate5y+25;
				x2 = notGate8x+imageWidth;
				y2 = y1;
				g.drawLine(x1,y1,x2,y2);//down line from gate4 to 1st input gate5 

				//next draw the not gates if they have been set for this particular diagram
        		Image notImage1 = createImage(imageWidth, imageHeight);
    			Graphics gNotImg1 = notImage1.getGraphics();
    			gNotImg1.setColor(Color.lightGray);
				gNotImg1.fillRect(0,0,notImage1.getWidth(this),notImage1.getHeight(this));

        		if (completed) {//need to draw the notGate1 properly or just a normal line through
        			if (notGate1Exists) {      				
    					//draw not gate	1
    					notImage1 = actualNotGate1.getImageRepresentation(notImage1, 10, 10, hintsOn);
        				g.drawImage(notImage1, notGate1x, notGate1y, this);        				
        			}
        		}
        		else {//need to draw the blank box or the user placed gate
        			if (userNotGate1 == null) { //draw the blank box
        				g.drawImage(boxImage, notGate1x, notGate1y, this);
        			}
        			else {//draw the gate the user thinks should be here
    					notImage1 = userNotGate1.getImageRepresentation(notImage1, 10, 10, hintsOn);
        				
         				//fill the external border to make it stand out
	  					gNotImg1.setColor(Color.gray);
					  	gNotImg1.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
        				
        				g.drawImage(notImage1, notGate1x, notGate1y, this);
        			}
        		}

       			//get the image of the 2nd not gate after first painting the background of the image
       			Image notImage2 = createImage(imageWidth, imageHeight);
   				Graphics gNotImg2 = notImage2.getGraphics();
   				gNotImg2.setColor(Color.lightGray);
				gNotImg2.fillRect(0,0,notImage2.getWidth(this),notImage2.getHeight(this));

        		if (completed) {//need to draw the notGate2 properly or just a normal line through
        			if (notGate2Exists) {
    					//draw not gate	2
    					notImage2 = actualNotGate2.getImageRepresentation(notImage2, 10, 10, hintsOn);
        				g.drawImage(notImage2, notGate2x, notGate2y, this);        				
        			}
        		}
        		else {//need to draw the blank box or the user placed gate
        			if (userNotGate2 == null) { //draw the blank box
        				g.drawImage(boxImage, notGate2x, notGate2y, this);
        			}
        			else {//draw the gate the user thinks should be here
    					notImage2 = userNotGate2.getImageRepresentation(notImage2, 10, 10, hintsOn);
        				
         				//fill the external border to make it stand out
	  					gNotImg2.setColor(Color.gray);
					  	gNotImg2.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
        				
        				g.drawImage(notImage2, notGate2x, notGate2y, this);
        			}
        		}

       			Image notImage3 = createImage(imageWidth, imageHeight);
   				Graphics gNotImg3 = notImage3.getGraphics();
    			gNotImg3.setColor(Color.lightGray);
				gNotImg3.fillRect(0,0,notImage3.getWidth(this),notImage3.getHeight(this));

        		if (completed) {//need to draw the notGate3 properly or just a normal line through
        			if (notGate3Exists) {
    					//draw not gate	3
    					notImage3 = actualNotGate3.getImageRepresentation(notImage3, 10, 10, hintsOn);
        				g.drawImage(notImage3, notGate3x, notGate3y, this);        				
        			}
        		}
        		else {//need to draw the blank box or the user placed gate
        			if (userNotGate3 == null) { //draw the blank box
        				g.drawImage(boxImage, notGate3x, notGate3y, this);
        			}
        			else {//draw the gate the user thinks should be here
    					notImage3 = userNotGate3.getImageRepresentation(notImage3, 10, 10, hintsOn);
        				
         				//fill the external border to make it stand out
	  					gNotImg3.setColor(Color.gray);
					  	gNotImg3.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
        				
        				g.drawImage(notImage3, notGate3x, notGate3y, this);
        			}
        		}

       			Image notImage4 = createImage(imageWidth, imageHeight);
   				Graphics gNotImg4 = notImage4.getGraphics();
   				gNotImg4.setColor(Color.lightGray);
				gNotImg4.fillRect(0, 0, notImage4.getWidth(this), notImage4.getHeight(this));

        		if (completed) {//need to draw the notGate4 properly or just a normal line through
        			if (notGate4Exists) {
    					//draw not gate	4
    					notImage4 = actualNotGate4.getImageRepresentation(notImage4, 10, 10, hintsOn);
        				g.drawImage(notImage4, notGate4x, notGate4y, this);        				
        			}
        		}
        		else {//need to draw the blank box or the user placed gate
        			if (userNotGate4 == null) { //draw the blank box
        				g.drawImage(boxImage, notGate4x, notGate4y, this);
        			}
        			else {//draw the gate the user thinks should be here
    					notImage4 = userNotGate4.getImageRepresentation(notImage4, 10, 10, hintsOn);
        				
         				//fill the external border to make it stand out
	  					gNotImg4.setColor(Color.gray);
					  	gNotImg4.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
        				
        				g.drawImage(notImage4, notGate4x, notGate4y, this);
        			}
        		}

        		Image notImage5 = createImage(imageWidth, imageHeight);
    			Graphics gNotImg5 = notImage5.getGraphics();
    			gNotImg5.setColor(Color.lightGray);
				gNotImg5.fillRect(0,0,notImage5.getWidth(this),notImage5.getHeight(this));

        		if (completed) {//need to draw the notGate5 properly or just a normal line through
        			if (notGate5Exists) {
    					//draw not gate	1
    					notImage5 = actualNotGate5.getImageRepresentation(notImage5, 10, 10, hintsOn);
        				g.drawImage(notImage5, notGate5x, notGate5y, this);        				
        			}
        		}
        		else {//need to draw the blank box or the user placed gate
        			if (userNotGate5 == null) { //draw the blank box
        				g.drawImage(boxImage, notGate5x, notGate5y, this);
        			}
        			else {//draw the gate the user thinks should be here
    					notImage5 = userNotGate5.getImageRepresentation(notImage5, 10, 10, hintsOn);
        				
         				//fill the external border to make it stand out
	  					gNotImg5.setColor(Color.gray);
					  	gNotImg5.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
        				
        				g.drawImage(notImage5, notGate5x, notGate5y, this);
        			}
        		}

       			Image notImage6 = createImage(imageWidth, imageHeight);
    			Graphics gNotImg6 = notImage6.getGraphics();
    			gNotImg6.setColor(Color.lightGray);
				gNotImg6.fillRect(0, 0, notImage6.getWidth(this), notImage6.getHeight(this));

        		if (completed) {//need to draw the notGate6 properly or just a normal line through
        			if (notGate6Exists) {
    					//draw not gate	6
    					notImage6 = actualNotGate6.getImageRepresentation(notImage6, 10, 10, hintsOn);
        				g.drawImage(notImage6, notGate6x, notGate6y, this);        				
        			}
        		}
        		else {//need to draw the blank box or the user placed gate
        			if (userNotGate6 == null) { //draw the blank box
        				g.drawImage(boxImage, notGate6x, notGate6y, this);
        			}
        			else {//draw the gate the user thinks should be here
    					notImage6 = userNotGate6.getImageRepresentation(notImage6, 10, 10, hintsOn);
        				
         				//fill the external border to make it stand out
	  					gNotImg6.setColor(Color.gray);
					  	gNotImg6.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
        				
        				g.drawImage(notImage6, notGate6x, notGate6y, this);
        			}
        		}

       			Image notImage7 = createImage(imageWidth, imageHeight);
    			Graphics gNotImg7 = notImage7.getGraphics();
    			gNotImg7.setColor(Color.lightGray);
				gNotImg7.fillRect(0, 0, notImage7.getWidth(this), notImage7.getHeight(this));

        		if (completed) {//need to draw the notGate7 properly or just a normal line through
        			if (notGate7Exists) {
    					//draw not gate	7
    					notImage7 = actualNotGate7.getImageRepresentation(notImage7, 10, 10, hintsOn);
        				g.drawImage(notImage7, notGate7x, notGate7y, this);        				
        			}
        		}
        		else {//need to draw the blank box or the user placed gate
        			if (userNotGate7 == null) { //draw the blank box
        				g.drawImage(boxImage, notGate7x, notGate7y, this);
        			}
        			else {//draw the gate the user thinks should be here
    					notImage7 = userNotGate7.getImageRepresentation(notImage7, 10, 10, hintsOn);
        				
         				//fill the external border to make it stand out
	  					gNotImg7.setColor(Color.gray);
					  	gNotImg7.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
        				
        				g.drawImage(notImage7, notGate7x, notGate7y, this);
        			}
        		}

       			Image notImage8 = createImage(imageWidth, imageHeight);
    			Graphics gNotImg8 = notImage8.getGraphics();
    			gNotImg8.setColor(Color.lightGray);
				gNotImg8.fillRect(0, 0, notImage8.getWidth(this), notImage8.getHeight(this));

        		if (completed) {//need to draw the notGate6 properly or just a normal line through
        			if (notGate8Exists) {
    					//draw not gate	8
    					notImage8 = actualNotGate8.getImageRepresentation(notImage8, 10, 10, hintsOn);
        				g.drawImage(notImage8, notGate8x, notGate8y, this);        				
        			}
        		}
        		else {//need to draw the blank box or the user placed gate
        			if (userNotGate8 == null) { //draw the blank box
        				g.drawImage(boxImage, notGate8x, notGate8y, this);
        			}
        			else {//draw the gate the user thinks should be here
    					notImage8 = userNotGate8.getImageRepresentation(notImage8, 10, 10, hintsOn);
        				
         				//fill the external border to make it stand out
	  					gNotImg8.setColor(Color.gray);
					  	gNotImg8.draw3DRect(1,1,imageWidth-6,imageHeight-3,true);
        				
        				g.drawImage(notImage8, notGate8x, notGate8y, this);
        			}
        		}

        		//the split connecters of the a and d inputs 
        		g.fillOval(gate1x-4, notGate1y+22,8,8);//a
        		g.fillOval(gate2x-4, notGate4y+22,8,8);//d

        		//next draw the text on to the circuit
        		g.drawString(aLabel, notGate1x-35, notGate1y+28);
    			g.drawString(bLabel, notGate2x-35, notGate2y+28);
    			g.drawString(cLabel, notGate3x-35, notGate3y+28);
    			g.drawString(dLabel, notGate4x-35, notGate4y+28);
    			g.drawString(eLabel, gate3x+5, gate3y+5);
    			g.drawString(fLabel, gate3x+5, gate3y+55);
    			g.drawString(gLabel, gate4x+5, gate4y+55);
    			g.drawString(hLabel, gate5x+5, gate5y+5);
    			g.drawString(iLabel, gate5x+5, gate5y+55);
    			g.drawString(outLabel, notGate8x+imageWidth-20, notGate8y+20);

    			g.setColor(Color.black);//redraw the outline box
    			g.drawRect(0,0,getWidth()-2,getHeight()-2);

    			if (hintsOn) {
        			//paint the preselected row of the correct answers numbers on to the panel

					//known outputs
					g.drawString(answers[hintRow][0].toString(), notGate1x-45, notGate1y+28);//a
					g.drawString(answers[hintRow][1].toString(), notGate2x-45, notGate2y+28);//b
					g.drawString(answers[hintRow][2].toString(), notGate3x-45, notGate3y+28);//c
					g.drawString(answers[hintRow][3].toString(), notGate4x-45, notGate4y+28);//d
					
					g.drawString(answers[hintRow][4].toString(), gate3x-10, gate3y+5); //e
					g.drawString(answers[hintRow][5].toString(), gate3x-10, gate3y+55);//f
					g.drawString(answers[hintRow][6].toString(), gate4x-10, gate4y+55);//g
					g.drawString(answers[hintRow][7].toString(), gate5x-10, gate5y+5); //h
					g.drawString(answers[hintRow][8].toString(), gate5x-10, gate5y+55);//i

					g.drawString(answers[hintRow][9].toString(), notGate8x+imageWidth-2, notGate8y+38);//output

					//after inverters outputs (if necessary)
					//note that the first five are after inverter's and the last three are before inverters
					if(notGate1Exists) {
						g.drawString(Integer.toString(actualNotGate1.getOutput(Integer.parseInt((answers[hintRow][0].toString())))), notGate1x+imageWidth-15, notGate1y+20);
					}

					if(notGate2Exists) {
						g.drawString(Integer.toString(actualNotGate2.getOutput(Integer.parseInt((answers[hintRow][1].toString())))), notGate2x+imageWidth-15, notGate2y+40);
					}

					if(notGate3Exists) {
						g.drawString(Integer.toString(actualNotGate3.getOutput(Integer.parseInt((answers[hintRow][2].toString())))), notGate3x+imageWidth-15, notGate3y+20);
					}

					if(notGate4Exists) {
						g.drawString(Integer.toString(actualNotGate4.getOutput(Integer.parseInt((answers[hintRow][3].toString())))), notGate4x+imageWidth-15, notGate4y+20);
					}

					if(notGate5Exists) {//the e value in the answer is already displayed so want the value before the not gate
						g.drawString(Integer.toString(actualNotGate5.getOutput(Integer.parseInt((answers[hintRow][4].toString())))), notGate5x-5, notGate5y+20);
					}

					if(notGate6Exists) {//the f value in the answer is already displayed so want the value before the not gate
						g.drawString(Integer.toString(actualNotGate6.getOutput(Integer.parseInt((answers[hintRow][5].toString())))), notGate6x-5, notGate6y+20);
					}

					if(notGate7Exists) {//the g value in the answer is already displayed so want the value before the not gate
						g.drawString(Integer.toString(actualNotGate7.getOutput(Integer.parseInt((answers[hintRow][6].toString())))), notGate7x-5, notGate7y+20);
					}

					if(notGate8Exists) {//the output value in the answer is already displayed so want the value before the not gate
						g.drawString(Integer.toString(actualNotGate8.getOutput(Integer.parseInt((answers[hintRow][9].toString())))), notGate8x-5, notGate8y+20);
					}
    			}
    		}
    	};
		dropTarget = new DropTarget(temp, new DTListener(this));
		return temp;
    }
}