import java.awt.*;
import java.lang.*;
import javax.swing.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;

/**
 * LogicCircuits.java 1.0 03/02/01
 * An abstract class that contains all the details of a particular question circuit
 * which is a drop target
 * <br>
 * Including details of
 * <br>
 * <ul>
 * <li>the diagram itself with spaces for users to fill in</li>
 * <li>the TruthTable corresponding to the question</li>
 * <li>hint options - same as circuit class type of hint</li>
 * <li>correct answer - etc</li>
 * </ul>
 * <br>
 * This abstract class also provides a method that allows a random
 * logic gate to be returned.
 * <br><br>
 * Created      : 1/02/2003
 * <br><br>
 * Last Updated : 1/03/2003
 * <br><br>
 * 
 * @author   James McMahon
 */
public abstract class DropTargetCircuit {

	/** 
	 * Tracks whether or not the answer gates have ever been revealed or not
	 */
	private boolean revealedStatus = false;

	/**
	 * Sets up the drop target that needs to be intialised by every implementating class
	 */
	public DropTarget dropTarget; 

	/**
	 * Should return the data that is created for the question.
	 * <br>
	 * Namely the truth table that corresponds to the circuit users are asked to complete.
	 */
	public abstract Object[][] getTruthTableDetails();

	/**
	 * should return the headings for the truth table (such as A, B, C, Output etc)
	 */
	public abstract String[] getTruthTableHeadings();

	/**
	 * Returns the circuit panel without hints and the drop targets highlighted.
	 * <br>
	 * Provided as a convenient wrapper
	 * 
	 * @return   unhintedCircuitDiagram The diagram for this question drawn on to a JPanel without hints
	 */
	public JPanel getCircuitDiagram() {
		this.revealedStatus = false;
		return getCircuitDiagramPanel(false, false);
	}

	/**
	 * Returns the circuit panel with a random hint drawn on to it and the drop targets
	 * highlighted.
	 * <br>
	 * Hints are of the same type as other questions but with the addition
	 * of each incorrectly placed/non placed gate by the user highlighted as well
	 * <br>
	 * provided as a convenient wrapper
	 * 
	 * @return   hintedCircuitDiagram The diagram for this question is returned with hints drawn on to it
	 */
	public JPanel getRandomHintedCircuitDiagram() {
		this.revealedStatus = false;
		return getCircuitDiagramPanel(true, false);
	}

	/**
	 * Should return a completed image of the question circuit without drop targets - having 
	 * them filled in with the gates used initially to create the question circuit.
	 * Does not have any hints displayed.
	 */
	public JPanel getCompletedCircuitDiagramPanel() {
		this.revealedStatus = true;
		return getCircuitDiagramPanel(false, true);
	}

	/**
	 * Indicates whether or not the last time the JPanel was called as to whether 
	 * or not the answer gates were actually on screen or not
	 *
	 * @return Whether or not the user last asked for the answer gates to be displayed or not
	 */
	public boolean getRevealedStatus() {
		return revealedStatus;	
	}
	
	/**
	 * Allows the status to be set depending on whether or not the user last 
	 * asked for the answer to be displayed or not
	 * 
	 * @param status True if the answer gates were last revealed, false otherwise. 
	 */
	public void setRevealedStatus(boolean status) {
		revealedStatus = status;	
	}

	/**
	 * Should return the filled in circuitPanel with a randomly generated hint draw on top
	 * if hintsOn is true, if this is false then it should only draw the circuit & no hints.
	 * If completed is true then it draws the actual answer gates on otherwise it outlines
	 * the drop target locations that are acceptable
	 * <br>
	 * The hint is envisaged to take one possible set of inputs and follow the changes on the
	 * connector lines until the output line if , otherwise it should not draw the hints.
	 *
	 * The revealed status of this object must be updated depending on the completed parameter
	 * 
	 * @return The JPanel for display that is also a droptarget
	 */
	public abstract JPanel getCircuitDiagramPanel(boolean hintsOn, boolean completed);
	
	/**
	 * Ensures that the user has at least placed random gates in all locations that need them.
	 * If all not gate locations have either not gates or "line" gates and the other gates
	 * have some kind of gate registered then it returns true- otherwise false
	 *
	 * @return If all gates have been filled and not gate positions have only a line or not gate placed
	 */
	public abstract boolean allUserGatesPlaced();

	/**
	 * gives access to whatever truthtable the gates that the user has placed creates
	 * <p>
	 * N.B. It is important that the gates are checked for correspondance to the actual
	 * truth table details - not just residual information regarding the gates which were
	 * intially used to set up the question! (as more than one setup could cause the same answers)
	 *
	 * @return The truthtable details corresponding to the user built circuit
	 */
	public abstract Object[][] getUserTruthTable();

	/**
	 * Needs to deal with a potential drop at xLoc, yLoc of LogicGate gate
	 * <br>
	 * It should either draw the gate and store it as valid if it is or if the location
	 * doesn't match a valid designated site then it can ignored
	 */
	public abstract void dealWithDrop(LogicGate gate, Point a);

	/**
	 * Should return a dropboxlist that contains all of the drop box locations 
	 * that are valid boxed locations for a user gate drop.
	 * <p>
	 * This allows the drop points to be queried easily
	 *
	 * @return A list of all available drop point boxes
	 */
	public abstract DropBoxList getDropBoxLocations();
	
	/**
	 * Allows the screen to be redrawn after a drop has occured
	 */
	public abstract void redrawScreen();
	
	/**
	 * Needs to provide the number of columns that are non-user modifiable
	 * within the truthtables JTable.
	 */
	public abstract int getNumberOfFixedColumns();
}