import java.awt.*;
import java.lang.Object;
import java.util.*;
import javax.swing.JApplet;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

/**
 * LogicCircuits.java 1.0 03/02/01
 * <br>
 * Author       : James McMahon
 * <br>
 * Logic Circuit Main Applet Class
 * <br>
 * Created      : 1/02/2003
 * <br>
 * Last Updated : 1/02/2003
 */

public class LogicCircuits extends JApplet {

	/**
	 * The truth table for user answers
	 * can be set using the information
	 * gained from the specific question that is randomly generated
	 * the default uses the default values for a table, which can later be
	 * set using the setUpTable function once you have question set data
	 */
	private TruthTablePanel tablePanel = new TruthTablePanel();

	/**
	 * The score tracker for the user
	 */
	private ScorePanel scorePanel = new ScorePanel();

	/**
	 * The menu panel that provides the user with the input options
	 * gets passed in this as it's parent so that the menulistener
	 * only needs to call the relevant methods inside this main class
	 */
	private MenuPanel menuPanel = new MenuPanel(this);

	/**
	 * A panel that can be used to display circuit diagrams
	 * <br>
	 * essentially just displays a painted border JPanel
	 */
	private CircuitPanel circuitPanel = new CircuitPanel();

	/**
	 * A placeholder JPanel to enable easier layout and to allow the
	 * changeover of board configurations.
	 * <br>
	 * Depending on the configuration it contains the score tracker, truth table and either a
	 * message or the drag panel source for making the circuit (which are actually placed on the
	 * tempPanel)
	 */
	private JPanel centralSection = new JPanel(new BorderLayout());

	/**
	 * A second placeholder JPanel that is normally used for the score tracker and the
	 * message or source gates for dragging
	 */
	private JPanel tempPositioningPanel = new JPanel(new BorderLayout());

	/**
	 * Allows a drop target panel to be set up that represents the question truthtable
	 * being asked about, should work in a similiar manner to the CircuitPanel, and should be set
	 * up using the proper procedures (by getting a normal question then modding it)
	 */
	private TargetCircuitPanel dropTargetCircuitPanel = new TargetCircuitPanel();

	/**
	 * Sets up the panel that contains the draggable circuit components that the user can select
	 * in order to place on the dropTargetCircuitPanel
	 */
	private GateSourcePanel gateSourcePanel;

	/**
	 * Provides access to all of the possible circuit diagrams that the program knows about
	 * see the actual relevant classes for details
	 */
	private QuestionGenerator qGenerator;

	/**
	 * This is the abstract question circuit that is returned by calling
	 * getQuestion() in QuestionGenerator
	 */
	private Circuit questionCircuit;

	/**
	 * This is the abstract dropable target question circuit that is returned by calling
	 * getDroppableCircuit() in QuestionGenerator
	 */
	private DropTargetCircuit dropQuestionCircuit;

	/**
	 * Tracks what type of board question is being asked
	 * (false indicates LG-TT, true is the unfinished TT-LG type)
	 */
	private boolean reversed = false;

	/**
	 * The contentPane of the applet
	 */
	private Container content = null;

	/**
	 * The main function for an applet
	 * <br>
	 * Sets up a first question from the easy question set circuits in the LGTT board
	 */
	public void init() {
		super.init();

		//next get access to the contentpane
		content = this.getContentPane();

		//set Layout manager to make layout look nice
		content.setLayout(new BorderLayout());

		qGenerator = new QuestionGenerator(this);
		setUpLGTT(QuestionGenerator.EASY);

		//the menu is never removed and needs to be added only once
		content.add(menuPanel, BorderLayout.SOUTH);

		LogicGate andGate  = new LogicGate(LogicGate.AND);
		LogicGate nandGate = new LogicGate(LogicGate.NAND);
		LogicGate orGate   = new LogicGate(LogicGate.OR);
		LogicGate norGate  = new LogicGate(LogicGate.NOR);
		LogicGate xorGate  = new LogicGate(LogicGate.XOR);
		LogicGate notGate  = new LogicGate(LogicGate.NOT);
		LogicGate lineGate = new LogicGate(LogicGate.LINE);

		Image andImage  = content.createImage(75,50);
		Image nandImage = content.createImage(75,50);
		Image orImage   = content.createImage(75,50);
		Image norImage  = content.createImage(75,50);
		Image xorImage  = content.createImage(75,50);
		Image notImage  = content.createImage(75,50);
		Image lineImage = content.createImage(75,50);

		andImage  = this.setBackground(andImage);
		nandImage = this.setBackground(nandImage);
		orImage   = this.setBackground(orImage);
		norImage  = this.setBackground(norImage);
		xorImage  = this.setBackground(xorImage);
		notImage  = this.setBackground(notImage);
		lineImage = this.setBackground(lineImage);

		TransferableImage im1 = new TransferableImage(andImage,  andGate,  this.getCodeBase());
		TransferableImage im2 = new TransferableImage(nandImage, nandGate, this.getCodeBase());
		TransferableImage im3 = new TransferableImage(orImage,   orGate,   this.getCodeBase());
		TransferableImage im4 = new TransferableImage(norImage,  norGate,  this.getCodeBase());
		TransferableImage im5 = new TransferableImage(xorImage,  xorGate,  this.getCodeBase());
		TransferableImage im6 = new TransferableImage(notImage,  notGate,  this.getCodeBase());
		TransferableImage im7 = new TransferableImage(lineImage, lineGate, this.getCodeBase());

		ArrayList transferableImagesArray = new ArrayList();
		transferableImagesArray.add(im1);
		transferableImagesArray.add(im2);
		transferableImagesArray.add(im3);
		transferableImagesArray.add(im4);
		transferableImagesArray.add(im5);
		transferableImagesArray.add(im6);
		transferableImagesArray.add(im7);

		gateSourcePanel = new GateSourcePanel(transferableImagesArray);

		this.validate();
		this.repaint();
	}

	/**
	 * Takes an image and makes the background gray with a blue border
	 *
	 * @param image The image to be modified
	 * @return The modified image
	 */
	public Image setBackground(Image image) {
		Graphics imageGraphics = image.getGraphics();

	  	//draw the background
	  	imageGraphics.setColor(Color.lightGray);
	  	imageGraphics.fillRect(0,0,75,50);

	  	//draw the surrounding box
	  	imageGraphics.setColor(Color.blue);
	  	imageGraphics.draw3DRect(0,0,image.getWidth(this)-1,image.getHeight(this)-1,true);

	  	return image;
	}

	/**
	 * Sets up the truthtable panel with a question that is generated
	 * via the qGenerator, also adds1 to the score tracker
	 * <br>
	 * QuestionCircuit then becomes the current question so access to all
	 * of the answer functions etc are available from that
	 * <br><br>
	 * TT-LG works differerntly based upon the DropQuestionCircuit
	 */
	public void setUpQuestion(int difficulty) {

		if (!reversed) {//setup a normal LG-TT question
			//get a new question
			questionCircuit = qGenerator.getQuestion(difficulty);

			//setup the truthtable panel
    	    tablePanel.setUpTruthTable(questionCircuit.getTruthTableHeadings(), questionCircuit.getTruthTableDetails(), questionCircuit.getNumberOfFixedColumns());

			//setup the circuit diagram
			circuitPanel.addDiagram(questionCircuit.getCircuitDiagram());
		}
		else {
			//get a new question
			dropQuestionCircuit = qGenerator.getDropTargetQuestion(difficulty);

			//setup the filled in truthtable panel
		    tablePanel.setUpTruthTable(dropQuestionCircuit.getTruthTableHeadings(), dropQuestionCircuit.getTruthTableDetails(), dropQuestionCircuit.getNumberOfFixedColumns());

			//setup the circuit panel
			dropTargetCircuitPanel.addDiagram(dropQuestionCircuit.getCircuitDiagram());
		}

		//register new question on score
		scorePanel.addQuestionAsked();

		//make sure it is all visible
		this.validate();
		this.repaint();
	}

	/**
	 * Shows a circuit panel that has been hinted for display.
	 * <br>
	 * N.B. hints are completely random and may repeat;
	 * it is not intended to reveal all of the answer, but to
	 * reveal a method of working through the question.
	 * <br>
	 * TT-LG should indicate the numbers and types of gate needed to complete the
	 * circuit at a minimum (not necessarily a single unique solution though)
	 */
	public void showHint() {
		if (!reversed) {//hint for LG-TT
			//setup a hinted circuit diagram
			circuitPanel.addDiagram(questionCircuit.getRandomHintedCircuitDiagram());

			//set the table cell renderer to paint incorrect cells red
			tablePanel.enableIncorrectCellMarking(true);

			//next match the known correct cells to the users cells
			//which will mark the cells that are actually incorrect
			matches(tablePanel.getUserAnswer(),questionCircuit.getCorrectAnswer());
		}
		else {//hint for the TT-LG
			if (dropQuestionCircuit.getRevealedStatus()) {
				//last time the gates had been revealed so we need to force the
				//diagram to display the hints and gates together
				dropTargetCircuitPanel.addDiagram(dropQuestionCircuit.getCircuitDiagramPanel(true,true));
			}
			else {//the gates were not on display - but we still want the hint so place
			      //the numbers around the empty panels/user panels
				dropTargetCircuitPanel.addDiagram(dropQuestionCircuit.getRandomHintedCircuitDiagram());
			}

			//set the table cell renderer to paint incorrect cells red
			tablePanel.enableIncorrectCellMarking(true);

			//next match the known correct cells to the users cells
			//which will mark the cells that are actually incorrect
			matches(dropQuestionCircuit.getUserTruthTable(),dropQuestionCircuit.getTruthTableDetails());
		}

		this.validate();
		this.repaint();
	}

	/**
	 * Checks the user's answer matches the right answer and updates score if that is the case
	 *
	 * @return   true if correct,
	 */
	public boolean checkAnswer() {

		if (!reversed) {
			Object[][] userAnswer    = tablePanel.getUserAnswer();
			Object[][] correctAnswer = questionCircuit.getCorrectAnswer();

			//first make sure any previous wrong answers that caused red table cells are reset to default
			tablePanel.blankColouredCells();

			if (matches(userAnswer,correctAnswer)) {//they were right so update the score
				scorePanel.addQuestionRight();
				JOptionPane.showMessageDialog(null,
											  "That was the right answer! Click New for another",
											  "Well Done",
											  JOptionPane.INFORMATION_MESSAGE);
				return true;
			}
			else {//answer was wrong so need to tell the user to try again
				JOptionPane.showMessageDialog(null,
											  "That is the wrong answer! Have you tried hint yet?",
											  "Sorry, that was wrong...",
											  JOptionPane.INFORMATION_MESSAGE);
				return false;
			}
		}
		else {

			if (dropQuestionCircuit.allUserGatesPlaced()) {//they have placed all gates so can check their answer

				//first make sure any previous wrong answers that caused red table cells are reset to default
				tablePanel.blankColouredCells();

				Object[][] userAnswer    = dropQuestionCircuit.getUserTruthTable();//the one the user has created
				Object[][] correctAnswer = dropQuestionCircuit.getTruthTableDetails();//this is the truthtable that was part of the question

				if (matches(userAnswer,correctAnswer)) {
					scorePanel.addQuestionRight();
					JOptionPane.showMessageDialog(null,
												  "That was the right answer! Click New for another",
												  "Well Done",
												  JOptionPane.INFORMATION_MESSAGE);
					return true;
				}
				else {
					JOptionPane.showMessageDialog(null,
											  "That is the wrong answer! Have you tried hint yet?",
											  "Sorry, that was wrong...",
											  JOptionPane.INFORMATION_MESSAGE);
					return false;//the two truthtables didn't match
				}
			}
			else {//answer was wrong so need to tell the user to try again
				JOptionPane.showMessageDialog(null,
											  "You haven't placed all of the gates properly yet! Have you tried hint to get some ideas?",
											  "Sorry, that was wrong...",
											  JOptionPane.INFORMATION_MESSAGE);
				return false;
			}
		}
	}

	/**
	 * Fills in the blanks in the truth table by creating a new fully populated one
	 * <br>
	 * Or displays the proper logic circuit
	 */
	public void showAnswer() {
		if (!reversed) {
			tablePanel.setUpTruthTable(questionCircuit.getTruthTableHeadings(),questionCircuit.getCorrectAnswer(),questionCircuit.getNumberOfFixedColumns());
		}
		else {
			dropTargetCircuitPanel.addDiagram(dropQuestionCircuit.getCompletedCircuitDiagramPanel());
		}
		this.validate();
		this.repaint();
	}

	/**
	 * Resets the truthtable and refreshes the circuit diagram by
	 * physically reapplying it to the screen (in case a hinted diagram is currently displyed)
     * <br>
     * Or it resets as required by the alternative screen
	 */
	public void resetAllScreens() {
		if (!reversed) {//then want to clear the LG-TT screen

			//need to do this in order to remove any possible hinted diagram on display
			circuitPanel.addDiagram(questionCircuit.getCircuitDiagram());

			//resets the truthtable
			tablePanel.setUpTruthTable(questionCircuit.getTruthTableHeadings(),
									   questionCircuit.getTruthTableDetails(),
									   questionCircuit.getNumberOfFixedColumns());
		}
		else {//need to clear the TT-LG screen
			//should repaint a cleared image of a circuit diagram on the screen
			//and wipe any record of user placed circuits
			dropTargetCircuitPanel.addDiagram(dropQuestionCircuit.getCircuitDiagram());

			//resets the tablepanel
			tablePanel.setUpTruthTable(dropQuestionCircuit.getTruthTableHeadings(),
									   dropQuestionCircuit.getTruthTableDetails(),
									   dropQuestionCircuit.getNumberOfFixedColumns());
		}
		this.validate();
		this.repaint();
	}

	/**
	 * Matches the userAnswer to correctAnswer and returns either true or false
	 * <br>
	 * It also highlights the incorrect entries in the table if there are any
	 * <br>
	 * N.B. the coloured cells will only be displayed if the table is set to allow it
	 *
	 * @return   true If the entire table matches
	 */
	private boolean matches(Object[][] userAnswer, Object[][] correctAnswer) {
		boolean result = true;//we assume they match and set to false if we find a mismatch

		//first make sure any previous wrong answers that caused red table cells are reset to default
		tablePanel.blankColouredCells();

		if (userAnswer.length == correctAnswer.length) {//should never be needed but just in case
														//kick us out with false if they don't
														//have the same number of entries
			//loop through all of the secondary arrays
			for (int row = 0; row<userAnswer.length; row++) {
	    		if (userAnswer[row].length == correctAnswer[row].length) {//check we have equal entries
	    			for (int col = 0; col<userAnswer[col].length; col++) {
				    	if (!(userAnswer[row][col]).equals(correctAnswer[row][col])) {
							tablePanel.colourCellAt(row,col);
				    		result = false;//because it doesn't match
				    	}
				    }
	    		}
	    		else {
	    			return false;//not enough entries to match on so not equal!
	    		}
	   		}
	   	}
	   	else {
	   		return false;//because the two arrays don't even have the same number of entries!
	   	}

		//reaching here means we tried to match on every item in both arrays
		//if there was a mismatch result should be false, otherwise it will be true
		//but first repaint the TruthTable to take account of any red painted cells!
		repaint();
		return result;
	}

	/**
	 * Changes the current board to one of the opposite type.
	 * <br>
	 * (i.e. TT-LG to LG-TT)
	 */
	public void changeBoard(int questionDifficulty) {
		if (!reversed) {//need to go to the TT-LG type and remove LG-TT
			reversed = true;
			removeLGTT();
			setUpTTLG(questionDifficulty);
	    	this.validate();
			this.repaint();
		}
		else {//need to go to the LG-TT type from the TT-LG
			reversed = false;
			removeTTLG();
			setUpLGTT(questionDifficulty);
			this.validate();
			this.repaint();
		}
		this.validate();
		this.repaint();
	}

	/**
	 * Removes the panels associated with the LGTT board configuration
	 */
	private void removeLGTT() {
		content.remove(circuitPanel);
		content.remove(centralSection);
		content.remove(tempPositioningPanel);
		this.validate();
		this.repaint();
	}

	/**
	 * Removes the panels associated with the TTLG boardconfiguration
	 */
	private void removeTTLG() {
		content.remove(dropTargetCircuitPanel);
		content.remove(centralSection);
		content.remove(tempPositioningPanel);
		this.validate();
		this.repaint();
	}

	/**
	 * Alters the board configuaration to TT-LG
	 */
	private void setUpTTLG(int questionDifficulty) {

		//first actually generate a question with a difficulty of easy
		setUpQuestion(questionDifficulty);

		//reinitilise the panels that are wanted
		centralSection = new JPanel(new BorderLayout());
		tempPositioningPanel = new JPanel(new BorderLayout());

		//reset the title of the truthtable panel
		tablePanel.setBorderTitle("Try to complete the circuit corresponding to this Truth Table...");

		//add the panels to the positioning ones
		centralSection.add(dropTargetCircuitPanel, BorderLayout.CENTER);

		tempPositioningPanel.add(gateSourcePanel, BorderLayout.CENTER);
		tempPositioningPanel.add(scorePanel, BorderLayout.EAST);
		tempPositioningPanel.setPreferredSize(new Dimension(100,60));

		centralSection.add(tempPositioningPanel, BorderLayout.SOUTH);

		//add all panels to the contentpane
		content.add(tablePanel, BorderLayout.NORTH);
		content.add(centralSection, BorderLayout.CENTER);

		this.validate();
		this.repaint();
	}

	/**
	 * Alters the board configuration to LG-TT
	 */
	private void setUpLGTT(int questionDifficulty) {

		//first actually generate a question with a difficulty of easy
		setUpQuestion(questionDifficulty);

		//reinitilise the panels that are wanted
		centralSection = new JPanel(new BorderLayout());
		tempPositioningPanel = new JPanel(new BorderLayout());

		//reset the title of the truthtable panel
		tablePanel.setBorderTitle("Try to complete the following Truth Table...");

		//add the panels to the positioning ones
		centralSection.add(tablePanel, BorderLayout.CENTER);

		tempPositioningPanel.add(new JLabel("Remember you'll need paper to work these out properly...!", JLabel.CENTER), BorderLayout.CENTER);
		tempPositioningPanel.add(scorePanel, BorderLayout.EAST);
		tempPositioningPanel.setPreferredSize(new Dimension(100,60));

		centralSection.add(tempPositioningPanel, BorderLayout.SOUTH);

		//add all panels to the contentpane
		content.add(circuitPanel, BorderLayout.NORTH);
		content.add(centralSection, BorderLayout.CENTER);

		this.validate();
		this.repaint();
	}

	//allow the applet to start and stop if the user minimises/maximises the browser window
	/**
	 * Does nothing special as no threads are explicitly created in this applet
	 */
	public void start() {
		super.start();
	}

	/**
	 * Does nothing special as no threads are explicitly created in this applet
	 */
	public void stop() {
		super.stop();
	}
}