import java.awt.*;
import java.lang.Object;
import java.lang.String;
import java.util.Vector;
import java.io.Serializable;
import java.net.*;

/**
 * LogicCircuits.java 1.0 03/02/01
 * <br>
 * Author       : James McMahon
 * <br>
 * A helper that wraps around definitions of gates.
 * <br>
 * N.B. tightly coupled to the LogicGates class
 * modification in here need to be reflected in the methods and functions
 * in LogicGates, noticably return random gate, and evaluate gate.
 * <br>
 * Created      : 1/02/2003
 * <br>
 * Last Updated : 1/02/2003
 */
public class LogicGate implements Serializable {

	/**
	 * An AND gate type
	 */
	public final static int AND = 1;

	/**
	 * A NAND gate type
	 */
	public final static int NAND = 2;

	/**
	 * An OR gate type
	 */
	public final static int OR = 3;

	/**
	 * An XOR gate type
	 */
	public final static int XOR = 4;

	/**
	 * A NOR gate type
	 */
	public final static int NOR = 5;

	/**
	 * Nasty this one - it only has one input so is very different to the others
	 */
	public final static int NOT = 50;

	/**
	 * Even worse this is an addition that simplifies the LG-TT side of the applet
	 * for when the user places a null "line" in a gate location.
	 * Has the effect of propagating the signal sent in.
	 */
	public final static int LINE = 100;

	/**
	 * The type of this actual logicGate
	 * <br>
	 * Default this to an AND gate
	 */
	private int logicGateType = 1;

	/**
	 * used in drawing the actual circuits
	 */
	private int scale = 30;

	/**
	 * Constructs a default AND gate
	 */
	public LogicGate() {
		logicGateType = AND;
	}

	/**
	 * Constructs a gate of type (type must be one of LogicGate.AND etc)
	 * <br>
	 * Funny things could happen if this is not the case!!!
	 */
	public LogicGate(int type) {
		logicGateType = type;
	}

	/**
	 * returns the type of this gate
	 *
	 * @return   1 An AND gate
	 */
	public int getTypeOfGate() {
		return logicGateType;
	}

	/**
	 * Returns the generated output of this gate given the two inputs
	 * <br>
	 * Assumes gate has two inputs (so no inverters)
	 * <br>
	 * Inputs must be 0 or 1, returns -1 if this is not the case
	 * <br>
	 * returns -2 if this is called on something that is not a valid gate type for the method
	 * (such as an inverter/ or an undefined gate etc)
	 *
	 * @param    a   The a input to the gate
	 *
	 * @param    b   The b input to the gate
	 *
	 * @return   0 If this is the output of the gate based on the inputs
	 */
	public int getOutput(int a, int b) {
		if (this.logicGateType == LogicGate.AND) {
			return andGateOutput(a,b);
		}
		else if (this.logicGateType == LogicGate.NAND) {
			return nandGateOutput(a,b);
		}
		else if (this.logicGateType == LogicGate.OR) {
			return orGateOutput(a,b);
		}
		else if (this.logicGateType == LogicGate.XOR) {
			return xorGateOutput(a,b);
		}
		else if (this.logicGateType == LogicGate.NOR) {
			return norGateOutput(a,b);
		}
		else if (this.logicGateType == LogicGate.LINE) {
			return a;//should only have one input really but a wins!
		}
		else {
			return -2;//not a valid gate type
		}
	}

	/**
	 * This will return the output from the gate if it only has one input.
	 * <br>
	 * Probably pretty useless as it doesn't take much to calculate
	 * an inverters output but included for completeness.
	 * <br>
	 * Also can be used if both inputs are the same because it will simply
	 * call the other getOutput method!
	 * <br>
	 * returns -1 if invalid input is used (inputs should only be 0 or 1)
	 *
	 * @param    a   The input to the gate (replicated if it is found to be a two input gate)
	 *
	 * @return   0 If the gate is valid and this is the output
	 */
	public int getOutput(int a) {
		if (this.logicGateType == LogicGate.NOT) {//got an inverter
			if (a == 1) {
				return 0;
			}
			else if (a == 0) {
				return 1;
			}
		}
		else if (this.logicGateType == LogicGate.LINE) {
			return a; //just propagate the signal
		}
		else {//not an inverter so must be a two input gate with the same input replicated
			return getOutput(a,a);
		}
		//if we reach here then the user has called the method with an invalid input
		return -1;
	}

	/**
	 * Returns all the two input gate types representations in a Vector
	 *
	 * @return   Vector Contains all possible two input gates
	 */
	public Vector all2InputGates() {
		Vector gateTypes = new Vector(5);

		gateTypes.addElement(new LogicGate(LogicGate.AND));
		gateTypes.addElement(new LogicGate(LogicGate.NAND));
		gateTypes.addElement(new LogicGate(LogicGate.OR));
		gateTypes.addElement(new LogicGate(LogicGate.XOR));
		gateTypes.addElement(new LogicGate(LogicGate.NOR));

		return gateTypes;
	}

	/**
	 * Gate output resolution - all return -1 if inputs aren't valid
	 * An AND gate output decider
	 *
	 * @param    a   The first input to the and gate
	 *
	 * @param    b   The second input to the and gate
	 *
	 * @return   0 If this is the output
	 */
	private int andGateOutput(int a, int b) {
		if ((a == 1) && (b == 1)) {
			return 1;
		}
		else if (((a == 1) && (b == 0)) ||
				 ((a == 0) && (b == 1)) ||
				 ((a == 0) && (b == 0))){
			return 0;
		}
		else {
			return -1;
		}
	}

	/**
	 * An NAND gate output decider
	 *
	 * @param    a   The first input to the nand gate
	 *
	 * @param    b   The second input to the nand gate
	 *
	 * @return   0 If this is the output
	 */
	private int nandGateOutput(int a, int b) {
		int result = andGateOutput(a,b);

		if(result != -1){//result is valid because input was valid
			if (result == 0) {
				return 1;//the inversion of the AND gate result
			}
			else if (result == 1) {
				return 0;//the inversion of the AND gate result
			}
			else {//should never happen but return -1 anyway
				return -1;
			}
		}
		else {//invalid input
			return -1;
		}
	}

	/**
	 * An OR gate output decider
	 *
	 * @param    a   The first input to the or gate
	 *
	 * @param    b   The second input to the or gate
	 *
	 * @return   0 If this is the output
	 */
	private int orGateOutput(int a, int b) {
		if ((a == 1) || (b == 1)) {
			return 1;
		}
		else if (a == 0 && b == 0) {
			return 0;
		}
		else {//invalid input
			return -1;
		}
	}

	/**
	 * An XOR gate output decider
	 *
	 * @param    a   The first input to the xor gate
	 *
	 * @param    b   The second input to the xor gate
	 *
	 * @return   0 If this is the output
	 */
	private int xorGateOutput(int a, int b) {
		if ((a == 0 && b == 0) ||
		    (a == 1 && b == 1)) {
			return 0;
		}
		else if ((a == 1) && (b == 0)) {
			return 1;
		}
		else if ((a == 0) && (b == 1)) {
			return 1;
		}

		else {//invalid input
			return -1;
		}
	}

	/**
	 * An NOR gate output decider
	 *
	 * @param    a   The first input to the nor gate
	 *
	 * @param    b   The second input to the nor gate
	 *
	 * @return   0 If this is the output
	 */
	private int norGateOutput(int a, int b) {
		int result = orGateOutput(a,b);
		if(result != -1){//result is valid because input was valid
			if (result == 0) {
				return 1;//the inversion of the OR gate result
			}
			else if (result == 1) {
				return 0;//the inversion of the OR gate result
			}
			else {//should never happen but return -1 anyway
				return -1;
			}
		}
		else {//invalid input
			return -1;
		}
	}

	/**
	 * Returns image with a gate drawn on it at xStart, yStart
	 * <br>
	 * if named is true then the gate will also have the gate name drawn on to it
	 * <br>
	 * connector lines are located at positions
	 * <br>
	 * yStart+8, yStart+22 on inputs and yStart+15 for output
	 * <br>
	 * (N.B. Not gate has only one input at yStart+15)
	 * <br>
	 *
	 * @param    image    The image the gate is to be drawn on to
	 *
	 * @param    xStart   The starting x position for the gate to be drawn at
	 *
	 * @param    yStart   The starting y position for the gate to be drawn at
	 *
	 * @param    named    If true the image will have the gate name painted on to it
	 *
	 * @return   Image An image with the gate drawn on it
	 */
	public Image getImageRepresentation(Image image, int xStart, int yStart, boolean named) {
		Graphics g = image.getGraphics();
		g.setColor(Color.black);

    	Color nameColor = Color.blue;//american spelling tsk!

 		if (this.logicGateType == LogicGate.AND) {
			g = drawAndGate(g, xStart, yStart);
			if (named) {//draw the name on to the gate
				g.setColor(nameColor);
				g.drawString("AND",xStart+8,yStart+19);
				g.setColor(Color.black);
			}
		}
		else if (this.logicGateType == LogicGate.NAND) {
			g = drawNandGate(g, xStart, yStart);
			if (named) {//draw the name on to the gate
				g.setColor(nameColor);
				g.drawString("NAND",xStart+5,yStart+19);
				g.setColor(Color.black);
			}
		}
		else if (this.logicGateType == LogicGate.OR) {
			g = drawOrGate(g, xStart, yStart);
			if (named) {//draw the name on to the gate
			g.setColor(nameColor);
				g.drawString("OR",xStart+7,yStart+19);
				g.setColor(Color.black);
			}
		}
		else if (this.logicGateType == LogicGate.XOR) {
			g = drawXorGate(g, xStart, yStart);
			if (named) {//draw the name on to the gate
				g.setColor(nameColor);
				g.drawString("XOR",xStart+9,yStart+19);
				g.setColor(Color.black);
			}
		}
		else if (this.logicGateType == LogicGate.NOR) {
			g = drawNorGate(g, xStart+2, yStart);
			if (named) {//draw the name on to the gate
				g.setColor(nameColor);
				g.drawString("NOR",xStart+8,yStart+19);
				g.setColor(Color.black);
			}
		}
		else if (this.logicGateType == LogicGate.NOT) {
			g = drawNotGate(g, xStart, yStart);
			if (named) {//draw the name on to the gate
				g.setColor(nameColor);
				g.drawString("NOT",xStart+2,yStart+19);
				g.setColor(Color.black);
			}
		}
		else if (this.logicGateType == LogicGate.LINE) {
			g.drawLine(0,25,75,25);//just draw a single line through the middle
			                       //no need for the "gate" to be named ever as it isn't really a gate
		}
		else {//not a proper gate so give an error message
			g.drawString("Invalid Gate Type", xStart, yStart);
			g.drawString("Gate requested is type :"+this.logicGateType,xStart,yStart+15);//will be an int of some form
		}
		return image;
	}

	//private methods for drawing gates onto a graphics context and returning it
	/**
	 * Draws an AND gate on the Graphics context
	 * (MODIFIFED SCOTTS CODE)
	 */
	private Graphics drawAndGate(Graphics g, int xStart, int yStart) {
		//draw the connector lines first so that it can be slightly out of proper alignment without worrying
		g.setColor(Color.black);
		g.drawLine(0,yStart+8,xStart,yStart+8);//input1
		g.drawLine(0,yStart+22,xStart,yStart+22);//input2
		g.drawLine(xStart+scale,yStart+15,150,yStart+15);//output, assumes images are never more than 150 pixels wide

		//draw basic white shapes
		g.setColor(Color.white);
		g.fillRect(xStart,yStart,scale,scale);
		g.fillArc(xStart+(scale/2),yStart,scale,scale,0,360);

		//draw borders
		g.setColor(Color.black);
		g.drawArc(xStart+(scale/2),yStart,scale,scale,0,90);
		g.drawArc(xStart+(scale/2),yStart,scale,scale,0,-90);
		g.drawLine(xStart,yStart,xStart+scale,yStart);
		g.drawLine(xStart,yStart,xStart,yStart+scale);
		g.drawLine(xStart,yStart+scale,xStart+scale,yStart+scale);

		return g;
	}

	/**
	 * Draws a NAND gate on the Graphics context
	 * (MODIFIFED SCOTTS CODE)
	 */
	private Graphics drawNandGate(Graphics g, int xStart, int yStart) {
		g = drawAndGate(g, xStart, yStart);
		g = drawInverterCircle(g, xStart, yStart);
		return g;
	}

	/**
	 * Draws a OR gate on the Graphics context
	 * (MODIFIFED SCOTTS CODE)
	 */
	private Graphics drawOrGate(Graphics g, int xStart, int yStart) {
		int[] xPoints =	{xStart,xStart+5,xStart+10,xStart+15,xStart+20,xStart+25,
						xStart+30,xStart+25,xStart+20,xStart+15,xStart+10,xStart+5,
						xStart,xStart+2,xStart+4,xStart+5,xStart+4,xStart+2,xStart };
		int[] yPoints = {yStart,yStart,yStart+1,yStart+3,yStart+6,yStart+10,
						yStart+15,yStart+20,yStart+24,yStart+27,yStart+29,yStart+30,yStart+30,yStart+25,
						yStart+20,yStart+15,yStart+10,yStart+5,yStart  };
		int nPoints = 19;

		g.setColor(Color.black);

		//draw the connector lines first so that it can be slightly out of proper alignment without worrying
		g.drawLine(0,yStart+8,xStart+5,yStart+8);//input1
		g.drawLine(0,yStart+22,xStart+5,yStart+22);//input2
		g.drawLine(xStart+scale,yStart+15,150,yStart+15);//output, assumes images are never more than 150 pixels wide

		//draw the actual shape of the OR gate
		g.setColor(Color.white);
		g.fillPolygon(xPoints,yPoints,nPoints);
		g.setColor(Color.black);
		g.drawPolygon(xPoints,yPoints,nPoints);

		return g;
	}

	/**
	 * Draws a XOR gate on the Graphics context
	 * (MODIFIFED SCOTTS CODE)
	 */
	private Graphics drawXorGate(Graphics g, int xStart, int yStart) {
		xStart = xStart+3;//little cheat makes this look better for the Xor gate

		g = drawOrGate(g, xStart, yStart);

		//get rid of the part of the lines not wanted
		g.setColor(Color.lightGray);
		g.drawLine(xStart-2,yStart+8,xStart+3,yStart+8);//input1
		g.drawLine(xStart-2,yStart+22,xStart+3,yStart+22);//input2

		// draw rear line
		int rearStart = xStart-scale;
		g.setColor(Color.black);
		g.drawArc(rearStart,yStart,scale,scale,0,2 * scale);
		g.drawArc(rearStart,yStart,scale,scale,0,-2 * scale);

		return g;
	}

	/**
	 * Draws a NOT gate on the Graphics context
	 * (MODIFIFED SCOTTS CODE)
	 */
	private Graphics drawNotGate(Graphics g, int xStart, int yStart) {
		int[] xPoints =	{ xStart,xStart,xStart+scale };
		int[] yPoints = { yStart,yStart+scale,yStart+(scale/2)};
		int nPoints = 3;

		//draw and fill polygon
		g.setColor(Color.white);
		g.fillPolygon(xPoints,yPoints,nPoints);
		g.setColor(Color.black);

		//draw triangle to complete the border
		g.drawLine(xStart,yStart,xStart,yStart+scale);
		g.drawLine(xStart,yStart+scale,xStart+scale,yStart+(scale/2));
		g.drawLine(xStart+scale,yStart+(scale/2),xStart,yStart);

		//draw circle at the end of the triangle
		g = drawInverterCircle(g, xStart-15, yStart);

		//draw the connector lines
		g.drawLine(0,yStart+15,xStart,yStart+15);//input
		g.drawLine(xStart+scale+10,yStart+15,150,yStart+15);//output, assumes images are never more than 150 pixels wide

		return g;
	}

	/**
	 * Draws a NOR gate on the Graphics context
	 * (MODIFIFED SCOTTS CODE)
	 */
	private Graphics drawNorGate(Graphics g, int xStart, int yStart) {
		g = drawOrGate(g, xStart, yStart);
		g = drawInverterCircle(g, xStart-15, yStart);
		return g;
	}

	/**
	 * Draws a circle at the end of a symbol at specified location
	 * (MODIFIFED SCOTTS CODE)
	 */
	private Graphics drawInverterCircle(Graphics g, int xStart, int yStart) {
		int diameter = 10;
		g.setColor(Color.white);
		g.fillOval(xStart + scale + scale/2, yStart+(scale/2)-(diameter/2),diameter,diameter);
		g.setColor(Color.black);
		g.drawOval(xStart + scale + scale/2, yStart+(scale/2)-(diameter/2),diameter,diameter);
		return g;
	}

	/**
	 * Returns a cursor that can be used when dragging a gate about the screen
	 *
	 * @param kit The toolkit to allow the cursor to be created
	 *
	 * @return A cursor representation of this LogicGate
	 */
	public Cursor getCursorRepresentation(Toolkit kit, URL appletCodeBase) {

		String name = "Default";
		Image image = null;

		try {
			String fileName = "";
			if (this.logicGateType == LogicGate.AND) {
				fileName = "andCursor.gif";
			}
			else if (this.logicGateType == LogicGate.NAND) {
				fileName = "nandCursor.gif";
			}
			else if (this.logicGateType == LogicGate.OR) {
				fileName = "orCursor.gif";
			}
			else if (this.logicGateType == LogicGate.XOR) {
				fileName = "xorCursor.gif";
			}
			else if (this.logicGateType == LogicGate.NOR) {
				fileName = "norCursor.gif";
			}
			else if (this.logicGateType == LogicGate.NOT) {
				fileName = "notCursor.gif";
			}
			else if (this.logicGateType == LogicGate.LINE) {
				fileName = "lineCursor.gif";
			}
			else {//not a proper gate so return a crosshair cursor
				return new Cursor(Cursor.CROSSHAIR_CURSOR);
			}

			URL url = new URL(appletCodeBase, fileName);
			image = kit.createImage(url);
		}
		catch(Exception lazy) {
			lazy.printStackTrace();
			return new Cursor(Cursor.CROSSHAIR_CURSOR);
		}
		return kit.createCustomCursor(image, new Point(0,0), name);
	}

	/**
	 * toString for the logic gate
	 *
	 * @return   String The type of this particular gate
	 */
	public String toString() {
		if (this.logicGateType == LogicGate.AND) {
			return "AND gate";
		}
		else if (this.logicGateType == LogicGate.NAND) {
			return "NAND gate";
		}
		else if (this.logicGateType == LogicGate.OR) {
			return "OR gate";
		}
		else if (this.logicGateType == LogicGate.XOR) {
			return "XOR gate";
		}
		else if (this.logicGateType == LogicGate.NOR) {
			return "NOR gate";
		}
		else if (this.logicGateType == LogicGate.NOT) {
			return "NOT gate";
		}
		else if (this.logicGateType == LogicGate.LINE) {
			return "LINE \"gate\"";
		}
		else {
			return "UNKNOWN LOGIC GATE TYPE!!!";
		}
	}
}