import java.lang.Object;
import java.util.Random;
import java.awt.*;

/**
 * LogicCircuits.java 1.0 03/02/01
 * <br>
 * Author       : James McMahon
 * <br>
 * Provides access to the questions that can be generated
 * <br>
 * This is where the multiple concrete instantations of the abstract Circuit class
 * <br>
 * This class provides methods to get a question from a specific level of difficulty
 * and return an object of circuit type
 * <br>
 * A question circuit which has drop targets defined is also returned in a similar manner 
 * by using the proper method.
 * <br>
 * Created      : 1/02/2003
 * <br>
 * Last Updated : 1/02/2003
 */
 
public class QuestionGenerator {
	
	//the three current settings that can be used
	/**
	 * Defines an easy circuit wanted
	 */
	public final static int EASY = 1;

	/**
	 * Defines a medium circuit wanted
	 */
	public final static int MEDIUM = 2;

	/**
	 * Defines a hard circuit wanted
	 */
	public final static int HARD = 3;	
	
	/**
	 * Random generator
	 */
	private Random randNum;

	/**
	 * The main applet that holds all of the applets components - allows the screen to be
	 * redrawn when necessary
	 */
	private LogicCircuits parent;
	
	/**
	 * Initilises the random generator so that we get a good spread of randomness
	 * <br>
	 * Otherwise, initlise just before use tends to result in similar "random" numbers being generated
	 * <br>
	 * (Due to the seed values being similar)
	 * <br>
	 * The best
	 */
	public QuestionGenerator(LogicCircuits parentContainer) {
		parent  = parentContainer;
		randNum = new Random(0);
	}

	/**
	 * Returns a randomly generated circuit depending on difficulty indicated
	 * at the moment three difficulty ratings are available EASY, MEDIUM and HARD
	 * <br>
	 * Easy retrns a random circuit from one of Circuit2GatesBottom, Cicuit2GatesTop or Circuit3Gates
	 * <br>
	 * Medium returns a random circuit from one of Circuit3LinearGates or Circuit4Gates
	 * <br>
	 * Hard returns a random circuit from Circuit5Gates
	 * <br>
	 * if difficulty is undefined then a random easy is returned
	 * 
	 * @param    difficulty   The difficulty of the circuit wanted (either EASY, MEDIUM or HARD)
	 * 
	 * @return   Circuit A concrete Circuit that corresponds to one of the availble circuits in each difficult class
	 */
	public Circuit getQuestion(int difficulty) {

		if (difficulty == EASY) {
			//Get a random int between 0,1,2
 			int circuitDiagram = randNum.nextInt(3);

 			if (circuitDiagram == 0) {
 				return new Circuit2GatesTop(new LogicGates());
 			}
 			else if (circuitDiagram == 1) {
 				return new Circuit2GatesBottom(new LogicGates());
 			}
 			else if (circuitDiagram == 2) {
 				return new Circuit3Gates(new LogicGates());
 			}
		}
		else if (difficulty == MEDIUM) {
			//returns 0 or 1;
			int circuitDiagram = randNum.nextInt(2);

			if (circuitDiagram == 0) {
				return new Circuit4Gates(new LogicGates());
			}
			else if (circuitDiagram == 1) {
				return new Circuit3LinearGates(new LogicGates());
			}
		}
		else if (difficulty == HARD) {
			return new Circuit5Gates(new LogicGates());
		}
		
		//not properly defined if we reach here so just return an random easy circuit
 		//by calling this method recursively with a proper input!
 		return getQuestion(EASY);
	}
	
	/**
	 * Returns a randomly generated circuit depending on difficulty indicated
	 * at the moment three difficulty ratings are available EASY, MEDIUM and HARD
	 * <br>
	 * Easy retrns a random circuit from one of DropCircuit2GatesBottom, DropCircuit2GatesTop, DropCircuit3Gates
	 * <br>
	 * Medium returns a random circuit from one of DropCircuit3LinearGates, DropCircuit4Gates
	 * <br>
	 * Hard returns a random circuit from DropCircuit5Gates
	 * <br>
	 * if difficulty is undefined then a random easy is returned
	 * 
	 * @param    difficulty   The difficulty of the circuit wanted (either EASY, MEDIUM or HARD)
	 * 
	 * @return   DropTargetCircuit A concrete DropTargetCircuit that corresponds to one of the availble circuits in each difficult class
	 */
	public DropTargetCircuit getDropTargetQuestion(int difficulty) {
		
		if (difficulty == EASY) {
			//Get a random int between 0,1,2
 			int circuitDiagram = randNum.nextInt(3);
	
 			if (circuitDiagram == 0) {
 				return new DropCircuit2GatesTop(new LogicGates(), this);
 			}
 			else if (circuitDiagram == 1) {
 				return new DropCircuit2GatesBottom(new LogicGates(), this);
 			}
 			else if (circuitDiagram == 2) {
 				return new DropCircuit3Gates(new LogicGates(), this);
 			}
		}
		else if (difficulty == MEDIUM) {
			//returns 0 or 1;
			int circuitDiagram = randNum.nextInt(2);

			if (circuitDiagram == 0) {
				return new DropCircuit4Gates(new LogicGates(), this);
			}
			else if (circuitDiagram == 1) {
				return new DropCircuit3LinearGates(new LogicGates(), this);
			}
		}
		else if (difficulty == HARD) {
			return new DropCircuit5Gates(new LogicGates(), this);
		}
		
		//not properly defined if we reach here so just return an random easy circuit
 		//by calling this method recursively with a proper input!
 		return getDropTargetQuestion(EASY);
	}
	
	/**
	 * Allows the main content screen to be repainted when necessary
	 */
	public void repaintScreen() {
		parent.validate();
		parent.repaint();
	}
}