import java.awt.Color;
import java.awt.Component;
import java.lang.Integer;
import java.lang.Object;
import java.util.Iterator;
import java.util.Vector;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;

/**
 * LogicCircuits.java 1.0 03/02/01
 * <br>
 * A JTable that is modified to become a TruthTable that can have coloured cells
 * <br>
 * Also modifies the cell renderers to give a better user interface
 * <br>
 * Created      : 1/02/2003
 * <br>
 * Last Updated : 1/02/2003
 * 
 * @author   James McMahon
 */
public class TruthTable extends JTable {
	
	/**
	 * Contains list of cells
	 */
	private Vector colouredCellsList = new Vector();
	
	/**
	 * An actual cell containing a row and a column number in that order
	 */
	private Vector cell;
	
	/**
	 * Determines whether cells that are marked as incorrect after a submitted answer
	 * are actually painted red or not
	 */
	private boolean cellMarkingOn = false;

	/**
	 * Creates a truthTable (JTable) based on the dataModel
	 * 
	 * @param   dataModel   A JTable table model for use in creating a JTable
	 */
	public TruthTable(TableModel dataModel) {
		super(dataModel);
	}

	/**
	 * Marks the cell at (row,col) of the table to be coloured if colouring is enabled
	 * 
	 * @param   row   The row of the cell to be coloured
	 * @param   col   The col of the cell to be coloured
	 */
	public void addToColouredCells(int row, int col) {
		cell = new Vector();
   		cell.add(new Integer(row));
   		cell.add(new Integer(col));
   		colouredCellsList.add(cell);
	}

	/**
	 * Reset all coloured cells to blank so that no cells will be coloured whether colouring is enabled ot not
	 */
	public void resetToBlank() {
		this.colouredCellsList.clear();
	}

	/**
	 * Determines if the cell at (row,col) should be coloured or not
	 * 
	 * @param    row   The row of the cell for checking 
	 * @param    col   The col of the cell for checking
	 * 
	 * @return   true If the cell should not be coloured
	 */
	public boolean isCellColoured(int row, int col) {
		if (!colouredCellsList.isEmpty()) {

   			Iterator itrFullList = colouredCellsList.iterator();


   			while (itrFullList.hasNext()) {
   				Vector cellsArray = (Vector) itrFullList.next();

   				int rowToCheck = ((Integer)cellsArray.elementAt(0)).intValue();
   				int colToCheck = ((Integer)cellsArray.elementAt(1)).intValue();

   				if ((rowToCheck==row) && (colToCheck==col)) {
   					return true;
   				}
   			}
		}
		//else
   		return false;
	}

	/**
	 * Sets whether cell painting is on or not
	 * 
	 * @param   enabled   Turns on cell colouring if true, or off if false
	 */
	public void enableCellPainting(boolean enabled) {
		cellMarkingOn = enabled;
	}

	/**
	 * returns the cell renderer for cell row, column
	 * 
	 * @param    row      The row of the cell wanted
	 * @param    column   The column of the cell wanted
	 * 
	 * @return   TableCellRenderer Returns a coloured cell renderer if the cell should be coloured
	 *           and colouring is on, otherwise it returns the imporved CellRenderer that has a better user interface
	 */
	public TableCellRenderer getCellRenderer(int row, int column) {
		if (isCellColoured(row,column) && cellMarkingOn) {
       		return new RedCellRenderer();
    	}
    	// else...
    		return new CellRenderer();
	}
}

//The following inner classes are used primarily to help with the user interface and the cell colouring
class CellRenderer extends DefaultTableCellRenderer {
	
	/**
	 * Ensures that when the user selects a cell without giving it focus (i.e. by using arrow keys)
	 * then it will still highlight the text in the cell
	 * 
	 * @return   Component The rendered component for the passed in cell
	 */
	public Component getTableCellRendererComponent(
						javax.swing.JTable truthTable,
						Object value,
						boolean isSelected,
						boolean hasFocus,
						int row,
						int col) {
			
		//this does the same thing as the list selection listener
		//Essentially it turns any cell who gets focus automatically into an editing cell
		if (hasFocus) {

		truthTable.editCellAt(row,col);

     	int col1 = truthTable.getSelectedColumn();
     	int row1 = truthTable.getSelectedRow();

    		if (row1 >= 0) {
   	 			Component comp = truthTable.getCellEditor(row1,col1).getTableCellEditorComponent(truthTable, truthTable.getValueAt(row1,col1), true,row1,col1);
    			((javax.swing.JTextField)comp).select(0, 1);
    		}
		}

		super.getTableCellRendererComponent(truthTable,value,isSelected,hasFocus,row,col);
		return this;
	}
}

//The red coloured cell renderer
class RedCellRenderer extends CellRenderer {
	
	/**
	 * Turns the cell background red
	 * 
	 * @return   Component The rendered component for the passed in cell
	 */
	public Component getTableCellRendererComponent(
							javax.swing.JTable truthTable,
							Object value,
							boolean isSelected,
							boolean hasFocus,
							int row,
							int col) {
		
		setBackground(Color.red);//simply want this to be red if it is set
		super.getTableCellRendererComponent(truthTable,value,isSelected,hasFocus,row,col);
		return this;
	}
}

class CellListener implements ListSelectionListener {
	private TruthTable truthTable;

	/**
	 * This is needed so that the selected cell can have it's text highlighted
	 * 
	 * @param   table   The table that is to be listened to
	 */
	public CellListener(TruthTable table) {
		truthTable = table;
	}

	/**
	 * Ensures that whenever a user clicks on a cell and gives it focus then the text inside is automatically highlighted
	 * 
	 * @param_stray   ListSelectionEvent the event generated by user's clibking on a cell
	 */
	public void valueChanged(ListSelectionEvent e) {
		ListSelectionModel lsm = (ListSelectionModel)e.getSource();
     	int col = truthTable.getSelectedColumn();
     	int row = truthTable.getSelectedRow();

    	if (row >= 0) {
   	 		Component comp = truthTable.getCellEditor(row,col).getTableCellEditorComponent(truthTable, truthTable.getValueAt(row,col), true,row,col);
    		((javax.swing.JTextField)comp).select(0, 1);
    	}
	}
}