import java.awt.Component;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.lang.NullPointerException;
import java.lang.Object;
import java.lang.String;
import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionListener;

/**
 * LogicCircuits.java 1.0 03/02/01
 * Author       : James McMahon
 * <br>
 * TruthTable panel that has the customised truth table
 * <br>
 * Created      : 1/02/2003
 * <br>
 * Last Updated : 1/02/2003
 */
public class TruthTablePanel extends JPanel {
	
	/**
	 * The truth tabel that is customised using various models
	 */
	private TruthTable truthTable;
	
	/**
	 * A scroll pane for displaying the table on
	 */
	private JScrollPane scroller;
	
	/**
	 * Column names for the table (defaulted to 4 columns)
	 */
	private String[] columnNames = {"X","Y","Z","Out"};
	
	/**
	 * Data for the table (defaulted to the basic representation)
	 */
	private Object[][] data = {{"0","0","0","?"},
							   {"0","0","1","?"},
							   {"0","1","0","?"},
							   {"0","1","1","?"},
							   {"1","0","0","?"},
							   {"1","0","1","?"},
							   {"1","1","0","?"},
							   {"1","1","1","?"}};
	
	/**
	 * The original data that needs to be stored so it that it can be
	 * used to reset the screen to the original values
	 */
	private Object[][] originalData;
	
	/**
	 * The number of fixed columns - defaulted to 3
	 */
	private int fixedColumns = 3;

	/**
	 * Should set up a default table with it's own listener
	 * and a 3 input 1 output (4 columns), with 8 rows table
	 * use setUpTruthTable to customise the actual table you want
	 */
	public TruthTablePanel() {
		this.setPreferredSize(new Dimension(200,200));
		this.setBorder(BorderFactory.createTitledBorder("Try to complete the following Truth Table..."));

	    originalData = data;//set the original's to the default data as well
		setUpTruthTable(columnNames, data, fixedColumns);
	}
	
	/**
	 * allows the title in the border to be set dynamically
	 */
	public void setBorderTitle(String txt) { 
		this.setBorder(BorderFactory.createTitledBorder(txt));
	}
	
	/**
	 * Sets the truth table displayed to be the one that can be created
	 * via titles as the titles of the columns and details as the data to be held
	 * in the rows ...
	 * <br>
	 * N.B. only 1, 0 , ? are allowed in the JTable (TruthTable)
	 * The number of fixed columns represents the number of non-user modifiable
	 * columns counting from left to right
	 */
	public void setUpTruthTable(String[] titles, Object[][] details, int numberOfFixedColumns) {
		columnNames=titles;
		data = details;
		originalData = details;
		fixedColumns = numberOfFixedColumns;

		TruthTableModel tableModel = new TruthTableModel(columnNames, data, fixedColumns);

		truthTable = new TruthTable(tableModel);
		truthTable.resetToBlank();//blanks any coloured cells to the default

		truthTable.setRowSelectionAllowed(true);
		truthTable.setColumnSelectionAllowed(true);

		truthTable.getTableHeader().setReorderingAllowed(false);//does the same job as the columnmodel listener used to do
		truthTable.getTableHeader().setResizingAllowed(false);

		ListSelectionListener cellListener = new CellListener(truthTable);
		truthTable.getSelectionModel().addListSelectionListener(cellListener);
		truthTable.getColumnModel().getSelectionModel().addListSelectionListener(cellListener);

		//Make a single click to be the point that cell editing begins
		DefaultCellEditor dce = new DefaultCellEditor(new JTextField());

		for(int i=0; i<truthTable.getColumnCount(); i++) {
			truthTable.getColumnModel().getColumn(i).setCellEditor(dce);
			Component comp = dce.getComponent();
			dce.setClickCountToStart(0);
		}

		displayTruthTable();
	}

	/**
	 * Displays the truth table that is curently available
	 */
	private void displayTruthTable() {
		removePreviousTable();
		//add the new truthtable to the scroll pane in case
		//it is too big to fit all on screen at once
		scroller = new JScrollPane(truthTable);
		scroller.setPreferredSize(new Dimension(590,150));

		this.add(scroller);
	}

	/**
	 * Removes from display any truthtbale currently on the panel
	 */
	private void removePreviousTable() {
		try {
			this.remove(scroller);
			scroller.remove(truthTable);
		}
		catch(NullPointerException npe){
			//didn't need to remove the scroller or truth table so job is already done
			//nothing more to do then
		}
	}

	/**
	 * Removes focus from any currently editing cells
	 * used primarily in getUserAnswer to ensure that the user doesn't input an answer
	 * and forget to manually remove focus, thereby causing an incorrect value
	 * in the cell to be evaluated
	 */
	private void removeEditingFocus() {
		int row = truthTable.getSelectedRow();
		int col = truthTable.getSelectedColumn();
		if (truthTable.isEditing()) {
			truthTable.getCellEditor(row,col).stopCellEditing();
		}
	}

	/**
	 * returns the answers that the user has input into the table
	 * what is actually returned is the full data contained in the
	 * table at the point of calling
	 * <br>
	 * It also ensures that if the user is currently editing a cell at the point
	 * of this being called then that cell editing is stopped and the partially entered data is
	 * accepted
	 */
	public Object[][] getUserAnswer() {
		removeEditingFocus();
		return data;
	}

	/**
	 * Clear all of the coloured cells (doesn't stop painting though)
	 */
	public void blankColouredCells() {
		truthTable.resetToBlank();
	}

	/**
	 * allow coloured cells to be painted
	 * 
	 * @param   enabled   If true then coloured cells are painted otherwise they are ignored
	 */
	public void enableIncorrectCellMarking(boolean enabled) {
		truthTable.enableCellPainting(enabled);
	}

	/**
	 * Mark cell at row, col as needing to be coloured
	 * 
	 * @param   row   The row of the cell to be coloured
	 * @param   col   The col of the cell to be coloured
	 */
	public void colourCellAt(int row, int col) {
		truthTable.addToColouredCells(row,col);
	}
}